/* Generated by Cython 0.29.21 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "pypivoter",
        "sources": [
            "src/pypivoter.pyx"
        ]
    },
    "module_name": "pypivoter"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_21"
#define CYTHON_HEX_VERSION 0x001D15F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pypivoter
#define __PYX_HAVE_API__pypivoter
/* Early includes */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/pypivoter.pyx",
};

/*--- Type declarations ---*/
struct __pyx_t_9pypivoter_s_Link;
struct __pyx_t_9pypivoter_s_LinkedList;
struct __pyx_t_9pypivoter_s_NeighborList;
struct __pyx_t_9pypivoter_s_NeighborListArray;

/* "pypivoter.pyx":23
 * 
 * # Stores data and pointers to next and previous links.
 * cdef struct s_Link:             # <<<<<<<<<<<<<<
 *     int data # arbitrary data stored in the link
 *     s_Link* next # the previous link in the chain
 */
struct __pyx_t_9pypivoter_s_Link {
  int data;
  struct __pyx_t_9pypivoter_s_Link *next;
  struct __pyx_t_9pypivoter_s_Link *prev;
};

/* "pypivoter.pyx":29
 * 
 * # Stores a linked list, with sentinel links for head and tail.
 * cdef struct s_LinkedList:             # <<<<<<<<<<<<<<
 *     s_Link* head # head of the linked list, a dummy sentinel
 *     s_Link* tail # tail of the linked list, a dummy sentinel
 */
struct __pyx_t_9pypivoter_s_LinkedList {
  struct __pyx_t_9pypivoter_s_Link *head;
  struct __pyx_t_9pypivoter_s_Link *tail;
};

/* "pypivoter.pyx":33
 *     s_Link* tail # tail of the linked list, a dummy sentinel
 * 
 * ctypedef s_Link Link             # <<<<<<<<<<<<<<
 * ctypedef s_LinkedList LinkedList
 * 
 */
typedef struct __pyx_t_9pypivoter_s_Link __pyx_t_9pypivoter_Link;

/* "pypivoter.pyx":34
 * 
 * ctypedef s_Link Link
 * ctypedef s_LinkedList LinkedList             # <<<<<<<<<<<<<<
 * 
 * ### The next several functions operate on Link structures.
 */
typedef struct __pyx_t_9pypivoter_s_LinkedList __pyx_t_9pypivoter_LinkedList;

/* "pypivoter.pyx":400
 * # For a given ordering, this struct stores later and earlier neighbors for a
 * # given vertex in linked lists
 * cdef struct s_NeighborList:             # <<<<<<<<<<<<<<
 *     int vertex # the vertex that owns this neighbor list
 *     LinkedList* earlier # a list of neighbors that are ordered before the vertex
 */
struct __pyx_t_9pypivoter_s_NeighborList {
  int vertex;
  __pyx_t_9pypivoter_LinkedList *earlier;
  __pyx_t_9pypivoter_LinkedList *later;
  int orderNumber;
};

/* "pypivoter.pyx":408
 * # For a given ordering, this struct stores later and earlier neighbors for a
 * # given vertex in arrays
 * cdef struct s_NeighborListArray:             # <<<<<<<<<<<<<<
 *     int vertex # the vertex that owns this neighbor list
 *     int* earlier # an array of neighbors that are ordered before the vertex
 */
struct __pyx_t_9pypivoter_s_NeighborListArray {
  int vertex;
  int *earlier;
  int earlierDegree;
  int *later;
  int laterDegree;
  int orderNumber;
};

/* "pypivoter.pyx":416
 *     int orderNumber # the position of the vertex in the ordering
 * 
 * ctypedef s_NeighborList NeighborList             # <<<<<<<<<<<<<<
 * ctypedef s_NeighborListArray NeighborListArray
 * 
 */
typedef struct __pyx_t_9pypivoter_s_NeighborList __pyx_t_9pypivoter_NeighborList;

/* "pypivoter.pyx":417
 * 
 * ctypedef s_NeighborList NeighborList
 * ctypedef s_NeighborListArray NeighborListArray             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef struct __pyx_t_9pypivoter_s_NeighborListArray __pyx_t_9pypivoter_NeighborListArray;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* None.proto */
static CYTHON_INLINE int __Pyx_div_int(int, int);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_NeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'pypivoter' */
static int __pyx_f_9pypivoter_isTail(__pyx_t_9pypivoter_Link *); /*proto*/
static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addAfter(__pyx_t_9pypivoter_Link *, int); /*proto*/
static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addBefore(__pyx_t_9pypivoter_Link *, int); /*proto*/
static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_removeLink(__pyx_t_9pypivoter_Link *); /*proto*/
static int __pyx_f_9pypivoter_deleteLink(__pyx_t_9pypivoter_Link *); /*proto*/
static __pyx_t_9pypivoter_LinkedList *__pyx_f_9pypivoter_createLinkedList(void); /*proto*/
static void __pyx_f_9pypivoter_destroyLinkedList(__pyx_t_9pypivoter_LinkedList *); /*proto*/
static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addFirst(__pyx_t_9pypivoter_LinkedList *, int); /*proto*/
static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addLast(__pyx_t_9pypivoter_LinkedList *, int); /*proto*/
static int __pyx_f_9pypivoter_isEmpty(__pyx_t_9pypivoter_LinkedList *); /*proto*/
static int __pyx_f_9pypivoter_getFirst(__pyx_t_9pypivoter_LinkedList *); /*proto*/
static int __pyx_f_9pypivoter_length(__pyx_t_9pypivoter_LinkedList *); /*proto*/
static __pyx_t_9pypivoter_NeighborListArray **__pyx_f_9pypivoter_computeDegeneracyOrderArray(__pyx_t_9pypivoter_LinkedList **, int); /*proto*/
static int __pyx_f_9pypivoter_ipow(int, int); /*proto*/
static __pyx_t_9pypivoter_LinkedList **__pyx_f_9pypivoter_graphAdjArrayToDoubleEdges(int, int, int **); /*proto*/
static void __pyx_f_9pypivoter_fillInPandXForRecursiveCall(int, int, int *, int *, __pyx_t_9pypivoter_NeighborListArray **, int **, int *, int, int *, int *); /*proto*/
static void __pyx_f_9pypivoter_moveToR(int, int *, int *, int **, int *, int *, int *, int *, int *); /*proto*/
static void __pyx_f_9pypivoter_moveFromRToX(int, int *, int *, int *, int *); /*proto*/
static int __pyx_f_9pypivoter_findBestPivotNonNeighbors(int **, int *, int *, int *, int **, int *, int, int); /*proto*/
static void __pyx_f_9pypivoter_listAllCliquesDegeneracyRecursive(int **, int *, int *, int *, int **, int *, int *, int *, int, int, int, int, int, int, int *, int); /*proto*/
static void __pyx_f_9pypivoter_listAllCliquesDegeneracy(int **, int *, __pyx_t_9pypivoter_NeighborListArray **, int, int, int); /*proto*/
static int *__pyx_f_9pypivoter_countCliques_c(__pyx_t_9pypivoter_LinkedList **, int, int *); /*proto*/
static int **__pyx_f_9pypivoter_enumerateCliques_c(__pyx_t_9pypivoter_LinkedList **, int, int *, int *); /*proto*/
static PyObject *__pyx_f_9pypivoter_countCliques(PyObject *, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_9pypivoter_enumerateCliques(PyObject *, int, int __pyx_skip_dispatch); /*proto*/
#define __Pyx_MODULE_NAME "pypivoter"
extern int __pyx_module_is_main_pypivoter;
int __pyx_module_is_main_pypivoter = 0;

/* Implementation of 'pypivoter' */
static PyObject *__pyx_builtin_range;
static const char __pyx_k_i[] = "i";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_os[] = "os";
static const char __pyx_k_adj[] = "adj";
static const char __pyx_k_max[] = "max";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_max_k[] = "max_k";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_astype[] = "astype";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_trim_zeros[] = "trim_zeros";
static const char __pyx_k_logical_and[] = "logical_and";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static PyObject *__pyx_n_s_adj;
static PyObject *__pyx_n_s_astype;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_u_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_logical_and;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max;
static PyObject *__pyx_n_s_max_k;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_os;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_trim_zeros;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_9pypivoter_countCliques(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_adj, int __pyx_v_max_k); /* proto */
static PyObject *__pyx_pf_9pypivoter_2enumerateCliques(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_adj, int __pyx_v_max_k); /* proto */
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_slice_;
/* Late includes */

/* "pypivoter.pyx":38
 * ### The next several functions operate on Link structures.
 * 
 * cdef int isHead(Link* list):             # <<<<<<<<<<<<<<
 *     """Determine if a Link is the head sentinel.
 * 
 */

static int __pyx_f_9pypivoter_isHead(__pyx_t_9pypivoter_Link *__pyx_v_list) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isHead", 0);

  /* "pypivoter.pyx":51
 *         True if the link is the head sentinel, false otherwise.
 *     """
 *     assert list != NULL             # <<<<<<<<<<<<<<
 *     return list.prev == NULL
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 51, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":52
 *     """
 *     assert list != NULL
 *     return list.prev == NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_list->prev == NULL);
  goto __pyx_L0;

  /* "pypivoter.pyx":38
 * ### The next several functions operate on Link structures.
 * 
 * cdef int isHead(Link* list):             # <<<<<<<<<<<<<<
 *     """Determine if a Link is the head sentinel.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.isHead", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":55
 * 
 * 
 * cdef int isTail(Link* list):             # <<<<<<<<<<<<<<
 *     """Determine if a Link is the tail sentinel.
 * 
 */

static int __pyx_f_9pypivoter_isTail(__pyx_t_9pypivoter_Link *__pyx_v_list) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isTail", 0);

  /* "pypivoter.pyx":68
 *         True if the link is the tail sentinel, false otherwise.
 *     """
 *     assert list != NULL             # <<<<<<<<<<<<<<
 *     return list.next == NULL
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 68, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":69
 *     """
 *     assert list != NULL
 *     return list.next == NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_list->next == NULL);
  goto __pyx_L0;

  /* "pypivoter.pyx":55
 * 
 * 
 * cdef int isTail(Link* list):             # <<<<<<<<<<<<<<
 *     """Determine if a Link is the tail sentinel.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.isTail", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":72
 * 
 * 
 * cdef Link* addAfter(Link* list, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add a link after a given link.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addAfter(__pyx_t_9pypivoter_Link *__pyx_v_list, int __pyx_v_data) {
  __pyx_t_9pypivoter_Link *__pyx_v_newLink;
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("addAfter", 0);

  /* "pypivoter.pyx":87
 *         A pointer to the Link that was added after link.
 *     """
 *     assert list != NULL             # <<<<<<<<<<<<<<
 *     assert list.next != NULL
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 87, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":88
 *     """
 *     assert list != NULL
 *     assert list.next != NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef Link* newLink = <Link*> malloc(sizeof(Link))
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list->next != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 88, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":90
 *     assert list.next != NULL
 * 
 *     cdef Link* newLink = <Link*> malloc(sizeof(Link))             # <<<<<<<<<<<<<<
 * 
 *     newLink.data = data
 */
  __pyx_v_newLink = ((__pyx_t_9pypivoter_Link *)malloc((sizeof(__pyx_t_9pypivoter_Link))));

  /* "pypivoter.pyx":92
 *     cdef Link* newLink = <Link*> malloc(sizeof(Link))
 * 
 *     newLink.data = data             # <<<<<<<<<<<<<<
 * 
 *     newLink.next = list.next
 */
  __pyx_v_newLink->data = __pyx_v_data;

  /* "pypivoter.pyx":94
 *     newLink.data = data
 * 
 *     newLink.next = list.next             # <<<<<<<<<<<<<<
 *     newLink.prev = list
 * 
 */
  __pyx_t_1 = __pyx_v_list->next;
  __pyx_v_newLink->next = __pyx_t_1;

  /* "pypivoter.pyx":95
 * 
 *     newLink.next = list.next
 *     newLink.prev = list             # <<<<<<<<<<<<<<
 * 
 *     list.next.prev = newLink
 */
  __pyx_v_newLink->prev = __pyx_v_list;

  /* "pypivoter.pyx":97
 *     newLink.prev = list
 * 
 *     list.next.prev = newLink             # <<<<<<<<<<<<<<
 *     list.next = newLink
 * 
 */
  __pyx_v_list->next->prev = __pyx_v_newLink;

  /* "pypivoter.pyx":98
 * 
 *     list.next.prev = newLink
 *     list.next = newLink             # <<<<<<<<<<<<<<
 * 
 *     return newLink
 */
  __pyx_v_list->next = __pyx_v_newLink;

  /* "pypivoter.pyx":100
 *     list.next = newLink
 * 
 *     return newLink             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_newLink;
  goto __pyx_L0;

  /* "pypivoter.pyx":72
 * 
 * 
 * cdef Link* addAfter(Link* list, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add a link after a given link.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.addAfter", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":103
 * 
 * 
 * cdef Link* addBefore(Link* list, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add a link before a given link.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addBefore(__pyx_t_9pypivoter_Link *__pyx_v_list, int __pyx_v_data) {
  __pyx_t_9pypivoter_Link *__pyx_v_newLink;
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("addBefore", 0);

  /* "pypivoter.pyx":118
 *         A pointer to the Link that was added before link.
 *     """
 *     assert list != NULL             # <<<<<<<<<<<<<<
 *     assert list.prev != NULL
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 118, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":119
 *     """
 *     assert list != NULL
 *     assert list.prev != NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef Link* newLink = <Link*> malloc(sizeof(Link))
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list->prev != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 119, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":121
 *     assert list.prev != NULL
 * 
 *     cdef Link* newLink = <Link*> malloc(sizeof(Link))             # <<<<<<<<<<<<<<
 * 
 *     newLink.data = data
 */
  __pyx_v_newLink = ((__pyx_t_9pypivoter_Link *)malloc((sizeof(__pyx_t_9pypivoter_Link))));

  /* "pypivoter.pyx":123
 *     cdef Link* newLink = <Link*> malloc(sizeof(Link))
 * 
 *     newLink.data = data             # <<<<<<<<<<<<<<
 * 
 *     newLink.next = list
 */
  __pyx_v_newLink->data = __pyx_v_data;

  /* "pypivoter.pyx":125
 *     newLink.data = data
 * 
 *     newLink.next = list             # <<<<<<<<<<<<<<
 *     newLink.prev = list.prev
 * 
 */
  __pyx_v_newLink->next = __pyx_v_list;

  /* "pypivoter.pyx":126
 * 
 *     newLink.next = list
 *     newLink.prev = list.prev             # <<<<<<<<<<<<<<
 * 
 *     list.prev.next = newLink
 */
  __pyx_t_1 = __pyx_v_list->prev;
  __pyx_v_newLink->prev = __pyx_t_1;

  /* "pypivoter.pyx":128
 *     newLink.prev = list.prev
 * 
 *     list.prev.next = newLink             # <<<<<<<<<<<<<<
 *     list.prev = newLink
 * 
 */
  __pyx_v_list->prev->next = __pyx_v_newLink;

  /* "pypivoter.pyx":129
 * 
 *     list.prev.next = newLink
 *     list.prev = newLink             # <<<<<<<<<<<<<<
 * 
 *     return newLink
 */
  __pyx_v_list->prev = __pyx_v_newLink;

  /* "pypivoter.pyx":131
 *     list.prev = newLink
 * 
 *     return newLink             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_newLink;
  goto __pyx_L0;

  /* "pypivoter.pyx":103
 * 
 * 
 * cdef Link* addBefore(Link* list, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add a link before a given link.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.addBefore", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":134
 * 
 * 
 * cdef Link* removeLink(Link* list):             # <<<<<<<<<<<<<<
 *     """A location-aware method to remove a link and return it.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_removeLink(__pyx_t_9pypivoter_Link *__pyx_v_list) {
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeLink", 0);

  /* "pypivoter.pyx":147
 *         The removed link.
 *     """
 *     assert list != NULL             # <<<<<<<<<<<<<<
 *     assert list.next != NULL
 *     assert list.prev != NULL
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 147, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":148
 *     """
 *     assert list != NULL
 *     assert list.next != NULL             # <<<<<<<<<<<<<<
 *     assert list.prev != NULL
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list->next != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 148, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":149
 *     assert list != NULL
 *     assert list.next != NULL
 *     assert list.prev != NULL             # <<<<<<<<<<<<<<
 * 
 *     list.next.prev = list.prev
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list->prev != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 149, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":151
 *     assert list.prev != NULL
 * 
 *     list.next.prev = list.prev             # <<<<<<<<<<<<<<
 *     list.prev.next = list.next
 * 
 */
  __pyx_t_1 = __pyx_v_list->prev;
  __pyx_v_list->next->prev = __pyx_t_1;

  /* "pypivoter.pyx":152
 * 
 *     list.next.prev = list.prev
 *     list.prev.next = list.next             # <<<<<<<<<<<<<<
 * 
 *     list.next = NULL
 */
  __pyx_t_1 = __pyx_v_list->next;
  __pyx_v_list->prev->next = __pyx_t_1;

  /* "pypivoter.pyx":154
 *     list.prev.next = list.next
 * 
 *     list.next = NULL             # <<<<<<<<<<<<<<
 *     list.prev = NULL
 * 
 */
  __pyx_v_list->next = NULL;

  /* "pypivoter.pyx":155
 * 
 *     list.next = NULL
 *     list.prev = NULL             # <<<<<<<<<<<<<<
 * 
 *     return list
 */
  __pyx_v_list->prev = NULL;

  /* "pypivoter.pyx":157
 *     list.prev = NULL
 * 
 *     return list             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_list;
  goto __pyx_L0;

  /* "pypivoter.pyx":134
 * 
 * 
 * cdef Link* removeLink(Link* list):             # <<<<<<<<<<<<<<
 *     """A location-aware method to remove a link and return it.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.removeLink", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":160
 * 
 * 
 * cdef int deleteLink(Link* list):             # <<<<<<<<<<<<<<
 *     """Delete the given link and return its data.
 * 
 */

static int __pyx_f_9pypivoter_deleteLink(__pyx_t_9pypivoter_Link *__pyx_v_list) {
  int __pyx_v_data;
  __pyx_t_9pypivoter_Link *__pyx_v_linkToFree;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deleteLink", 0);

  /* "pypivoter.pyx":173
 *         The data that was in the link (must be freed if previously allocated.)
 *     """
 *     assert list != NULL             # <<<<<<<<<<<<<<
 *     assert list.next != NULL
 *     assert list.prev != NULL
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 173, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":174
 *     """
 *     assert list != NULL
 *     assert list.next != NULL             # <<<<<<<<<<<<<<
 *     assert list.prev != NULL
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list->next != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 174, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":175
 *     assert list != NULL
 *     assert list.next != NULL
 *     assert list.prev != NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef int data = list.data
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_list->prev != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 175, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":177
 *     assert list.prev != NULL
 * 
 *     cdef int data = list.data             # <<<<<<<<<<<<<<
 *     cdef Link* linkToFree = removeLink(list)
 * 
 */
  __pyx_t_1 = __pyx_v_list->data;
  __pyx_v_data = __pyx_t_1;

  /* "pypivoter.pyx":178
 * 
 *     cdef int data = list.data
 *     cdef Link* linkToFree = removeLink(list)             # <<<<<<<<<<<<<<
 * 
 *     free(linkToFree)
 */
  __pyx_v_linkToFree = __pyx_f_9pypivoter_removeLink(__pyx_v_list);

  /* "pypivoter.pyx":180
 *     cdef Link* linkToFree = removeLink(list)
 * 
 *     free(linkToFree)             # <<<<<<<<<<<<<<
 * 
 *     return data
 */
  free(__pyx_v_linkToFree);

  /* "pypivoter.pyx":182
 *     free(linkToFree)
 * 
 *     return data             # <<<<<<<<<<<<<<
 * 
 * ### The next several functions operate on LinkedList structures.
 */
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "pypivoter.pyx":160
 * 
 * 
 * cdef int deleteLink(Link* list):             # <<<<<<<<<<<<<<
 *     """Delete the given link and return its data.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.deleteLink", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":186
 * ### The next several functions operate on LinkedList structures.
 * 
 * cdef LinkedList* createLinkedList():             # <<<<<<<<<<<<<<
 *     """Create a new empty linked list.
 * 
 */

static __pyx_t_9pypivoter_LinkedList *__pyx_f_9pypivoter_createLinkedList(void) {
  __pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList;
  __pyx_t_9pypivoter_LinkedList *__pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_1;
  __Pyx_RefNannySetupContext("createLinkedList", 0);

  /* "pypivoter.pyx":194
 *         The created linked list.
 *     """
 *     cdef LinkedList* linkedList = <LinkedList*> malloc(sizeof(LinkedList))             # <<<<<<<<<<<<<<
 * 
 *     linkedList.head = <Link *> malloc(sizeof(Link))
 */
  __pyx_v_linkedList = ((__pyx_t_9pypivoter_LinkedList *)malloc((sizeof(__pyx_t_9pypivoter_LinkedList))));

  /* "pypivoter.pyx":196
 *     cdef LinkedList* linkedList = <LinkedList*> malloc(sizeof(LinkedList))
 * 
 *     linkedList.head = <Link *> malloc(sizeof(Link))             # <<<<<<<<<<<<<<
 *     linkedList.tail = <Link *> malloc(sizeof(Link))
 * 
 */
  __pyx_v_linkedList->head = ((__pyx_t_9pypivoter_Link *)malloc((sizeof(__pyx_t_9pypivoter_Link))));

  /* "pypivoter.pyx":197
 * 
 *     linkedList.head = <Link *> malloc(sizeof(Link))
 *     linkedList.tail = <Link *> malloc(sizeof(Link))             # <<<<<<<<<<<<<<
 * 
 *     linkedList.head.prev = NULL
 */
  __pyx_v_linkedList->tail = ((__pyx_t_9pypivoter_Link *)malloc((sizeof(__pyx_t_9pypivoter_Link))));

  /* "pypivoter.pyx":199
 *     linkedList.tail = <Link *> malloc(sizeof(Link))
 * 
 *     linkedList.head.prev = NULL             # <<<<<<<<<<<<<<
 *     linkedList.head.next = linkedList.tail
 *     linkedList.head.data = <int> 0xDEAD0000
 */
  __pyx_v_linkedList->head->prev = NULL;

  /* "pypivoter.pyx":200
 * 
 *     linkedList.head.prev = NULL
 *     linkedList.head.next = linkedList.tail             # <<<<<<<<<<<<<<
 *     linkedList.head.data = <int> 0xDEAD0000
 * 
 */
  __pyx_t_1 = __pyx_v_linkedList->tail;
  __pyx_v_linkedList->head->next = __pyx_t_1;

  /* "pypivoter.pyx":201
 *     linkedList.head.prev = NULL
 *     linkedList.head.next = linkedList.tail
 *     linkedList.head.data = <int> 0xDEAD0000             # <<<<<<<<<<<<<<
 * 
 *     linkedList.tail.prev = linkedList.head
 */
  __pyx_v_linkedList->head->data = ((int)0xDEAD0000);

  /* "pypivoter.pyx":203
 *     linkedList.head.data = <int> 0xDEAD0000
 * 
 *     linkedList.tail.prev = linkedList.head             # <<<<<<<<<<<<<<
 *     linkedList.tail.next = NULL
 *     linkedList.tail.data = <int> 0xDEADFFFF
 */
  __pyx_t_1 = __pyx_v_linkedList->head;
  __pyx_v_linkedList->tail->prev = __pyx_t_1;

  /* "pypivoter.pyx":204
 * 
 *     linkedList.tail.prev = linkedList.head
 *     linkedList.tail.next = NULL             # <<<<<<<<<<<<<<
 *     linkedList.tail.data = <int> 0xDEADFFFF
 * 
 */
  __pyx_v_linkedList->tail->next = NULL;

  /* "pypivoter.pyx":205
 *     linkedList.tail.prev = linkedList.head
 *     linkedList.tail.next = NULL
 *     linkedList.tail.data = <int> 0xDEADFFFF             # <<<<<<<<<<<<<<
 * 
 *     return linkedList
 */
  __pyx_v_linkedList->tail->data = ((int)0xDEADFFFF);

  /* "pypivoter.pyx":207
 *     linkedList.tail.data = <int> 0xDEADFFFF
 * 
 *     return linkedList             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_linkedList;
  goto __pyx_L0;

  /* "pypivoter.pyx":186
 * ### The next several functions operate on LinkedList structures.
 * 
 * cdef LinkedList* createLinkedList():             # <<<<<<<<<<<<<<
 *     """Create a new empty linked list.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":210
 * 
 * 
 * cdef void destroyLinkedList(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Destroy a linked list.
 * 
 */

static void __pyx_f_9pypivoter_destroyLinkedList(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  __pyx_t_9pypivoter_Link *__pyx_v_curr;
  __pyx_t_9pypivoter_Link *__pyx_v_currNext;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("destroyLinkedList", 0);

  /* "pypivoter.pyx":218
 *         The linked list to destroy.
 *     """
 *     cdef Link* curr = linkedList.head             # <<<<<<<<<<<<<<
 *     cdef Link* currNext
 * 
 */
  __pyx_t_1 = __pyx_v_linkedList->head;
  __pyx_v_curr = __pyx_t_1;

  /* "pypivoter.pyx":221
 *     cdef Link* currNext
 * 
 *     while curr != NULL:             # <<<<<<<<<<<<<<
 *         currNext = curr.next
 *         free(curr)
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_curr != NULL) != 0);
    if (!__pyx_t_2) break;

    /* "pypivoter.pyx":222
 * 
 *     while curr != NULL:
 *         currNext = curr.next             # <<<<<<<<<<<<<<
 *         free(curr)
 *         curr = currNext
 */
    __pyx_t_1 = __pyx_v_curr->next;
    __pyx_v_currNext = __pyx_t_1;

    /* "pypivoter.pyx":223
 *     while curr != NULL:
 *         currNext = curr.next
 *         free(curr)             # <<<<<<<<<<<<<<
 *         curr = currNext
 * 
 */
    free(__pyx_v_curr);

    /* "pypivoter.pyx":224
 *         currNext = curr.next
 *         free(curr)
 *         curr = currNext             # <<<<<<<<<<<<<<
 * 
 *     free(linkedList)
 */
    __pyx_v_curr = __pyx_v_currNext;
  }

  /* "pypivoter.pyx":226
 *         curr = currNext
 * 
 *     free(linkedList)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_linkedList);

  /* "pypivoter.pyx":210
 * 
 * 
 * cdef void destroyLinkedList(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Destroy a linked list.
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":229
 * 
 * 
 * cdef void copyLinkedList(LinkedList* destination, LinkedList* source):             # <<<<<<<<<<<<<<
 *     """Copy a linked list.
 * 
 */

static void __pyx_f_9pypivoter_copyLinkedList(__pyx_t_9pypivoter_LinkedList *__pyx_v_destination, __pyx_t_9pypivoter_LinkedList *__pyx_v_source) {
  __pyx_t_9pypivoter_Link *__pyx_v_curr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copyLinkedList", 0);

  /* "pypivoter.pyx":239
 *         Linked list to be copied.
 *     """
 *     assert destination != NULL and source != NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef Link* curr = source.head.next
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = ((__pyx_v_destination != NULL) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_source != NULL) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L3_bool_binop_done:;
    if (unlikely(!__pyx_t_1)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 239, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":241
 *     assert destination != NULL and source != NULL
 * 
 *     cdef Link* curr = source.head.next             # <<<<<<<<<<<<<<
 * 
 *     while not isTail(curr):
 */
  __pyx_t_3 = __pyx_v_source->head->next;
  __pyx_v_curr = __pyx_t_3;

  /* "pypivoter.pyx":243
 *     cdef Link* curr = source.head.next
 * 
 *     while not isTail(curr):             # <<<<<<<<<<<<<<
 *         addLast(destination, curr.data)
 *         curr = curr.next
 */
  while (1) {
    __pyx_t_1 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_curr) != 0)) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":244
 * 
 *     while not isTail(curr):
 *         addLast(destination, curr.data)             # <<<<<<<<<<<<<<
 *         curr = curr.next
 * 
 */
    (void)(__pyx_f_9pypivoter_addLast(__pyx_v_destination, __pyx_v_curr->data));

    /* "pypivoter.pyx":245
 *     while not isTail(curr):
 *         addLast(destination, curr.data)
 *         curr = curr.next             # <<<<<<<<<<<<<<
 * 
 * cdef Link* addFirst(LinkedList* linkedList, int data):
 */
    __pyx_t_3 = __pyx_v_curr->next;
    __pyx_v_curr = __pyx_t_3;
  }

  /* "pypivoter.pyx":229
 * 
 * 
 * cdef void copyLinkedList(LinkedList* destination, LinkedList* source):             # <<<<<<<<<<<<<<
 *     """Copy a linked list.
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.copyLinkedList", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":247
 *         curr = curr.next
 * 
 * cdef Link* addFirst(LinkedList* linkedList, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add data to the beginning of a linked list.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addFirst(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList, int __pyx_v_data) {
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("addFirst", 0);

  /* "pypivoter.pyx":262
 *         The link where the data was placed in the linked list.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 *     return addAfter(linkedList.head, data)
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 262, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":263
 *     """
 *     assert linkedList != NULL
 *     return addAfter(linkedList.head, data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_9pypivoter_addAfter(__pyx_v_linkedList->head, __pyx_v_data);
  goto __pyx_L0;

  /* "pypivoter.pyx":247
 *         curr = curr.next
 * 
 * cdef Link* addFirst(LinkedList* linkedList, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add data to the beginning of a linked list.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.addFirst", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":266
 * 
 * 
 * cdef Link* addLast(LinkedList* linkedList, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add data to the end of a linked list.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_addLast(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList, int __pyx_v_data) {
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("addLast", 0);

  /* "pypivoter.pyx":281
 *         The link where the data was placed in the linked list.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 *     return addBefore(linkedList.tail, data)
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 281, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":282
 *     """
 *     assert linkedList != NULL
 *     return addBefore(linkedList.tail, data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_9pypivoter_addBefore(__pyx_v_linkedList->tail, __pyx_v_data);
  goto __pyx_L0;

  /* "pypivoter.pyx":266
 * 
 * 
 * cdef Link* addLast(LinkedList* linkedList, int data):             # <<<<<<<<<<<<<<
 *     """A location-aware function to add data to the end of a linked list.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.addLast", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":285
 * 
 * 
 * cdef int isEmpty(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Determine if a linked list is empty.
 * 
 */

static int __pyx_f_9pypivoter_isEmpty(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isEmpty", 0);

  /* "pypivoter.pyx":298
 *         Non-zero if the linked list is empty, zero otherwise.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 *     return isTail(linkedList.head.next)
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 298, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":299
 *     """
 *     assert linkedList != NULL
 *     return isTail(linkedList.head.next)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_9pypivoter_isTail(__pyx_v_linkedList->head->next);
  goto __pyx_L0;

  /* "pypivoter.pyx":285
 * 
 * 
 * cdef int isEmpty(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Determine if a linked list is empty.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.isEmpty", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":302
 * 
 * 
 * cdef int getFirst(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Return the first piece of data in a linked list.
 * 
 */

static int __pyx_f_9pypivoter_getFirst(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getFirst", 0);

  /* "pypivoter.pyx":315
 *         The data in the first link of the linked list.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 *     assert not isEmpty(linkedList)
 *     return linkedList.head.next.data
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 315, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":316
 *     """
 *     assert linkedList != NULL
 *     assert not isEmpty(linkedList)             # <<<<<<<<<<<<<<
 *     return linkedList.head.next.data
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((!(__pyx_f_9pypivoter_isEmpty(__pyx_v_linkedList) != 0)) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 316, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":317
 *     assert linkedList != NULL
 *     assert not isEmpty(linkedList)
 *     return linkedList.head.next.data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_linkedList->head->next->data;
  goto __pyx_L0;

  /* "pypivoter.pyx":302
 * 
 * 
 * cdef int getFirst(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Return the first piece of data in a linked list.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.getFirst", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":320
 * 
 * 
 * cdef Link* removeFirst(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Remove and return the first link from a linked list.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_removeFirst(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeFirst", 0);

  /* "pypivoter.pyx":333
 *         The first link of the linked list.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 * 
 *     if not isEmpty(linkedList):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 333, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":335
 *     assert linkedList != NULL
 * 
 *     if not isEmpty(linkedList):             # <<<<<<<<<<<<<<
 *         return removeLink(linkedList.head.next)
 *     return NULL
 */
  __pyx_t_1 = ((!(__pyx_f_9pypivoter_isEmpty(__pyx_v_linkedList) != 0)) != 0);
  if (__pyx_t_1) {

    /* "pypivoter.pyx":336
 * 
 *     if not isEmpty(linkedList):
 *         return removeLink(linkedList.head.next)             # <<<<<<<<<<<<<<
 *     return NULL
 * 
 */
    __pyx_r = __pyx_f_9pypivoter_removeLink(__pyx_v_linkedList->head->next);
    goto __pyx_L0;

    /* "pypivoter.pyx":335
 *     assert linkedList != NULL
 * 
 *     if not isEmpty(linkedList):             # <<<<<<<<<<<<<<
 *         return removeLink(linkedList.head.next)
 *     return NULL
 */
  }

  /* "pypivoter.pyx":337
 *     if not isEmpty(linkedList):
 *         return removeLink(linkedList.head.next)
 *     return NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = NULL;
  goto __pyx_L0;

  /* "pypivoter.pyx":320
 * 
 * 
 * cdef Link* removeFirst(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Remove and return the first link from a linked list.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.removeFirst", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":340
 * 
 * 
 * cdef Link* removeLast(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Remove and return the last link from a linked list.
 * 
 */

static __pyx_t_9pypivoter_Link *__pyx_f_9pypivoter_removeLast(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  __pyx_t_9pypivoter_Link *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeLast", 0);

  /* "pypivoter.pyx":353
 *         The last link of the linked list.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 * 
 *     if not isEmpty(linkedList):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 353, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":355
 *     assert linkedList != NULL
 * 
 *     if not isEmpty(linkedList):             # <<<<<<<<<<<<<<
 *         return removeLink(linkedList.tail.prev)
 *     return NULL
 */
  __pyx_t_1 = ((!(__pyx_f_9pypivoter_isEmpty(__pyx_v_linkedList) != 0)) != 0);
  if (__pyx_t_1) {

    /* "pypivoter.pyx":356
 * 
 *     if not isEmpty(linkedList):
 *         return removeLink(linkedList.tail.prev)             # <<<<<<<<<<<<<<
 *     return NULL
 * 
 */
    __pyx_r = __pyx_f_9pypivoter_removeLink(__pyx_v_linkedList->tail->prev);
    goto __pyx_L0;

    /* "pypivoter.pyx":355
 *     assert linkedList != NULL
 * 
 *     if not isEmpty(linkedList):             # <<<<<<<<<<<<<<
 *         return removeLink(linkedList.tail.prev)
 *     return NULL
 */
  }

  /* "pypivoter.pyx":357
 *     if not isEmpty(linkedList):
 *         return removeLink(linkedList.tail.prev)
 *     return NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = NULL;
  goto __pyx_L0;

  /* "pypivoter.pyx":340
 * 
 * 
 * cdef Link* removeLast(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Remove and return the last link from a linked list.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.removeLast", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":360
 * 
 * 
 * cdef void deleteLast(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Delete the last link in a linked list.
 * 
 */

static void __pyx_f_9pypivoter_deleteLast(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deleteLast", 0);

  /* "pypivoter.pyx":368
 *         The linked list from which to delete the last link.
 *     """
 *     assert linkedList != NULL             # <<<<<<<<<<<<<<
 *     if not isEmpty(linkedList):
 *         deleteLink(linkedList.tail.prev)
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_linkedList != NULL) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 368, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":369
 *     """
 *     assert linkedList != NULL
 *     if not isEmpty(linkedList):             # <<<<<<<<<<<<<<
 *         deleteLink(linkedList.tail.prev)
 *     return
 */
  __pyx_t_1 = ((!(__pyx_f_9pypivoter_isEmpty(__pyx_v_linkedList) != 0)) != 0);
  if (__pyx_t_1) {

    /* "pypivoter.pyx":370
 *     assert linkedList != NULL
 *     if not isEmpty(linkedList):
 *         deleteLink(linkedList.tail.prev)             # <<<<<<<<<<<<<<
 *     return
 * 
 */
    (void)(__pyx_f_9pypivoter_deleteLink(__pyx_v_linkedList->tail->prev));

    /* "pypivoter.pyx":369
 *     """
 *     assert linkedList != NULL
 *     if not isEmpty(linkedList):             # <<<<<<<<<<<<<<
 *         deleteLink(linkedList.tail.prev)
 *     return
 */
  }

  /* "pypivoter.pyx":371
 *     if not isEmpty(linkedList):
 *         deleteLink(linkedList.tail.prev)
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  goto __pyx_L0;

  /* "pypivoter.pyx":360
 * 
 * 
 * cdef void deleteLast(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Delete the last link in a linked list.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.deleteLast", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":374
 * 
 * 
 * cdef int length(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Compute the number of data elements in a linked list.
 * 
 */

static int __pyx_f_9pypivoter_length(__pyx_t_9pypivoter_LinkedList *__pyx_v_linkedList) {
  int __pyx_v_length;
  __pyx_t_9pypivoter_Link *__pyx_v_curr;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("length", 0);

  /* "pypivoter.pyx":387
 *         The number of data elements in the linked list.
 *     """
 *     cdef int length = 0             # <<<<<<<<<<<<<<
 *     cdef Link* curr = linkedList.head.next
 * 
 */
  __pyx_v_length = 0;

  /* "pypivoter.pyx":388
 *     """
 *     cdef int length = 0
 *     cdef Link* curr = linkedList.head.next             # <<<<<<<<<<<<<<
 * 
 *     while not isTail(curr):
 */
  __pyx_t_1 = __pyx_v_linkedList->head->next;
  __pyx_v_curr = __pyx_t_1;

  /* "pypivoter.pyx":390
 *     cdef Link* curr = linkedList.head.next
 * 
 *     while not isTail(curr):             # <<<<<<<<<<<<<<
 *         length += 1
 *         curr = curr.next
 */
  while (1) {
    __pyx_t_2 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_curr) != 0)) != 0);
    if (!__pyx_t_2) break;

    /* "pypivoter.pyx":391
 * 
 *     while not isTail(curr):
 *         length += 1             # <<<<<<<<<<<<<<
 *         curr = curr.next
 * 
 */
    __pyx_v_length = (__pyx_v_length + 1);

    /* "pypivoter.pyx":392
 *     while not isTail(curr):
 *         length += 1
 *         curr = curr.next             # <<<<<<<<<<<<<<
 * 
 *     return length
 */
    __pyx_t_1 = __pyx_v_curr->next;
    __pyx_v_curr = __pyx_t_1;
  }

  /* "pypivoter.pyx":394
 *         curr = curr.next
 * 
 *     return length             # <<<<<<<<<<<<<<
 * 
 * ### The following structs provide the basic data structures of neighbor lists.
 */
  __pyx_r = __pyx_v_length;
  goto __pyx_L0;

  /* "pypivoter.pyx":374
 * 
 * 
 * cdef int length(LinkedList* linkedList):             # <<<<<<<<<<<<<<
 *     """Compute the number of data elements in a linked list.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":420
 * 
 * 
 * cdef int computeDegeneracy(LinkedList** list, int size):             # <<<<<<<<<<<<<<
 *     """Compute the degeneracy (maximum number of edges per node) of a graph.
 * 
 */

static int __pyx_f_9pypivoter_computeDegeneracy(__pyx_t_9pypivoter_LinkedList **__pyx_v_list, int __pyx_v_size) {
  int __pyx_v_i;
  PyObject *__pyx_v_degeneracy = 0;
  __pyx_t_9pypivoter_LinkedList **__pyx_v_verticesByDegree;
  __pyx_t_9pypivoter_Link **__pyx_v_vertexLocator;
  int *__pyx_v_degree;
  int __pyx_v_currentDegree;
  int __pyx_v_numVerticesRemoved;
  int __pyx_v_vertex;
  int __pyx_v_neighbor;
  __pyx_t_9pypivoter_LinkedList *__pyx_v_neighborList;
  __pyx_t_9pypivoter_Link *__pyx_v_neighborLink;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("computeDegeneracy", 0);

  /* "pypivoter.pyx":435
 *         The degeneracy of the graph.
 *     """
 *     cdef int i = 0             # <<<<<<<<<<<<<<
 *     cdef degeneracy =0
 * 
 */
  __pyx_v_i = 0;

  /* "pypivoter.pyx":436
 *     """
 *     cdef int i = 0
 *     cdef degeneracy =0             # <<<<<<<<<<<<<<
 * 
 *     # array of lists of vertices, indexed by degree
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_degeneracy = __pyx_int_0;

  /* "pypivoter.pyx":440
 *     # array of lists of vertices, indexed by degree
 *     cdef LinkedList** verticesByDegree = \
 *         <LinkedList**> calloc(size, sizeof(LinkedList*))             # <<<<<<<<<<<<<<
 *     # array of lists of vertices that are first among those of their degree
 *     cdef Link** vertexLocator = <Link**> calloc(size, sizeof(Link*))
 */
  __pyx_v_verticesByDegree = ((__pyx_t_9pypivoter_LinkedList **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_LinkedList *))));

  /* "pypivoter.pyx":442
 *         <LinkedList**> calloc(size, sizeof(LinkedList*))
 *     # array of lists of vertices that are first among those of their degree
 *     cdef Link** vertexLocator = <Link**> calloc(size, sizeof(Link*))             # <<<<<<<<<<<<<<
 * 
 *     cdef int* degree = <int*> calloc(size, sizeof(int))
 */
  __pyx_v_vertexLocator = ((__pyx_t_9pypivoter_Link **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_Link *))));

  /* "pypivoter.pyx":444
 *     cdef Link** vertexLocator = <Link**> calloc(size, sizeof(Link*))
 * 
 *     cdef int* degree = <int*> calloc(size, sizeof(int))             # <<<<<<<<<<<<<<
 * 
 *     for i in range(size):
 */
  __pyx_v_degree = ((int *)calloc(__pyx_v_size, (sizeof(int))));

  /* "pypivoter.pyx":446
 *     cdef int* degree = <int*> calloc(size, sizeof(int))
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         verticesByDegree[i] = createLinkedList()
 * 
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":447
 * 
 *     for i in range(size):
 *         verticesByDegree[i] = createLinkedList()             # <<<<<<<<<<<<<<
 * 
 *     # fill each cell of the degree lookup table, then use that degree to
 */
    (__pyx_v_verticesByDegree[__pyx_v_i]) = __pyx_f_9pypivoter_createLinkedList();
  }

  /* "pypivoter.pyx":451
 *     # fill each cell of the degree lookup table, then use that degree to
 *     # populate the lists of vertices indexed by degree
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         degree[i] = length(list[i])
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":452
 *     # populate the lists of vertices indexed by degree
 *     for i in range(size):
 *         degree[i] = length(list[i])             # <<<<<<<<<<<<<<
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 * 
 */
    (__pyx_v_degree[__pyx_v_i]) = __pyx_f_9pypivoter_length((__pyx_v_list[__pyx_v_i]));

    /* "pypivoter.pyx":453
 *     for i in range(size):
 *         degree[i] = length(list[i])
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)             # <<<<<<<<<<<<<<
 * 
 *     cdef int currentDegree = 0
 */
    (__pyx_v_vertexLocator[__pyx_v_i]) = __pyx_f_9pypivoter_addFirst((__pyx_v_verticesByDegree[(__pyx_v_degree[__pyx_v_i])]), __pyx_v_i);
  }

  /* "pypivoter.pyx":455
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 * 
 *     cdef int currentDegree = 0             # <<<<<<<<<<<<<<
 *     cdef int numVerticesRemoved = 0
 *     cdef int vertex
 */
  __pyx_v_currentDegree = 0;

  /* "pypivoter.pyx":456
 * 
 *     cdef int currentDegree = 0
 *     cdef int numVerticesRemoved = 0             # <<<<<<<<<<<<<<
 *     cdef int vertex
 *     cdef int neighbor
 */
  __pyx_v_numVerticesRemoved = 0;

  /* "pypivoter.pyx":462
 *     cdef Link* neighborLink
 * 
 *     while numVerticesRemoved < size:             # <<<<<<<<<<<<<<
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)
 */
  while (1) {
    __pyx_t_4 = ((__pyx_v_numVerticesRemoved < __pyx_v_size) != 0);
    if (!__pyx_t_4) break;

    /* "pypivoter.pyx":463
 * 
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):             # <<<<<<<<<<<<<<
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 */
    __pyx_t_4 = ((!(__pyx_f_9pypivoter_isEmpty((__pyx_v_verticesByDegree[__pyx_v_currentDegree])) != 0)) != 0);
    if (__pyx_t_4) {

      /* "pypivoter.pyx":464
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)             # <<<<<<<<<<<<<<
 *             vertex = getFirst(verticesByDegree[currentDegree])
 *             deleteLink(vertexLocator[vertex])
 */
      __pyx_t_1 = __pyx_v_currentDegree;
      __Pyx_INCREF(__pyx_v_degeneracy);
      __pyx_t_5 = __pyx_v_degeneracy;
      __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(__pyx_t_7, __pyx_t_5, Py_GT); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (__pyx_t_4) {
        __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 464, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_6 = __pyx_t_8;
        __pyx_t_8 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_6 = __pyx_t_5;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __pyx_t_6;
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_degeneracy, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "pypivoter.pyx":465
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])             # <<<<<<<<<<<<<<
 *             deleteLink(vertexLocator[vertex])
 *             degree[vertex] = -1
 */
      __pyx_v_vertex = __pyx_f_9pypivoter_getFirst((__pyx_v_verticesByDegree[__pyx_v_currentDegree]));

      /* "pypivoter.pyx":466
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 *             deleteLink(vertexLocator[vertex])             # <<<<<<<<<<<<<<
 *             degree[vertex] = -1
 * 
 */
      (void)(__pyx_f_9pypivoter_deleteLink((__pyx_v_vertexLocator[__pyx_v_vertex])));

      /* "pypivoter.pyx":467
 *             vertex = getFirst(verticesByDegree[currentDegree])
 *             deleteLink(vertexLocator[vertex])
 *             degree[vertex] = -1             # <<<<<<<<<<<<<<
 * 
 *             neighborList = list[vertex]
 */
      (__pyx_v_degree[__pyx_v_vertex]) = -1;

      /* "pypivoter.pyx":469
 *             degree[vertex] = -1
 * 
 *             neighborList = list[vertex]             # <<<<<<<<<<<<<<
 *             neighborLink = neighborList.head.next
 * 
 */
      __pyx_v_neighborList = (__pyx_v_list[__pyx_v_vertex]);

      /* "pypivoter.pyx":470
 * 
 *             neighborList = list[vertex]
 *             neighborLink = neighborList.head.next             # <<<<<<<<<<<<<<
 * 
 *             while not isTail(neighborLink):
 */
      __pyx_t_9 = __pyx_v_neighborList->head->next;
      __pyx_v_neighborLink = __pyx_t_9;

      /* "pypivoter.pyx":472
 *             neighborLink = neighborList.head.next
 * 
 *             while not isTail(neighborLink):             # <<<<<<<<<<<<<<
 *                 neighbor = neighborLink.data
 *                 if degree[neighbor] != -1:
 */
      while (1) {
        __pyx_t_4 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_neighborLink) != 0)) != 0);
        if (!__pyx_t_4) break;

        /* "pypivoter.pyx":473
 * 
 *             while not isTail(neighborLink):
 *                 neighbor = neighborLink.data             # <<<<<<<<<<<<<<
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])
 */
        __pyx_t_1 = __pyx_v_neighborLink->data;
        __pyx_v_neighbor = __pyx_t_1;

        /* "pypivoter.pyx":474
 *             while not isTail(neighborLink):
 *                 neighbor = neighborLink.data
 *                 if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                     deleteLink(vertexLocator[neighbor])
 *                     degree[neighbor] -= 1
 */
        __pyx_t_4 = (((__pyx_v_degree[__pyx_v_neighbor]) != -1L) != 0);
        if (__pyx_t_4) {

          /* "pypivoter.pyx":475
 *                 neighbor = neighborLink.data
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])             # <<<<<<<<<<<<<<
 *                     degree[neighbor] -= 1
 * 
 */
          (void)(__pyx_f_9pypivoter_deleteLink((__pyx_v_vertexLocator[__pyx_v_neighbor])));

          /* "pypivoter.pyx":476
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])
 *                     degree[neighbor] -= 1             # <<<<<<<<<<<<<<
 * 
 *                     if degree[neighbor] != -1:
 */
          __pyx_t_1 = __pyx_v_neighbor;
          (__pyx_v_degree[__pyx_t_1]) = ((__pyx_v_degree[__pyx_t_1]) - 1);

          /* "pypivoter.pyx":478
 *                     degree[neighbor] -= 1
 * 
 *                     if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                         vertexLocator[neighbor] = \
 *                             addFirst(verticesByDegree[degree[neighbor]],
 */
          __pyx_t_4 = (((__pyx_v_degree[__pyx_v_neighbor]) != -1L) != 0);
          if (__pyx_t_4) {

            /* "pypivoter.pyx":479
 * 
 *                     if degree[neighbor] != -1:
 *                         vertexLocator[neighbor] = \             # <<<<<<<<<<<<<<
 *                             addFirst(verticesByDegree[degree[neighbor]],
 *                                      neighbor)
 */
            (__pyx_v_vertexLocator[__pyx_v_neighbor]) = __pyx_f_9pypivoter_addFirst((__pyx_v_verticesByDegree[(__pyx_v_degree[__pyx_v_neighbor])]), __pyx_v_neighbor);

            /* "pypivoter.pyx":478
 *                     degree[neighbor] -= 1
 * 
 *                     if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                         vertexLocator[neighbor] = \
 *                             addFirst(verticesByDegree[degree[neighbor]],
 */
          }

          /* "pypivoter.pyx":474
 *             while not isTail(neighborLink):
 *                 neighbor = neighborLink.data
 *                 if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                     deleteLink(vertexLocator[neighbor])
 *                     degree[neighbor] -= 1
 */
        }

        /* "pypivoter.pyx":482
 *                             addFirst(verticesByDegree[degree[neighbor]],
 *                                      neighbor)
 *                 neighborLink = neighborLink.next             # <<<<<<<<<<<<<<
 *             numVerticesRemoved += 1
 *             currentDegree = 0
 */
        __pyx_t_9 = __pyx_v_neighborLink->next;
        __pyx_v_neighborLink = __pyx_t_9;
      }

      /* "pypivoter.pyx":483
 *                                      neighbor)
 *                 neighborLink = neighborLink.next
 *             numVerticesRemoved += 1             # <<<<<<<<<<<<<<
 *             currentDegree = 0
 *         else:
 */
      __pyx_v_numVerticesRemoved = (__pyx_v_numVerticesRemoved + 1);

      /* "pypivoter.pyx":484
 *                 neighborLink = neighborLink.next
 *             numVerticesRemoved += 1
 *             currentDegree = 0             # <<<<<<<<<<<<<<
 *         else:
 *             currentDegree += 1
 */
      __pyx_v_currentDegree = 0;

      /* "pypivoter.pyx":463
 * 
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):             # <<<<<<<<<<<<<<
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 */
      goto __pyx_L9;
    }

    /* "pypivoter.pyx":486
 *             currentDegree = 0
 *         else:
 *             currentDegree += 1             # <<<<<<<<<<<<<<
 * 
 *     for i in range(size):
 */
    /*else*/ {
      __pyx_v_currentDegree = (__pyx_v_currentDegree + 1);
    }
    __pyx_L9:;
  }

  /* "pypivoter.pyx":488
 *             currentDegree += 1
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         destroyLinkedList(verticesByDegree[i])
 * 
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":489
 * 
 *     for i in range(size):
 *         destroyLinkedList(verticesByDegree[i])             # <<<<<<<<<<<<<<
 * 
 *     free(vertexLocator)
 */
    __pyx_f_9pypivoter_destroyLinkedList((__pyx_v_verticesByDegree[__pyx_v_i]));
  }

  /* "pypivoter.pyx":491
 *         destroyLinkedList(verticesByDegree[i])
 * 
 *     free(vertexLocator)             # <<<<<<<<<<<<<<
 *     free(verticesByDegree)
 *     free(degree)
 */
  free(__pyx_v_vertexLocator);

  /* "pypivoter.pyx":492
 * 
 *     free(vertexLocator)
 *     free(verticesByDegree)             # <<<<<<<<<<<<<<
 *     free(degree)
 * 
 */
  free(__pyx_v_verticesByDegree);

  /* "pypivoter.pyx":493
 *     free(vertexLocator)
 *     free(verticesByDegree)
 *     free(degree)             # <<<<<<<<<<<<<<
 * 
 *     return degeneracy
 */
  free(__pyx_v_degree);

  /* "pypivoter.pyx":495
 *     free(degree)
 * 
 *     return degeneracy             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_degeneracy); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 495, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pypivoter.pyx":420
 * 
 * 
 * cdef int computeDegeneracy(LinkedList** list, int size):             # <<<<<<<<<<<<<<
 *     """Compute the degeneracy (maximum number of edges per node) of a graph.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("pypivoter.computeDegeneracy", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_degeneracy);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":498
 * 
 * 
 * cdef NeighborList** computeDegeneracyOrderList(LinkedList** list, int size):             # <<<<<<<<<<<<<<
 *     """Find the smallest-last ordering of a graph as an array of NeighborLists.
 * 
 */

static __pyx_t_9pypivoter_NeighborList **__pyx_f_9pypivoter_computeDegeneracyOrderList(__pyx_t_9pypivoter_LinkedList **__pyx_v_list, int __pyx_v_size) {
  __pyx_t_9pypivoter_NeighborList **__pyx_v_ordering;
  int __pyx_v_i;
  int __pyx_v_degeneracy;
  __pyx_t_9pypivoter_LinkedList **__pyx_v_verticesByDegree;
  __pyx_t_9pypivoter_Link **__pyx_v_vertexLocator;
  int *__pyx_v_degree;
  int __pyx_v_currentDegree;
  int __pyx_v_numVerticesRemoved;
  int __pyx_v_vertex;
  int __pyx_v_neighbor;
  __pyx_t_9pypivoter_LinkedList *__pyx_v_neighborList;
  __pyx_t_9pypivoter_Link *__pyx_v_neighborLink;
  __pyx_t_9pypivoter_NeighborList **__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_5;
  __Pyx_RefNannySetupContext("computeDegeneracyOrderList", 0);

  /* "pypivoter.pyx":515
 *     """
 *     cdef NeighborList** ordering = \
 *         <NeighborList**> calloc(size, sizeof(NeighborList*))             # <<<<<<<<<<<<<<
 *     cdef int i = 0
 *     cdef int degeneracy = 0
 */
  __pyx_v_ordering = ((__pyx_t_9pypivoter_NeighborList **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_NeighborList *))));

  /* "pypivoter.pyx":516
 *     cdef NeighborList** ordering = \
 *         <NeighborList**> calloc(size, sizeof(NeighborList*))
 *     cdef int i = 0             # <<<<<<<<<<<<<<
 *     cdef int degeneracy = 0
 *     # array of lists of vertices, indexed by degree
 */
  __pyx_v_i = 0;

  /* "pypivoter.pyx":517
 *         <NeighborList**> calloc(size, sizeof(NeighborList*))
 *     cdef int i = 0
 *     cdef int degeneracy = 0             # <<<<<<<<<<<<<<
 *     # array of lists of vertices, indexed by degree
 *     cdef LinkedList** verticesByDegree = \
 */
  __pyx_v_degeneracy = 0;

  /* "pypivoter.pyx":520
 *     # array of lists of vertices, indexed by degree
 *     cdef LinkedList** verticesByDegree = \
 *         <LinkedList**> calloc(size, sizeof(LinkedList*))             # <<<<<<<<<<<<<<
 *     # array of lists of vertices that are first among those of their degree
 *     cdef Link** vertexLocator = \
 */
  __pyx_v_verticesByDegree = ((__pyx_t_9pypivoter_LinkedList **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_LinkedList *))));

  /* "pypivoter.pyx":523
 *     # array of lists of vertices that are first among those of their degree
 *     cdef Link** vertexLocator = \
 *         <Link**> calloc(size, sizeof(Link*))             # <<<<<<<<<<<<<<
 * 
 *     cdef int* degree = <int*> calloc(size, sizeof(int))
 */
  __pyx_v_vertexLocator = ((__pyx_t_9pypivoter_Link **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_Link *))));

  /* "pypivoter.pyx":525
 *         <Link**> calloc(size, sizeof(Link*))
 * 
 *     cdef int* degree = <int*> calloc(size, sizeof(int))             # <<<<<<<<<<<<<<
 * 
 *     for i in range(size):
 */
  __pyx_v_degree = ((int *)calloc(__pyx_v_size, (sizeof(int))));

  /* "pypivoter.pyx":527
 *     cdef int* degree = <int*> calloc(size, sizeof(int))
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         verticesByDegree[i] = createLinkedList()
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":528
 * 
 *     for i in range(size):
 *         verticesByDegree[i] = createLinkedList()             # <<<<<<<<<<<<<<
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 *         ordering[i].earlier = createLinkedList()
 */
    (__pyx_v_verticesByDegree[__pyx_v_i]) = __pyx_f_9pypivoter_createLinkedList();

    /* "pypivoter.pyx":529
 *     for i in range(size):
 *         verticesByDegree[i] = createLinkedList()
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))             # <<<<<<<<<<<<<<
 *         ordering[i].earlier = createLinkedList()
 *         ordering[i].later = createLinkedList()
 */
    (__pyx_v_ordering[__pyx_v_i]) = ((__pyx_t_9pypivoter_NeighborList *)malloc((sizeof(__pyx_t_9pypivoter_NeighborList))));

    /* "pypivoter.pyx":530
 *         verticesByDegree[i] = createLinkedList()
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 *         ordering[i].earlier = createLinkedList()             # <<<<<<<<<<<<<<
 *         ordering[i].later = createLinkedList()
 * 
 */
    (__pyx_v_ordering[__pyx_v_i])->earlier = __pyx_f_9pypivoter_createLinkedList();

    /* "pypivoter.pyx":531
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 *         ordering[i].earlier = createLinkedList()
 *         ordering[i].later = createLinkedList()             # <<<<<<<<<<<<<<
 * 
 *     # fill each cell of the degree lookup table, then use that degree to
 */
    (__pyx_v_ordering[__pyx_v_i])->later = __pyx_f_9pypivoter_createLinkedList();
  }

  /* "pypivoter.pyx":535
 *     # fill each cell of the degree lookup table, then use that degree to
 *     # populate the lists of vertices indexed by degree
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         degree[i] = length(list[i])
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":536
 *     # populate the lists of vertices indexed by degree
 *     for i in range(size):
 *         degree[i] = length(list[i])             # <<<<<<<<<<<<<<
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 * 
 */
    (__pyx_v_degree[__pyx_v_i]) = __pyx_f_9pypivoter_length((__pyx_v_list[__pyx_v_i]));

    /* "pypivoter.pyx":537
 *     for i in range(size):
 *         degree[i] = length(list[i])
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)             # <<<<<<<<<<<<<<
 * 
 *     cdef int currentDegree = 0
 */
    (__pyx_v_vertexLocator[__pyx_v_i]) = __pyx_f_9pypivoter_addFirst((__pyx_v_verticesByDegree[(__pyx_v_degree[__pyx_v_i])]), __pyx_v_i);
  }

  /* "pypivoter.pyx":539
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 * 
 *     cdef int currentDegree = 0             # <<<<<<<<<<<<<<
 *     cdef int numVerticesRemoved = 0
 *     cdef int vertex
 */
  __pyx_v_currentDegree = 0;

  /* "pypivoter.pyx":540
 * 
 *     cdef int currentDegree = 0
 *     cdef int numVerticesRemoved = 0             # <<<<<<<<<<<<<<
 *     cdef int vertex
 *     cdef int neighbor
 */
  __pyx_v_numVerticesRemoved = 0;

  /* "pypivoter.pyx":546
 *     cdef Link* neighborLink
 * 
 *     while numVerticesRemoved < size:             # <<<<<<<<<<<<<<
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)
 */
  while (1) {
    __pyx_t_4 = ((__pyx_v_numVerticesRemoved < __pyx_v_size) != 0);
    if (!__pyx_t_4) break;

    /* "pypivoter.pyx":547
 * 
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):             # <<<<<<<<<<<<<<
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 */
    __pyx_t_4 = ((!(__pyx_f_9pypivoter_isEmpty((__pyx_v_verticesByDegree[__pyx_v_currentDegree])) != 0)) != 0);
    if (__pyx_t_4) {

      /* "pypivoter.pyx":548
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)             # <<<<<<<<<<<<<<
 *             vertex = getFirst(verticesByDegree[currentDegree])
 * 
 */
      __pyx_t_1 = __pyx_v_currentDegree;
      __pyx_t_2 = __pyx_v_degeneracy;
      if (((__pyx_t_1 > __pyx_t_2) != 0)) {
        __pyx_t_3 = __pyx_t_1;
      } else {
        __pyx_t_3 = __pyx_t_2;
      }
      __pyx_v_degeneracy = __pyx_t_3;

      /* "pypivoter.pyx":549
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])             # <<<<<<<<<<<<<<
 * 
 *             deleteLink(vertexLocator[vertex]) # delete lowest-degeneracy vertex
 */
      __pyx_v_vertex = __pyx_f_9pypivoter_getFirst((__pyx_v_verticesByDegree[__pyx_v_currentDegree]));

      /* "pypivoter.pyx":551
 *             vertex = getFirst(verticesByDegree[currentDegree])
 * 
 *             deleteLink(vertexLocator[vertex]) # delete lowest-degeneracy vertex             # <<<<<<<<<<<<<<
 * 
 *             ordering[vertex].vertex = vertex
 */
      (void)(__pyx_f_9pypivoter_deleteLink((__pyx_v_vertexLocator[__pyx_v_vertex])));

      /* "pypivoter.pyx":553
 *             deleteLink(vertexLocator[vertex]) # delete lowest-degeneracy vertex
 * 
 *             ordering[vertex].vertex = vertex             # <<<<<<<<<<<<<<
 *             ordering[vertex].orderNumber = numVerticesRemoved
 * 
 */
      (__pyx_v_ordering[__pyx_v_vertex])->vertex = __pyx_v_vertex;

      /* "pypivoter.pyx":554
 * 
 *             ordering[vertex].vertex = vertex
 *             ordering[vertex].orderNumber = numVerticesRemoved             # <<<<<<<<<<<<<<
 * 
 *             degree[vertex] = -1
 */
      (__pyx_v_ordering[__pyx_v_vertex])->orderNumber = __pyx_v_numVerticesRemoved;

      /* "pypivoter.pyx":556
 *             ordering[vertex].orderNumber = numVerticesRemoved
 * 
 *             degree[vertex] = -1             # <<<<<<<<<<<<<<
 * 
 *             neighborList = list[vertex]
 */
      (__pyx_v_degree[__pyx_v_vertex]) = -1;

      /* "pypivoter.pyx":558
 *             degree[vertex] = -1
 * 
 *             neighborList = list[vertex]             # <<<<<<<<<<<<<<
 *             neighborLink = neighborList.head.next
 * 
 */
      __pyx_v_neighborList = (__pyx_v_list[__pyx_v_vertex]);

      /* "pypivoter.pyx":559
 * 
 *             neighborList = list[vertex]
 *             neighborLink = neighborList.head.next             # <<<<<<<<<<<<<<
 * 
 *             while not isTail(neighborLink):
 */
      __pyx_t_5 = __pyx_v_neighborList->head->next;
      __pyx_v_neighborLink = __pyx_t_5;

      /* "pypivoter.pyx":561
 *             neighborLink = neighborList.head.next
 * 
 *             while not isTail(neighborLink):             # <<<<<<<<<<<<<<
 *                 neighbor = neighborLink.data
 * 
 */
      while (1) {
        __pyx_t_4 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_neighborLink) != 0)) != 0);
        if (!__pyx_t_4) break;

        /* "pypivoter.pyx":562
 * 
 *             while not isTail(neighborLink):
 *                 neighbor = neighborLink.data             # <<<<<<<<<<<<<<
 * 
 *                 if degree[neighbor] != -1:
 */
        __pyx_t_3 = __pyx_v_neighborLink->data;
        __pyx_v_neighbor = __pyx_t_3;

        /* "pypivoter.pyx":564
 *                 neighbor = neighborLink.data
 * 
 *                 if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                     deleteLink(vertexLocator[neighbor])
 *                     addLast(ordering[vertex].later, neighbor)
 */
        __pyx_t_4 = (((__pyx_v_degree[__pyx_v_neighbor]) != -1L) != 0);
        if (__pyx_t_4) {

          /* "pypivoter.pyx":565
 * 
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])             # <<<<<<<<<<<<<<
 *                     addLast(ordering[vertex].later, neighbor)
 * 
 */
          (void)(__pyx_f_9pypivoter_deleteLink((__pyx_v_vertexLocator[__pyx_v_neighbor])));

          /* "pypivoter.pyx":566
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])
 *                     addLast(ordering[vertex].later, neighbor)             # <<<<<<<<<<<<<<
 * 
 *                     degree[neighbor] -= 1
 */
          (void)(__pyx_f_9pypivoter_addLast((__pyx_v_ordering[__pyx_v_vertex])->later, __pyx_v_neighbor));

          /* "pypivoter.pyx":568
 *                     addLast(ordering[vertex].later, neighbor)
 * 
 *                     degree[neighbor] -= 1             # <<<<<<<<<<<<<<
 * 
 *                     if degree[neighbor] != -1:
 */
          __pyx_t_3 = __pyx_v_neighbor;
          (__pyx_v_degree[__pyx_t_3]) = ((__pyx_v_degree[__pyx_t_3]) - 1);

          /* "pypivoter.pyx":570
 *                     degree[neighbor] -= 1
 * 
 *                     if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                         vertexLocator[neighbor] = \
 *                             addFirst(verticesByDegree[degree[neighbor]],
 */
          __pyx_t_4 = (((__pyx_v_degree[__pyx_v_neighbor]) != -1L) != 0);
          if (__pyx_t_4) {

            /* "pypivoter.pyx":571
 * 
 *                     if degree[neighbor] != -1:
 *                         vertexLocator[neighbor] = \             # <<<<<<<<<<<<<<
 *                             addFirst(verticesByDegree[degree[neighbor]],
 *                                      neighbor)
 */
            (__pyx_v_vertexLocator[__pyx_v_neighbor]) = __pyx_f_9pypivoter_addFirst((__pyx_v_verticesByDegree[(__pyx_v_degree[__pyx_v_neighbor])]), __pyx_v_neighbor);

            /* "pypivoter.pyx":570
 *                     degree[neighbor] -= 1
 * 
 *                     if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                         vertexLocator[neighbor] = \
 *                             addFirst(verticesByDegree[degree[neighbor]],
 */
          }

          /* "pypivoter.pyx":564
 *                 neighbor = neighborLink.data
 * 
 *                 if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                     deleteLink(vertexLocator[neighbor])
 *                     addLast(ordering[vertex].later, neighbor)
 */
          goto __pyx_L12;
        }

        /* "pypivoter.pyx":575
 *                                      neighbor)
 *                 else:
 *                     addLast(ordering[vertex].earlier, neighbor)             # <<<<<<<<<<<<<<
 * 
 *                 neighborLink = neighborLink.next
 */
        /*else*/ {
          (void)(__pyx_f_9pypivoter_addLast((__pyx_v_ordering[__pyx_v_vertex])->earlier, __pyx_v_neighbor));
        }
        __pyx_L12:;

        /* "pypivoter.pyx":577
 *                     addLast(ordering[vertex].earlier, neighbor)
 * 
 *                 neighborLink = neighborLink.next             # <<<<<<<<<<<<<<
 *             numVerticesRemoved += 1
 *             currentDegree = 0
 */
        __pyx_t_5 = __pyx_v_neighborLink->next;
        __pyx_v_neighborLink = __pyx_t_5;
      }

      /* "pypivoter.pyx":578
 * 
 *                 neighborLink = neighborLink.next
 *             numVerticesRemoved += 1             # <<<<<<<<<<<<<<
 *             currentDegree = 0
 *         else:
 */
      __pyx_v_numVerticesRemoved = (__pyx_v_numVerticesRemoved + 1);

      /* "pypivoter.pyx":579
 *                 neighborLink = neighborLink.next
 *             numVerticesRemoved += 1
 *             currentDegree = 0             # <<<<<<<<<<<<<<
 *         else:
 *             currentDegree += 1
 */
      __pyx_v_currentDegree = 0;

      /* "pypivoter.pyx":547
 * 
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):             # <<<<<<<<<<<<<<
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 */
      goto __pyx_L9;
    }

    /* "pypivoter.pyx":581
 *             currentDegree = 0
 *         else:
 *             currentDegree += 1             # <<<<<<<<<<<<<<
 * 
 *     for i in range(size):
 */
    /*else*/ {
      __pyx_v_currentDegree = (__pyx_v_currentDegree + 1);
    }
    __pyx_L9:;
  }

  /* "pypivoter.pyx":583
 *             currentDegree += 1
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         destroyLinkedList(verticesByDegree[i])
 * 
 */
  __pyx_t_3 = __pyx_v_size;
  __pyx_t_1 = __pyx_t_3;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "pypivoter.pyx":584
 * 
 *     for i in range(size):
 *         destroyLinkedList(verticesByDegree[i])             # <<<<<<<<<<<<<<
 * 
 *     free(vertexLocator)
 */
    __pyx_f_9pypivoter_destroyLinkedList((__pyx_v_verticesByDegree[__pyx_v_i]));
  }

  /* "pypivoter.pyx":586
 *         destroyLinkedList(verticesByDegree[i])
 * 
 *     free(vertexLocator)             # <<<<<<<<<<<<<<
 *     free(verticesByDegree)
 *     free(degree)
 */
  free(__pyx_v_vertexLocator);

  /* "pypivoter.pyx":587
 * 
 *     free(vertexLocator)
 *     free(verticesByDegree)             # <<<<<<<<<<<<<<
 *     free(degree)
 * 
 */
  free(__pyx_v_verticesByDegree);

  /* "pypivoter.pyx":588
 *     free(vertexLocator)
 *     free(verticesByDegree)
 *     free(degree)             # <<<<<<<<<<<<<<
 * 
 *     return ordering
 */
  free(__pyx_v_degree);

  /* "pypivoter.pyx":590
 *     free(degree)
 * 
 *     return ordering             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_ordering;
  goto __pyx_L0;

  /* "pypivoter.pyx":498
 * 
 * 
 * cdef NeighborList** computeDegeneracyOrderList(LinkedList** list, int size):             # <<<<<<<<<<<<<<
 *     """Find the smallest-last ordering of a graph as an array of NeighborLists.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":593
 * 
 * 
 * cdef NeighborListArray** computeDegeneracyOrderArray(LinkedList** list, int size):             # <<<<<<<<<<<<<<
 *     """Find the smallest-last ordering of a graph as an array of NeighborLists.
 * 
 */

static __pyx_t_9pypivoter_NeighborListArray **__pyx_f_9pypivoter_computeDegeneracyOrderArray(__pyx_t_9pypivoter_LinkedList **__pyx_v_list, int __pyx_v_size) {
  __pyx_t_9pypivoter_NeighborList **__pyx_v_ordering;
  int __pyx_v_i;
  int __pyx_v_degeneracy;
  __pyx_t_9pypivoter_LinkedList **__pyx_v_verticesByDegree;
  __pyx_t_9pypivoter_Link **__pyx_v_vertexLocator;
  int *__pyx_v_degree;
  int __pyx_v_currentDegree;
  int __pyx_v_numVerticesRemoved;
  int __pyx_v_vertex;
  int __pyx_v_neighbor;
  __pyx_t_9pypivoter_LinkedList *__pyx_v_neighborList;
  __pyx_t_9pypivoter_Link *__pyx_v_neighborLink;
  __pyx_t_9pypivoter_NeighborListArray **__pyx_v_orderingArray;
  int __pyx_v_j;
  __pyx_t_9pypivoter_Link *__pyx_v_curr;
  __pyx_t_9pypivoter_NeighborListArray **__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  struct __pyx_t_9pypivoter_s_Link *__pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("computeDegeneracyOrderArray", 0);

  /* "pypivoter.pyx":610
 *     """
 *     cdef NeighborList** ordering = \
 *         <NeighborList**> calloc(size, sizeof(NeighborList*))             # <<<<<<<<<<<<<<
 *     cdef int i = 0
 *     cdef int degeneracy = 0
 */
  __pyx_v_ordering = ((__pyx_t_9pypivoter_NeighborList **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_NeighborList *))));

  /* "pypivoter.pyx":611
 *     cdef NeighborList** ordering = \
 *         <NeighborList**> calloc(size, sizeof(NeighborList*))
 *     cdef int i = 0             # <<<<<<<<<<<<<<
 *     cdef int degeneracy = 0
 *     # array of lists of vertices, indexed by degree
 */
  __pyx_v_i = 0;

  /* "pypivoter.pyx":612
 *         <NeighborList**> calloc(size, sizeof(NeighborList*))
 *     cdef int i = 0
 *     cdef int degeneracy = 0             # <<<<<<<<<<<<<<
 *     # array of lists of vertices, indexed by degree
 *     cdef LinkedList** verticesByDegree = \
 */
  __pyx_v_degeneracy = 0;

  /* "pypivoter.pyx":615
 *     # array of lists of vertices, indexed by degree
 *     cdef LinkedList** verticesByDegree = \
 *         <LinkedList**> calloc(size, sizeof(LinkedList*))             # <<<<<<<<<<<<<<
 *     # array of lists of vertices that are first among those of their degree
 *     cdef Link** vertexLocator = \
 */
  __pyx_v_verticesByDegree = ((__pyx_t_9pypivoter_LinkedList **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_LinkedList *))));

  /* "pypivoter.pyx":618
 *     # array of lists of vertices that are first among those of their degree
 *     cdef Link** vertexLocator = \
 *         <Link**> calloc(size, sizeof(Link*))             # <<<<<<<<<<<<<<
 * 
 *     cdef int* degree = <int*> calloc(size, sizeof(int))
 */
  __pyx_v_vertexLocator = ((__pyx_t_9pypivoter_Link **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_Link *))));

  /* "pypivoter.pyx":620
 *         <Link**> calloc(size, sizeof(Link*))
 * 
 *     cdef int* degree = <int*> calloc(size, sizeof(int))             # <<<<<<<<<<<<<<
 * 
 *     for i in range(size):
 */
  __pyx_v_degree = ((int *)calloc(__pyx_v_size, (sizeof(int))));

  /* "pypivoter.pyx":622
 *     cdef int* degree = <int*> calloc(size, sizeof(int))
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         verticesByDegree[i] = createLinkedList()
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":623
 * 
 *     for i in range(size):
 *         verticesByDegree[i] = createLinkedList()             # <<<<<<<<<<<<<<
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 *         ordering[i].earlier = createLinkedList()
 */
    (__pyx_v_verticesByDegree[__pyx_v_i]) = __pyx_f_9pypivoter_createLinkedList();

    /* "pypivoter.pyx":624
 *     for i in range(size):
 *         verticesByDegree[i] = createLinkedList()
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))             # <<<<<<<<<<<<<<
 *         ordering[i].earlier = createLinkedList()
 *         ordering[i].later = createLinkedList()
 */
    (__pyx_v_ordering[__pyx_v_i]) = ((__pyx_t_9pypivoter_NeighborList *)malloc((sizeof(__pyx_t_9pypivoter_NeighborList))));

    /* "pypivoter.pyx":625
 *         verticesByDegree[i] = createLinkedList()
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 *         ordering[i].earlier = createLinkedList()             # <<<<<<<<<<<<<<
 *         ordering[i].later = createLinkedList()
 * 
 */
    (__pyx_v_ordering[__pyx_v_i])->earlier = __pyx_f_9pypivoter_createLinkedList();

    /* "pypivoter.pyx":626
 *         ordering[i] = <NeighborList*> malloc(sizeof(NeighborList))
 *         ordering[i].earlier = createLinkedList()
 *         ordering[i].later = createLinkedList()             # <<<<<<<<<<<<<<
 * 
 *     # fill each cell of the degree lookup table, then use that degree to
 */
    (__pyx_v_ordering[__pyx_v_i])->later = __pyx_f_9pypivoter_createLinkedList();
  }

  /* "pypivoter.pyx":630
 *     # fill each cell of the degree lookup table, then use that degree to
 *     # populate the lists of vertices indexed by degree
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         degree[i] = length(list[i])
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 */
  __pyx_t_1 = __pyx_v_size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":631
 *     # populate the lists of vertices indexed by degree
 *     for i in range(size):
 *         degree[i] = length(list[i])             # <<<<<<<<<<<<<<
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 * 
 */
    (__pyx_v_degree[__pyx_v_i]) = __pyx_f_9pypivoter_length((__pyx_v_list[__pyx_v_i]));

    /* "pypivoter.pyx":632
 *     for i in range(size):
 *         degree[i] = length(list[i])
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)             # <<<<<<<<<<<<<<
 * 
 *     cdef int currentDegree = 0
 */
    (__pyx_v_vertexLocator[__pyx_v_i]) = __pyx_f_9pypivoter_addFirst((__pyx_v_verticesByDegree[(__pyx_v_degree[__pyx_v_i])]), __pyx_v_i);
  }

  /* "pypivoter.pyx":634
 *         vertexLocator[i] = addFirst(verticesByDegree[degree[i]], i)
 * 
 *     cdef int currentDegree = 0             # <<<<<<<<<<<<<<
 *     cdef int numVerticesRemoved = 0
 *     cdef int vertex
 */
  __pyx_v_currentDegree = 0;

  /* "pypivoter.pyx":635
 * 
 *     cdef int currentDegree = 0
 *     cdef int numVerticesRemoved = 0             # <<<<<<<<<<<<<<
 *     cdef int vertex
 *     cdef int neighbor
 */
  __pyx_v_numVerticesRemoved = 0;

  /* "pypivoter.pyx":641
 *     cdef Link* neighborLink
 * 
 *     while numVerticesRemoved < size:             # <<<<<<<<<<<<<<
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)
 */
  while (1) {
    __pyx_t_4 = ((__pyx_v_numVerticesRemoved < __pyx_v_size) != 0);
    if (!__pyx_t_4) break;

    /* "pypivoter.pyx":642
 * 
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):             # <<<<<<<<<<<<<<
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 */
    __pyx_t_4 = ((!(__pyx_f_9pypivoter_isEmpty((__pyx_v_verticesByDegree[__pyx_v_currentDegree])) != 0)) != 0);
    if (__pyx_t_4) {

      /* "pypivoter.pyx":643
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)             # <<<<<<<<<<<<<<
 *             vertex = getFirst(verticesByDegree[currentDegree])
 * 
 */
      __pyx_t_1 = __pyx_v_currentDegree;
      __pyx_t_2 = __pyx_v_degeneracy;
      if (((__pyx_t_1 > __pyx_t_2) != 0)) {
        __pyx_t_3 = __pyx_t_1;
      } else {
        __pyx_t_3 = __pyx_t_2;
      }
      __pyx_v_degeneracy = __pyx_t_3;

      /* "pypivoter.pyx":644
 *         if not isEmpty(verticesByDegree[currentDegree]):
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])             # <<<<<<<<<<<<<<
 * 
 *             deleteLink(vertexLocator[vertex]) # delete lowest-degeneracy vertex
 */
      __pyx_v_vertex = __pyx_f_9pypivoter_getFirst((__pyx_v_verticesByDegree[__pyx_v_currentDegree]));

      /* "pypivoter.pyx":646
 *             vertex = getFirst(verticesByDegree[currentDegree])
 * 
 *             deleteLink(vertexLocator[vertex]) # delete lowest-degeneracy vertex             # <<<<<<<<<<<<<<
 * 
 *             ordering[vertex].vertex = vertex
 */
      (void)(__pyx_f_9pypivoter_deleteLink((__pyx_v_vertexLocator[__pyx_v_vertex])));

      /* "pypivoter.pyx":648
 *             deleteLink(vertexLocator[vertex]) # delete lowest-degeneracy vertex
 * 
 *             ordering[vertex].vertex = vertex             # <<<<<<<<<<<<<<
 *             ordering[vertex].orderNumber = numVerticesRemoved
 * 
 */
      (__pyx_v_ordering[__pyx_v_vertex])->vertex = __pyx_v_vertex;

      /* "pypivoter.pyx":649
 * 
 *             ordering[vertex].vertex = vertex
 *             ordering[vertex].orderNumber = numVerticesRemoved             # <<<<<<<<<<<<<<
 * 
 *             degree[vertex] = -1
 */
      (__pyx_v_ordering[__pyx_v_vertex])->orderNumber = __pyx_v_numVerticesRemoved;

      /* "pypivoter.pyx":651
 *             ordering[vertex].orderNumber = numVerticesRemoved
 * 
 *             degree[vertex] = -1             # <<<<<<<<<<<<<<
 * 
 *             neighborList = list[vertex]
 */
      (__pyx_v_degree[__pyx_v_vertex]) = -1;

      /* "pypivoter.pyx":653
 *             degree[vertex] = -1
 * 
 *             neighborList = list[vertex]             # <<<<<<<<<<<<<<
 *             neighborLink = neighborList.head.next
 * 
 */
      __pyx_v_neighborList = (__pyx_v_list[__pyx_v_vertex]);

      /* "pypivoter.pyx":654
 * 
 *             neighborList = list[vertex]
 *             neighborLink = neighborList.head.next             # <<<<<<<<<<<<<<
 * 
 *             while not isTail(neighborLink):
 */
      __pyx_t_5 = __pyx_v_neighborList->head->next;
      __pyx_v_neighborLink = __pyx_t_5;

      /* "pypivoter.pyx":656
 *             neighborLink = neighborList.head.next
 * 
 *             while not isTail(neighborLink):             # <<<<<<<<<<<<<<
 *                 neighbor = neighborLink.data
 * 
 */
      while (1) {
        __pyx_t_4 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_neighborLink) != 0)) != 0);
        if (!__pyx_t_4) break;

        /* "pypivoter.pyx":657
 * 
 *             while not isTail(neighborLink):
 *                 neighbor = neighborLink.data             # <<<<<<<<<<<<<<
 * 
 *                 if degree[neighbor] != -1:
 */
        __pyx_t_3 = __pyx_v_neighborLink->data;
        __pyx_v_neighbor = __pyx_t_3;

        /* "pypivoter.pyx":659
 *                 neighbor = neighborLink.data
 * 
 *                 if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                     deleteLink(vertexLocator[neighbor])
 *                     addLast(ordering[vertex].later, neighbor)
 */
        __pyx_t_4 = (((__pyx_v_degree[__pyx_v_neighbor]) != -1L) != 0);
        if (__pyx_t_4) {

          /* "pypivoter.pyx":660
 * 
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])             # <<<<<<<<<<<<<<
 *                     addLast(ordering[vertex].later, neighbor)
 * 
 */
          (void)(__pyx_f_9pypivoter_deleteLink((__pyx_v_vertexLocator[__pyx_v_neighbor])));

          /* "pypivoter.pyx":661
 *                 if degree[neighbor] != -1:
 *                     deleteLink(vertexLocator[neighbor])
 *                     addLast(ordering[vertex].later, neighbor)             # <<<<<<<<<<<<<<
 * 
 *                     degree[neighbor] -= 1
 */
          (void)(__pyx_f_9pypivoter_addLast((__pyx_v_ordering[__pyx_v_vertex])->later, __pyx_v_neighbor));

          /* "pypivoter.pyx":663
 *                     addLast(ordering[vertex].later, neighbor)
 * 
 *                     degree[neighbor] -= 1             # <<<<<<<<<<<<<<
 * 
 *                     if degree[neighbor] != -1:
 */
          __pyx_t_3 = __pyx_v_neighbor;
          (__pyx_v_degree[__pyx_t_3]) = ((__pyx_v_degree[__pyx_t_3]) - 1);

          /* "pypivoter.pyx":665
 *                     degree[neighbor] -= 1
 * 
 *                     if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                         vertexLocator[neighbor] = \
 *                             addFirst(verticesByDegree[degree[neighbor]],
 */
          __pyx_t_4 = (((__pyx_v_degree[__pyx_v_neighbor]) != -1L) != 0);
          if (__pyx_t_4) {

            /* "pypivoter.pyx":666
 * 
 *                     if degree[neighbor] != -1:
 *                         vertexLocator[neighbor] = \             # <<<<<<<<<<<<<<
 *                             addFirst(verticesByDegree[degree[neighbor]],
 *                                      neighbor)
 */
            (__pyx_v_vertexLocator[__pyx_v_neighbor]) = __pyx_f_9pypivoter_addFirst((__pyx_v_verticesByDegree[(__pyx_v_degree[__pyx_v_neighbor])]), __pyx_v_neighbor);

            /* "pypivoter.pyx":665
 *                     degree[neighbor] -= 1
 * 
 *                     if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                         vertexLocator[neighbor] = \
 *                             addFirst(verticesByDegree[degree[neighbor]],
 */
          }

          /* "pypivoter.pyx":659
 *                 neighbor = neighborLink.data
 * 
 *                 if degree[neighbor] != -1:             # <<<<<<<<<<<<<<
 *                     deleteLink(vertexLocator[neighbor])
 *                     addLast(ordering[vertex].later, neighbor)
 */
          goto __pyx_L12;
        }

        /* "pypivoter.pyx":670
 *                                      neighbor)
 *                 else:
 *                     addLast(ordering[vertex].earlier, neighbor)             # <<<<<<<<<<<<<<
 * 
 *                 neighborLink = neighborLink.next
 */
        /*else*/ {
          (void)(__pyx_f_9pypivoter_addLast((__pyx_v_ordering[__pyx_v_vertex])->earlier, __pyx_v_neighbor));
        }
        __pyx_L12:;

        /* "pypivoter.pyx":672
 *                     addLast(ordering[vertex].earlier, neighbor)
 * 
 *                 neighborLink = neighborLink.next             # <<<<<<<<<<<<<<
 *             numVerticesRemoved += 1
 *             currentDegree = 0
 */
        __pyx_t_5 = __pyx_v_neighborLink->next;
        __pyx_v_neighborLink = __pyx_t_5;
      }

      /* "pypivoter.pyx":673
 * 
 *                 neighborLink = neighborLink.next
 *             numVerticesRemoved += 1             # <<<<<<<<<<<<<<
 *             currentDegree = 0
 *         else:
 */
      __pyx_v_numVerticesRemoved = (__pyx_v_numVerticesRemoved + 1);

      /* "pypivoter.pyx":674
 *                 neighborLink = neighborLink.next
 *             numVerticesRemoved += 1
 *             currentDegree = 0             # <<<<<<<<<<<<<<
 *         else:
 *             currentDegree += 1
 */
      __pyx_v_currentDegree = 0;

      /* "pypivoter.pyx":642
 * 
 *     while numVerticesRemoved < size:
 *         if not isEmpty(verticesByDegree[currentDegree]):             # <<<<<<<<<<<<<<
 *             degeneracy = max(degeneracy, currentDegree)
 *             vertex = getFirst(verticesByDegree[currentDegree])
 */
      goto __pyx_L9;
    }

    /* "pypivoter.pyx":676
 *             currentDegree = 0
 *         else:
 *             currentDegree += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef NeighborListArray** orderingArray = \
 */
    /*else*/ {
      __pyx_v_currentDegree = (__pyx_v_currentDegree + 1);
    }
    __pyx_L9:;
  }

  /* "pypivoter.pyx":679
 * 
 *     cdef NeighborListArray** orderingArray = \
 *         <NeighborListArray**> calloc(size, sizeof(NeighborListArray))             # <<<<<<<<<<<<<<
 *     cdef int j
 *     cdef Link* curr
 */
  __pyx_v_orderingArray = ((__pyx_t_9pypivoter_NeighborListArray **)calloc(__pyx_v_size, (sizeof(__pyx_t_9pypivoter_NeighborListArray))));

  /* "pypivoter.pyx":683
 *     cdef Link* curr
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         orderingArray[i] = \
 *             <NeighborListArray*> malloc(sizeof(NeighborListArray))
 */
  __pyx_t_3 = __pyx_v_size;
  __pyx_t_1 = __pyx_t_3;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "pypivoter.pyx":684
 * 
 *     for i in range(size):
 *         orderingArray[i] = \             # <<<<<<<<<<<<<<
 *             <NeighborListArray*> malloc(sizeof(NeighborListArray))
 *         orderingArray[i].vertex = ordering[i].vertex
 */
    (__pyx_v_orderingArray[__pyx_v_i]) = ((__pyx_t_9pypivoter_NeighborListArray *)malloc((sizeof(__pyx_t_9pypivoter_NeighborListArray))));

    /* "pypivoter.pyx":686
 *         orderingArray[i] = \
 *             <NeighborListArray*> malloc(sizeof(NeighborListArray))
 *         orderingArray[i].vertex = ordering[i].vertex             # <<<<<<<<<<<<<<
 *         orderingArray[i].orderNumber = ordering[i].orderNumber
 *         orderingArray[i].laterDegree = length(ordering[i].later)
 */
    __pyx_t_6 = (__pyx_v_ordering[__pyx_v_i])->vertex;
    (__pyx_v_orderingArray[__pyx_v_i])->vertex = __pyx_t_6;

    /* "pypivoter.pyx":687
 *             <NeighborListArray*> malloc(sizeof(NeighborListArray))
 *         orderingArray[i].vertex = ordering[i].vertex
 *         orderingArray[i].orderNumber = ordering[i].orderNumber             # <<<<<<<<<<<<<<
 *         orderingArray[i].laterDegree = length(ordering[i].later)
 *         orderingArray[i].later = \
 */
    __pyx_t_6 = (__pyx_v_ordering[__pyx_v_i])->orderNumber;
    (__pyx_v_orderingArray[__pyx_v_i])->orderNumber = __pyx_t_6;

    /* "pypivoter.pyx":688
 *         orderingArray[i].vertex = ordering[i].vertex
 *         orderingArray[i].orderNumber = ordering[i].orderNumber
 *         orderingArray[i].laterDegree = length(ordering[i].later)             # <<<<<<<<<<<<<<
 *         orderingArray[i].later = \
 *             <int*> calloc(orderingArray[i].laterDegree, sizeof(int))
 */
    (__pyx_v_orderingArray[__pyx_v_i])->laterDegree = __pyx_f_9pypivoter_length((__pyx_v_ordering[__pyx_v_i])->later);

    /* "pypivoter.pyx":689
 *         orderingArray[i].orderNumber = ordering[i].orderNumber
 *         orderingArray[i].laterDegree = length(ordering[i].later)
 *         orderingArray[i].later = \             # <<<<<<<<<<<<<<
 *             <int*> calloc(orderingArray[i].laterDegree, sizeof(int))
 * 
 */
    (__pyx_v_orderingArray[__pyx_v_i])->later = ((int *)calloc((__pyx_v_orderingArray[__pyx_v_i])->laterDegree, (sizeof(int))));

    /* "pypivoter.pyx":692
 *             <int*> calloc(orderingArray[i].laterDegree, sizeof(int))
 * 
 *         j = 0             # <<<<<<<<<<<<<<
 *         curr = ordering[i].later.head.next # get first link in ordering
 *         while not isTail(curr):
 */
    __pyx_v_j = 0;

    /* "pypivoter.pyx":693
 * 
 *         j = 0
 *         curr = ordering[i].later.head.next # get first link in ordering             # <<<<<<<<<<<<<<
 *         while not isTail(curr):
 *             orderingArray[i].later[j] = curr.data
 */
    __pyx_t_5 = (__pyx_v_ordering[__pyx_v_i])->later->head->next;
    __pyx_v_curr = __pyx_t_5;

    /* "pypivoter.pyx":694
 *         j = 0
 *         curr = ordering[i].later.head.next # get first link in ordering
 *         while not isTail(curr):             # <<<<<<<<<<<<<<
 *             orderingArray[i].later[j] = curr.data
 *             curr = curr.next
 */
    while (1) {
      __pyx_t_4 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_curr) != 0)) != 0);
      if (!__pyx_t_4) break;

      /* "pypivoter.pyx":695
 *         curr = ordering[i].later.head.next # get first link in ordering
 *         while not isTail(curr):
 *             orderingArray[i].later[j] = curr.data             # <<<<<<<<<<<<<<
 *             curr = curr.next
 *             j += 1
 */
      __pyx_t_6 = __pyx_v_curr->data;
      ((__pyx_v_orderingArray[__pyx_v_i])->later[__pyx_v_j]) = __pyx_t_6;

      /* "pypivoter.pyx":696
 *         while not isTail(curr):
 *             orderingArray[i].later[j] = curr.data
 *             curr = curr.next             # <<<<<<<<<<<<<<
 *             j += 1
 * 
 */
      __pyx_t_5 = __pyx_v_curr->next;
      __pyx_v_curr = __pyx_t_5;

      /* "pypivoter.pyx":697
 *             orderingArray[i].later[j] = curr.data
 *             curr = curr.next
 *             j += 1             # <<<<<<<<<<<<<<
 * 
 *         orderingArray[i].earlierDegree = length(ordering[i].earlier)
 */
      __pyx_v_j = (__pyx_v_j + 1);
    }

    /* "pypivoter.pyx":699
 *             j += 1
 * 
 *         orderingArray[i].earlierDegree = length(ordering[i].earlier)             # <<<<<<<<<<<<<<
 *         orderingArray[i].earlier = \
 *             <int*> calloc(orderingArray[i].earlierDegree, sizeof(int))
 */
    (__pyx_v_orderingArray[__pyx_v_i])->earlierDegree = __pyx_f_9pypivoter_length((__pyx_v_ordering[__pyx_v_i])->earlier);

    /* "pypivoter.pyx":700
 * 
 *         orderingArray[i].earlierDegree = length(ordering[i].earlier)
 *         orderingArray[i].earlier = \             # <<<<<<<<<<<<<<
 *             <int*> calloc(orderingArray[i].earlierDegree, sizeof(int))
 * 
 */
    (__pyx_v_orderingArray[__pyx_v_i])->earlier = ((int *)calloc((__pyx_v_orderingArray[__pyx_v_i])->earlierDegree, (sizeof(int))));

    /* "pypivoter.pyx":703
 *             <int*> calloc(orderingArray[i].earlierDegree, sizeof(int))
 * 
 *         j = 0             # <<<<<<<<<<<<<<
 *         curr = ordering[i].earlier.head.next
 *         while not isTail(curr):
 */
    __pyx_v_j = 0;

    /* "pypivoter.pyx":704
 * 
 *         j = 0
 *         curr = ordering[i].earlier.head.next             # <<<<<<<<<<<<<<
 *         while not isTail(curr):
 *             orderingArray[i].earlier[j] = curr.data
 */
    __pyx_t_5 = (__pyx_v_ordering[__pyx_v_i])->earlier->head->next;
    __pyx_v_curr = __pyx_t_5;

    /* "pypivoter.pyx":705
 *         j = 0
 *         curr = ordering[i].earlier.head.next
 *         while not isTail(curr):             # <<<<<<<<<<<<<<
 *             orderingArray[i].earlier[j] = curr.data
 *             curr = curr.next
 */
    while (1) {
      __pyx_t_4 = ((!(__pyx_f_9pypivoter_isTail(__pyx_v_curr) != 0)) != 0);
      if (!__pyx_t_4) break;

      /* "pypivoter.pyx":706
 *         curr = ordering[i].earlier.head.next
 *         while not isTail(curr):
 *             orderingArray[i].earlier[j] = curr.data             # <<<<<<<<<<<<<<
 *             curr = curr.next
 *             j += 1
 */
      __pyx_t_6 = __pyx_v_curr->data;
      ((__pyx_v_orderingArray[__pyx_v_i])->earlier[__pyx_v_j]) = __pyx_t_6;

      /* "pypivoter.pyx":707
 *         while not isTail(curr):
 *             orderingArray[i].earlier[j] = curr.data
 *             curr = curr.next             # <<<<<<<<<<<<<<
 *             j += 1
 * 
 */
      __pyx_t_5 = __pyx_v_curr->next;
      __pyx_v_curr = __pyx_t_5;

      /* "pypivoter.pyx":708
 *             orderingArray[i].earlier[j] = curr.data
 *             curr = curr.next
 *             j += 1             # <<<<<<<<<<<<<<
 * 
 *     for i in range(size):
 */
      __pyx_v_j = (__pyx_v_j + 1);
    }
  }

  /* "pypivoter.pyx":710
 *             j += 1
 * 
 *     for i in range(size):             # <<<<<<<<<<<<<<
 *         free(ordering[i])
 *         destroyLinkedList(verticesByDegree[i])
 */
  __pyx_t_3 = __pyx_v_size;
  __pyx_t_1 = __pyx_t_3;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "pypivoter.pyx":711
 * 
 *     for i in range(size):
 *         free(ordering[i])             # <<<<<<<<<<<<<<
 *         destroyLinkedList(verticesByDegree[i])
 * 
 */
    free((__pyx_v_ordering[__pyx_v_i]));

    /* "pypivoter.pyx":712
 *     for i in range(size):
 *         free(ordering[i])
 *         destroyLinkedList(verticesByDegree[i])             # <<<<<<<<<<<<<<
 * 
 *     free(vertexLocator)
 */
    __pyx_f_9pypivoter_destroyLinkedList((__pyx_v_verticesByDegree[__pyx_v_i]));
  }

  /* "pypivoter.pyx":714
 *         destroyLinkedList(verticesByDegree[i])
 * 
 *     free(vertexLocator)             # <<<<<<<<<<<<<<
 *     free(verticesByDegree)
 *     free(degree)
 */
  free(__pyx_v_vertexLocator);

  /* "pypivoter.pyx":715
 * 
 *     free(vertexLocator)
 *     free(verticesByDegree)             # <<<<<<<<<<<<<<
 *     free(degree)
 * 
 */
  free(__pyx_v_verticesByDegree);

  /* "pypivoter.pyx":716
 *     free(vertexLocator)
 *     free(verticesByDegree)
 *     free(degree)             # <<<<<<<<<<<<<<
 * 
 *     return orderingArray
 */
  free(__pyx_v_degree);

  /* "pypivoter.pyx":718
 *     free(degree)
 * 
 *     return orderingArray             # <<<<<<<<<<<<<<
 * 
 * cdef int ipow(int base, int exp):
 */
  __pyx_r = __pyx_v_orderingArray;
  goto __pyx_L0;

  /* "pypivoter.pyx":593
 * 
 * 
 * cdef NeighborListArray** computeDegeneracyOrderArray(LinkedList** list, int size):             # <<<<<<<<<<<<<<
 *     """Find the smallest-last ordering of a graph as an array of NeighborLists.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":720
 *     return orderingArray
 * 
 * cdef int ipow(int base, int exp):             # <<<<<<<<<<<<<<
 *     """Raise one integer to the power of another integer.
 * 
 */

static int __pyx_f_9pypivoter_ipow(int __pyx_v_base, int __pyx_v_exp) {
  int __pyx_v_result;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ipow", 0);

  /* "pypivoter.pyx":735
 *         Base raised to the power of exp.
 *     """
 *     if exp == 0:             # <<<<<<<<<<<<<<
 *         return 1
 *     assert exp > 0
 */
  __pyx_t_1 = ((__pyx_v_exp == 0) != 0);
  if (__pyx_t_1) {

    /* "pypivoter.pyx":736
 *     """
 *     if exp == 0:
 *         return 1             # <<<<<<<<<<<<<<
 *     assert exp > 0
 *     cdef int result = 1
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "pypivoter.pyx":735
 *         Base raised to the power of exp.
 *     """
 *     if exp == 0:             # <<<<<<<<<<<<<<
 *         return 1
 *     assert exp > 0
 */
  }

  /* "pypivoter.pyx":737
 *     if exp == 0:
 *         return 1
 *     assert exp > 0             # <<<<<<<<<<<<<<
 *     cdef int result = 1
 *     while 1:
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_exp > 0) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 737, __pyx_L1_error)
    }
  }
  #endif

  /* "pypivoter.pyx":738
 *         return 1
 *     assert exp > 0
 *     cdef int result = 1             # <<<<<<<<<<<<<<
 *     while 1:
 *         if exp & 1:
 */
  __pyx_v_result = 1;

  /* "pypivoter.pyx":739
 *     assert exp > 0
 *     cdef int result = 1
 *     while 1:             # <<<<<<<<<<<<<<
 *         if exp & 1:
 *             result *= base
 */
  while (1) {

    /* "pypivoter.pyx":740
 *     cdef int result = 1
 *     while 1:
 *         if exp & 1:             # <<<<<<<<<<<<<<
 *             result *= base
 *         exp >>= 1 # bit-shift the exponent
 */
    __pyx_t_1 = ((__pyx_v_exp & 1) != 0);
    if (__pyx_t_1) {

      /* "pypivoter.pyx":741
 *     while 1:
 *         if exp & 1:
 *             result *= base             # <<<<<<<<<<<<<<
 *         exp >>= 1 # bit-shift the exponent
 *         if not exp:
 */
      __pyx_v_result = (__pyx_v_result * __pyx_v_base);

      /* "pypivoter.pyx":740
 *     cdef int result = 1
 *     while 1:
 *         if exp & 1:             # <<<<<<<<<<<<<<
 *             result *= base
 *         exp >>= 1 # bit-shift the exponent
 */
    }

    /* "pypivoter.pyx":742
 *         if exp & 1:
 *             result *= base
 *         exp >>= 1 # bit-shift the exponent             # <<<<<<<<<<<<<<
 *         if not exp:
 *             break
 */
    __pyx_v_exp = (__pyx_v_exp >> 1);

    /* "pypivoter.pyx":743
 *             result *= base
 *         exp >>= 1 # bit-shift the exponent
 *         if not exp:             # <<<<<<<<<<<<<<
 *             break
 *         base *= base
 */
    __pyx_t_1 = ((!(__pyx_v_exp != 0)) != 0);
    if (__pyx_t_1) {

      /* "pypivoter.pyx":744
 *         exp >>= 1 # bit-shift the exponent
 *         if not exp:
 *             break             # <<<<<<<<<<<<<<
 *         base *= base
 *     return result
 */
      goto __pyx_L5_break;

      /* "pypivoter.pyx":743
 *             result *= base
 *         exp >>= 1 # bit-shift the exponent
 *         if not exp:             # <<<<<<<<<<<<<<
 *             break
 *         base *= base
 */
    }

    /* "pypivoter.pyx":745
 *         if not exp:
 *             break
 *         base *= base             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
    __pyx_v_base = (__pyx_v_base * __pyx_v_base);
  }
  __pyx_L5_break:;

  /* "pypivoter.pyx":746
 *             break
 *         base *= base
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pypivoter.pyx":720
 *     return orderingArray
 * 
 * cdef int ipow(int base, int exp):             # <<<<<<<<<<<<<<
 *     """Raise one integer to the power of another integer.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.ipow", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":749
 * 
 * 
 * cdef LinkedList** graphAdjArrayToDoubleEdges(int n, int m, int** arr):             # <<<<<<<<<<<<<<
 *     """Read in a graph from a 2 x m array of vertices that form edges and
 *        return an adjacency list as an array of linked lists of integers.
 */

static __pyx_t_9pypivoter_LinkedList **__pyx_f_9pypivoter_graphAdjArrayToDoubleEdges(int __pyx_v_n, int __pyx_v_m, int **__pyx_v_arr) {
  int __pyx_v_u;
  int __pyx_v_v;
  __pyx_t_9pypivoter_LinkedList **__pyx_v_adjList;
  int __pyx_v_i;
  __pyx_t_9pypivoter_LinkedList **__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("graphAdjArrayToDoubleEdges", 0);

  /* "pypivoter.pyx":773
 *     cdef int u, v # end vertices, to read edges
 * 
 *     cdef LinkedList** adjList = <LinkedList**> calloc(n, sizeof(LinkedList*))             # <<<<<<<<<<<<<<
 * 
 *     cdef int i = 0
 */
  __pyx_v_adjList = ((__pyx_t_9pypivoter_LinkedList **)calloc(__pyx_v_n, (sizeof(__pyx_t_9pypivoter_LinkedList *))));

  /* "pypivoter.pyx":775
 *     cdef LinkedList** adjList = <LinkedList**> calloc(n, sizeof(LinkedList*))
 * 
 *     cdef int i = 0             # <<<<<<<<<<<<<<
 *     while i < n:
 *         adjList[i] = createLinkedList()
 */
  __pyx_v_i = 0;

  /* "pypivoter.pyx":776
 * 
 *     cdef int i = 0
 *     while i < n:             # <<<<<<<<<<<<<<
 *         adjList[i] = createLinkedList()
 *         i += 1
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_n) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":777
 *     cdef int i = 0
 *     while i < n:
 *         adjList[i] = createLinkedList()             # <<<<<<<<<<<<<<
 *         i += 1
 * 
 */
    (__pyx_v_adjList[__pyx_v_i]) = __pyx_f_9pypivoter_createLinkedList();

    /* "pypivoter.pyx":778
 *     while i < n:
 *         adjList[i] = createLinkedList()
 *         i += 1             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }

  /* "pypivoter.pyx":780
 *         i += 1
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while i < m:
 *         u = arr[0][i]
 */
  __pyx_v_i = 0;

  /* "pypivoter.pyx":781
 * 
 *     i = 0
 *     while i < m:             # <<<<<<<<<<<<<<
 *         u = arr[0][i]
 *         v = arr[1][i]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_m) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":782
 *     i = 0
 *     while i < m:
 *         u = arr[0][i]             # <<<<<<<<<<<<<<
 *         v = arr[1][i]
 *         assert u < n and u > -1
 */
    __pyx_v_u = ((__pyx_v_arr[0])[__pyx_v_i]);

    /* "pypivoter.pyx":783
 *     while i < m:
 *         u = arr[0][i]
 *         v = arr[1][i]             # <<<<<<<<<<<<<<
 *         assert u < n and u > -1
 *         assert v < n and v > -1
 */
    __pyx_v_v = ((__pyx_v_arr[1])[__pyx_v_i]);

    /* "pypivoter.pyx":784
 *         u = arr[0][i]
 *         v = arr[1][i]
 *         assert u < n and u > -1             # <<<<<<<<<<<<<<
 *         assert v < n and v > -1
 *         assert u != v
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_2 = ((__pyx_v_u < __pyx_v_n) != 0);
      if (__pyx_t_2) {
      } else {
        __pyx_t_1 = __pyx_t_2;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_2 = ((__pyx_v_u > -1L) != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L7_bool_binop_done:;
      if (unlikely(!__pyx_t_1)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 784, __pyx_L1_error)
      }
    }
    #endif

    /* "pypivoter.pyx":785
 *         v = arr[1][i]
 *         assert u < n and u > -1
 *         assert v < n and v > -1             # <<<<<<<<<<<<<<
 *         assert u != v
 * 
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_2 = ((__pyx_v_v < __pyx_v_n) != 0);
      if (__pyx_t_2) {
      } else {
        __pyx_t_1 = __pyx_t_2;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_2 = ((__pyx_v_v > -1L) != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L9_bool_binop_done:;
      if (unlikely(!__pyx_t_1)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 785, __pyx_L1_error)
      }
    }
    #endif

    /* "pypivoter.pyx":786
 *         assert u < n and u > -1
 *         assert v < n and v > -1
 *         assert u != v             # <<<<<<<<<<<<<<
 * 
 *         addLast(adjList[u], v)
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((__pyx_v_u != __pyx_v_v) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 786, __pyx_L1_error)
      }
    }
    #endif

    /* "pypivoter.pyx":788
 *         assert u != v
 * 
 *         addLast(adjList[u], v)             # <<<<<<<<<<<<<<
 *         addLast(adjList[v], u)
 * 
 */
    (void)(__pyx_f_9pypivoter_addLast((__pyx_v_adjList[__pyx_v_u]), __pyx_v_v));

    /* "pypivoter.pyx":789
 * 
 *         addLast(adjList[u], v)
 *         addLast(adjList[v], u)             # <<<<<<<<<<<<<<
 * 
 *         i += 1
 */
    (void)(__pyx_f_9pypivoter_addLast((__pyx_v_adjList[__pyx_v_v]), __pyx_v_u));

    /* "pypivoter.pyx":791
 *         addLast(adjList[v], u)
 * 
 *         i += 1             # <<<<<<<<<<<<<<
 * 
 *     return adjList
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }

  /* "pypivoter.pyx":793
 *         i += 1
 * 
 *     return adjList             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_adjList;
  goto __pyx_L0;

  /* "pypivoter.pyx":749
 * 
 * 
 * cdef LinkedList** graphAdjArrayToDoubleEdges(int n, int m, int** arr):             # <<<<<<<<<<<<<<
 *     """Read in a graph from a 2 x m array of vertices that form edges and
 *        return an adjacency list as an array of linked lists of integers.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.graphAdjArrayToDoubleEdges", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":796
 * 
 * 
 * cdef void fillInPandXForRecursiveCall(int vertex, int orderNumber,             # <<<<<<<<<<<<<<
 *                                       int* vertexSets, int* vertexLookup,
 *                                       NeighborListArray** orderingArray,
 */

static void __pyx_f_9pypivoter_fillInPandXForRecursiveCall(int __pyx_v_vertex, int __pyx_v_orderNumber, int *__pyx_v_vertexSets, int *__pyx_v_vertexLookup, __pyx_t_9pypivoter_NeighborListArray **__pyx_v_orderingArray, int **__pyx_v_neighborsInP, int *__pyx_v_numNeighbors, int __pyx_v_lastVertIndex, int *__pyx_v_pBeginP, int *__pyx_v_pBeginR) {
  int __pyx_v_vertexLocation;
  int __pyx_v_j;
  int __pyx_v_neighbor;
  int __pyx_v_neighborLocation;
  int __pyx_v_vertexInP;
  int __pyx_v_k;
  int __pyx_v_laterNeighbor;
  int __pyx_v_laterNeighborLocation;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("fillInPandXForRecursiveCall", 0);

  /* "pypivoter.pyx":835
 *         after adding vertex to R.
 *     """
 *     cdef int vertexLocation = vertexLookup[vertex]             # <<<<<<<<<<<<<<
 * 
 *     # move vertex to R and update data structures accordingly
 */
  __pyx_v_vertexLocation = (__pyx_v_vertexLookup[__pyx_v_vertex]);

  /* "pypivoter.pyx":838
 * 
 *     # move vertex to R and update data structures accordingly
 *     vertexSets[vertexLocation] = vertexSets[lastVertIndex]             # <<<<<<<<<<<<<<
 *     vertexLookup[vertexSets[lastVertIndex]] = vertexLocation
 *     vertexSets[lastVertIndex] = vertex
 */
  (__pyx_v_vertexSets[__pyx_v_vertexLocation]) = (__pyx_v_vertexSets[__pyx_v_lastVertIndex]);

  /* "pypivoter.pyx":839
 *     # move vertex to R and update data structures accordingly
 *     vertexSets[vertexLocation] = vertexSets[lastVertIndex]
 *     vertexLookup[vertexSets[lastVertIndex]] = vertexLocation             # <<<<<<<<<<<<<<
 *     vertexSets[lastVertIndex] = vertex
 *     vertexLookup[vertex] = lastVertIndex
 */
  (__pyx_v_vertexLookup[(__pyx_v_vertexSets[__pyx_v_lastVertIndex])]) = __pyx_v_vertexLocation;

  /* "pypivoter.pyx":840
 *     vertexSets[vertexLocation] = vertexSets[lastVertIndex]
 *     vertexLookup[vertexSets[lastVertIndex]] = vertexLocation
 *     vertexSets[lastVertIndex] = vertex             # <<<<<<<<<<<<<<
 *     vertexLookup[vertex] = lastVertIndex
 * 
 */
  (__pyx_v_vertexSets[__pyx_v_lastVertIndex]) = __pyx_v_vertex;

  /* "pypivoter.pyx":841
 *     vertexLookup[vertexSets[lastVertIndex]] = vertexLocation
 *     vertexSets[lastVertIndex] = vertex
 *     vertexLookup[vertex] = lastVertIndex             # <<<<<<<<<<<<<<
 * 
 *     pBeginR[0] = lastVertIndex # cython analog of *pBeginR = lastVertIndex
 */
  (__pyx_v_vertexLookup[__pyx_v_vertex]) = __pyx_v_lastVertIndex;

  /* "pypivoter.pyx":843
 *     vertexLookup[vertex] = lastVertIndex
 * 
 *     pBeginR[0] = lastVertIndex # cython analog of *pBeginR = lastVertIndex             # <<<<<<<<<<<<<<
 *     pBeginP[0] = lastVertIndex # cython analog of *pBeginP = lastVertIndex
 * 
 */
  (__pyx_v_pBeginR[0]) = __pyx_v_lastVertIndex;

  /* "pypivoter.pyx":844
 * 
 *     pBeginR[0] = lastVertIndex # cython analog of *pBeginR = lastVertIndex
 *     pBeginP[0] = lastVertIndex # cython analog of *pBeginP = lastVertIndex             # <<<<<<<<<<<<<<
 * 
 *     # swap later neighbors of vertex into P section of vertexSets
 */
  (__pyx_v_pBeginP[0]) = __pyx_v_lastVertIndex;

  /* "pypivoter.pyx":847
 * 
 *     # swap later neighbors of vertex into P section of vertexSets
 *     cdef int j = 0             # <<<<<<<<<<<<<<
 *     cdef int neighbor, neighborLocation
 *     while j < orderingArray[orderNumber].laterDegree:
 */
  __pyx_v_j = 0;

  /* "pypivoter.pyx":849
 *     cdef int j = 0
 *     cdef int neighbor, neighborLocation
 *     while j < orderingArray[orderNumber].laterDegree:             # <<<<<<<<<<<<<<
 *         neighbor = orderingArray[orderNumber].later[j]
 *         neighborLocation = vertexLookup[neighbor]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_j < (__pyx_v_orderingArray[__pyx_v_orderNumber])->laterDegree) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":850
 *     cdef int neighbor, neighborLocation
 *     while j < orderingArray[orderNumber].laterDegree:
 *         neighbor = orderingArray[orderNumber].later[j]             # <<<<<<<<<<<<<<
 *         neighborLocation = vertexLookup[neighbor]
 * 
 */
    __pyx_v_neighbor = ((__pyx_v_orderingArray[__pyx_v_orderNumber])->later[__pyx_v_j]);

    /* "pypivoter.pyx":851
 *     while j < orderingArray[orderNumber].laterDegree:
 *         neighbor = orderingArray[orderNumber].later[j]
 *         neighborLocation = vertexLookup[neighbor]             # <<<<<<<<<<<<<<
 * 
 *         pBeginP[0] -= 1 # cython analog of *pBeginP -= 1
 */
    __pyx_v_neighborLocation = (__pyx_v_vertexLookup[__pyx_v_neighbor]);

    /* "pypivoter.pyx":853
 *         neighborLocation = vertexLookup[neighbor]
 * 
 *         pBeginP[0] -= 1 # cython analog of *pBeginP -= 1             # <<<<<<<<<<<<<<
 * 
 *         vertexSets[neighborLocation] = vertexSets[pBeginP[0]]
 */
    __pyx_t_2 = 0;
    (__pyx_v_pBeginP[__pyx_t_2]) = ((__pyx_v_pBeginP[__pyx_t_2]) - 1);

    /* "pypivoter.pyx":855
 *         pBeginP[0] -= 1 # cython analog of *pBeginP -= 1
 * 
 *         vertexSets[neighborLocation] = vertexSets[pBeginP[0]]             # <<<<<<<<<<<<<<
 *         vertexLookup[vertexSets[pBeginP[0]]] = neighborLocation
 *         vertexSets[pBeginP[0]] = neighbor
 */
    (__pyx_v_vertexSets[__pyx_v_neighborLocation]) = (__pyx_v_vertexSets[(__pyx_v_pBeginP[0])]);

    /* "pypivoter.pyx":856
 * 
 *         vertexSets[neighborLocation] = vertexSets[pBeginP[0]]
 *         vertexLookup[vertexSets[pBeginP[0]]] = neighborLocation             # <<<<<<<<<<<<<<
 *         vertexSets[pBeginP[0]] = neighbor
 *         vertexLookup[neighbor] = pBeginP[0]
 */
    (__pyx_v_vertexLookup[(__pyx_v_vertexSets[(__pyx_v_pBeginP[0])])]) = __pyx_v_neighborLocation;

    /* "pypivoter.pyx":857
 *         vertexSets[neighborLocation] = vertexSets[pBeginP[0]]
 *         vertexLookup[vertexSets[pBeginP[0]]] = neighborLocation
 *         vertexSets[pBeginP[0]] = neighbor             # <<<<<<<<<<<<<<
 *         vertexLookup[neighbor] = pBeginP[0]
 * 
 */
    (__pyx_v_vertexSets[(__pyx_v_pBeginP[0])]) = __pyx_v_neighbor;

    /* "pypivoter.pyx":858
 *         vertexLookup[vertexSets[pBeginP[0]]] = neighborLocation
 *         vertexSets[pBeginP[0]] = neighbor
 *         vertexLookup[neighbor] = pBeginP[0]             # <<<<<<<<<<<<<<
 * 
 *         j += 1
 */
    (__pyx_v_vertexLookup[__pyx_v_neighbor]) = (__pyx_v_pBeginP[0]);

    /* "pypivoter.pyx":860
 *         vertexLookup[neighbor] = pBeginP[0]
 * 
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 *     # reset numNeighbors and neighborsInP for this vertex
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pypivoter.pyx":865
 *     cdef int vertexInP
 *     cdef int calloc_size
 *     j = pBeginP[0]             # <<<<<<<<<<<<<<
 *     while j < pBeginR[0]:
 *         vertexInP = vertexSets[j]
 */
  __pyx_v_j = (__pyx_v_pBeginP[0]);

  /* "pypivoter.pyx":866
 *     cdef int calloc_size
 *     j = pBeginP[0]
 *     while j < pBeginR[0]:             # <<<<<<<<<<<<<<
 *         vertexInP = vertexSets[j]
 *         numNeighbors[vertexInP] = 0
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_j < (__pyx_v_pBeginR[0])) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":867
 *     j = pBeginP[0]
 *     while j < pBeginR[0]:
 *         vertexInP = vertexSets[j]             # <<<<<<<<<<<<<<
 *         numNeighbors[vertexInP] = 0
 *         j += 1
 */
    __pyx_v_vertexInP = (__pyx_v_vertexSets[__pyx_v_j]);

    /* "pypivoter.pyx":868
 *     while j < pBeginR[0]:
 *         vertexInP = vertexSets[j]
 *         numNeighbors[vertexInP] = 0             # <<<<<<<<<<<<<<
 *         j += 1
 * 
 */
    (__pyx_v_numNeighbors[__pyx_v_vertexInP]) = 0;

    /* "pypivoter.pyx":869
 *         vertexInP = vertexSets[j]
 *         numNeighbors[vertexInP] = 0
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 *     # count neighbors in P, and fill array of neighbors in P
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pypivoter.pyx":872
 * 
 *     # count neighbors in P, and fill array of neighbors in P
 *     j = pBeginP[0]             # <<<<<<<<<<<<<<
 *     cdef int k, laterNeighbor, laterNeighborLocation
 *     while j < pBeginR[0]:
 */
  __pyx_v_j = (__pyx_v_pBeginP[0]);

  /* "pypivoter.pyx":874
 *     j = pBeginP[0]
 *     cdef int k, laterNeighbor, laterNeighborLocation
 *     while j < pBeginR[0]:             # <<<<<<<<<<<<<<
 *         vertexInP = vertexSets[j]
 *         k = 0
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_j < (__pyx_v_pBeginR[0])) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":875
 *     cdef int k, laterNeighbor, laterNeighborLocation
 *     while j < pBeginR[0]:
 *         vertexInP = vertexSets[j]             # <<<<<<<<<<<<<<
 *         k = 0
 *         while k < orderingArray[vertexInP].laterDegree:
 */
    __pyx_v_vertexInP = (__pyx_v_vertexSets[__pyx_v_j]);

    /* "pypivoter.pyx":876
 *     while j < pBeginR[0]:
 *         vertexInP = vertexSets[j]
 *         k = 0             # <<<<<<<<<<<<<<
 *         while k < orderingArray[vertexInP].laterDegree:
 *             laterNeighbor = orderingArray[vertexInP].later[k]
 */
    __pyx_v_k = 0;

    /* "pypivoter.pyx":877
 *         vertexInP = vertexSets[j]
 *         k = 0
 *         while k < orderingArray[vertexInP].laterDegree:             # <<<<<<<<<<<<<<
 *             laterNeighbor = orderingArray[vertexInP].later[k]
 *             laterNeighborLocation = vertexLookup[laterNeighbor]
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_k < (__pyx_v_orderingArray[__pyx_v_vertexInP])->laterDegree) != 0);
      if (!__pyx_t_1) break;

      /* "pypivoter.pyx":878
 *         k = 0
 *         while k < orderingArray[vertexInP].laterDegree:
 *             laterNeighbor = orderingArray[vertexInP].later[k]             # <<<<<<<<<<<<<<
 *             laterNeighborLocation = vertexLookup[laterNeighbor]
 * 
 */
      __pyx_v_laterNeighbor = ((__pyx_v_orderingArray[__pyx_v_vertexInP])->later[__pyx_v_k]);

      /* "pypivoter.pyx":879
 *         while k < orderingArray[vertexInP].laterDegree:
 *             laterNeighbor = orderingArray[vertexInP].later[k]
 *             laterNeighborLocation = vertexLookup[laterNeighbor]             # <<<<<<<<<<<<<<
 * 
 *             if laterNeighborLocation >= pBeginP[0] and \
 */
      __pyx_v_laterNeighborLocation = (__pyx_v_vertexLookup[__pyx_v_laterNeighbor]);

      /* "pypivoter.pyx":881
 *             laterNeighborLocation = vertexLookup[laterNeighbor]
 * 
 *             if laterNeighborLocation >= pBeginP[0] and \             # <<<<<<<<<<<<<<
 *                     laterNeighborLocation < pBeginR[0]:
 *                 neighborsInP[vertexInP][numNeighbors[vertexInP]] = \
 */
      __pyx_t_3 = ((__pyx_v_laterNeighborLocation >= (__pyx_v_pBeginP[0])) != 0);
      if (__pyx_t_3) {
      } else {
        __pyx_t_1 = __pyx_t_3;
        goto __pyx_L12_bool_binop_done;
      }

      /* "pypivoter.pyx":882
 * 
 *             if laterNeighborLocation >= pBeginP[0] and \
 *                     laterNeighborLocation < pBeginR[0]:             # <<<<<<<<<<<<<<
 *                 neighborsInP[vertexInP][numNeighbors[vertexInP]] = \
 *                     laterNeighbor
 */
      __pyx_t_3 = ((__pyx_v_laterNeighborLocation < (__pyx_v_pBeginR[0])) != 0);
      __pyx_t_1 = __pyx_t_3;
      __pyx_L12_bool_binop_done:;

      /* "pypivoter.pyx":881
 *             laterNeighborLocation = vertexLookup[laterNeighbor]
 * 
 *             if laterNeighborLocation >= pBeginP[0] and \             # <<<<<<<<<<<<<<
 *                     laterNeighborLocation < pBeginR[0]:
 *                 neighborsInP[vertexInP][numNeighbors[vertexInP]] = \
 */
      if (__pyx_t_1) {

        /* "pypivoter.pyx":883
 *             if laterNeighborLocation >= pBeginP[0] and \
 *                     laterNeighborLocation < pBeginR[0]:
 *                 neighborsInP[vertexInP][numNeighbors[vertexInP]] = \             # <<<<<<<<<<<<<<
 *                     laterNeighbor
 *                 numNeighbors[vertexInP] += 1
 */
        ((__pyx_v_neighborsInP[__pyx_v_vertexInP])[(__pyx_v_numNeighbors[__pyx_v_vertexInP])]) = __pyx_v_laterNeighbor;

        /* "pypivoter.pyx":885
 *                 neighborsInP[vertexInP][numNeighbors[vertexInP]] = \
 *                     laterNeighbor
 *                 numNeighbors[vertexInP] += 1             # <<<<<<<<<<<<<<
 *                 neighborsInP[laterNeighbor][numNeighbors[laterNeighbor]] = \
 *                     vertexInP
 */
        __pyx_t_4 = __pyx_v_vertexInP;
        (__pyx_v_numNeighbors[__pyx_t_4]) = ((__pyx_v_numNeighbors[__pyx_t_4]) + 1);

        /* "pypivoter.pyx":886
 *                     laterNeighbor
 *                 numNeighbors[vertexInP] += 1
 *                 neighborsInP[laterNeighbor][numNeighbors[laterNeighbor]] = \             # <<<<<<<<<<<<<<
 *                     vertexInP
 *                 numNeighbors[laterNeighbor] += 1
 */
        ((__pyx_v_neighborsInP[__pyx_v_laterNeighbor])[(__pyx_v_numNeighbors[__pyx_v_laterNeighbor])]) = __pyx_v_vertexInP;

        /* "pypivoter.pyx":888
 *                 neighborsInP[laterNeighbor][numNeighbors[laterNeighbor]] = \
 *                     vertexInP
 *                 numNeighbors[laterNeighbor] += 1             # <<<<<<<<<<<<<<
 * 
 *             k += 1
 */
        __pyx_t_4 = __pyx_v_laterNeighbor;
        (__pyx_v_numNeighbors[__pyx_t_4]) = ((__pyx_v_numNeighbors[__pyx_t_4]) + 1);

        /* "pypivoter.pyx":881
 *             laterNeighborLocation = vertexLookup[laterNeighbor]
 * 
 *             if laterNeighborLocation >= pBeginP[0] and \             # <<<<<<<<<<<<<<
 *                     laterNeighborLocation < pBeginR[0]:
 *                 neighborsInP[vertexInP][numNeighbors[vertexInP]] = \
 */
      }

      /* "pypivoter.pyx":890
 *                 numNeighbors[laterNeighbor] += 1
 * 
 *             k += 1             # <<<<<<<<<<<<<<
 *         j += 1
 * 
 */
      __pyx_v_k = (__pyx_v_k + 1);
    }

    /* "pypivoter.pyx":891
 * 
 *             k += 1
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pypivoter.pyx":796
 * 
 * 
 * cdef void fillInPandXForRecursiveCall(int vertex, int orderNumber,             # <<<<<<<<<<<<<<
 *                                       int* vertexSets, int* vertexLookup,
 *                                       NeighborListArray** orderingArray,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":894
 * 
 * 
 * cdef void moveToR(int vertex, int* vertexSets, int* vertexLookup,             # <<<<<<<<<<<<<<
 *                   int** neighborsInP, int* numNeighbors, int* pBeginP,
 *                   int* pBeginR, int* pNewBeginP, int* pNewBeginR):
 */

static void __pyx_f_9pypivoter_moveToR(int __pyx_v_vertex, int *__pyx_v_vertexSets, int *__pyx_v_vertexLookup, int **__pyx_v_neighborsInP, int *__pyx_v_numNeighbors, int *__pyx_v_pBeginP, int *__pyx_v_pBeginR, int *__pyx_v_pNewBeginP, int *__pyx_v_pNewBeginR) {
  int __pyx_v_vertexLocation;
  int __pyx_v_sizeOfP;
  int __pyx_v_j;
  int __pyx_v_neighbor;
  int __pyx_v_neighborLocation;
  int __pyx_v_numPotentialNeighbors;
  int __pyx_v_k;
  int __pyx_v_thisVertex;
  int __pyx_v_numNeighborsInP;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("moveToR", 0);

  /* "pypivoter.pyx":927
 *        vertexSets after adding vertex to R.
 *     """
 *     cdef int vertexLocation = vertexLookup[vertex]             # <<<<<<<<<<<<<<
 * 
 *     pBeginR[0] -= 1 # cython analog of *pBeginR -= 1
 */
  __pyx_v_vertexLocation = (__pyx_v_vertexLookup[__pyx_v_vertex]);

  /* "pypivoter.pyx":929
 *     cdef int vertexLocation = vertexLookup[vertex]
 * 
 *     pBeginR[0] -= 1 # cython analog of *pBeginR -= 1             # <<<<<<<<<<<<<<
 *     vertexSets[vertexLocation] = vertexSets[pBeginR[0]]
 *     vertexLookup[vertexSets[pBeginR[0]]] = vertexLocation
 */
  __pyx_t_1 = 0;
  (__pyx_v_pBeginR[__pyx_t_1]) = ((__pyx_v_pBeginR[__pyx_t_1]) - 1);

  /* "pypivoter.pyx":930
 * 
 *     pBeginR[0] -= 1 # cython analog of *pBeginR -= 1
 *     vertexSets[vertexLocation] = vertexSets[pBeginR[0]]             # <<<<<<<<<<<<<<
 *     vertexLookup[vertexSets[pBeginR[0]]] = vertexLocation
 *     vertexSets[pBeginR[0]] = vertex
 */
  (__pyx_v_vertexSets[__pyx_v_vertexLocation]) = (__pyx_v_vertexSets[(__pyx_v_pBeginR[0])]);

  /* "pypivoter.pyx":931
 *     pBeginR[0] -= 1 # cython analog of *pBeginR -= 1
 *     vertexSets[vertexLocation] = vertexSets[pBeginR[0]]
 *     vertexLookup[vertexSets[pBeginR[0]]] = vertexLocation             # <<<<<<<<<<<<<<
 *     vertexSets[pBeginR[0]] = vertex
 *     vertexLookup[vertex] = pBeginR[0]
 */
  (__pyx_v_vertexLookup[(__pyx_v_vertexSets[(__pyx_v_pBeginR[0])])]) = __pyx_v_vertexLocation;

  /* "pypivoter.pyx":932
 *     vertexSets[vertexLocation] = vertexSets[pBeginR[0]]
 *     vertexLookup[vertexSets[pBeginR[0]]] = vertexLocation
 *     vertexSets[pBeginR[0]] = vertex             # <<<<<<<<<<<<<<
 *     vertexLookup[vertex] = pBeginR[0]
 * 
 */
  (__pyx_v_vertexSets[(__pyx_v_pBeginR[0])]) = __pyx_v_vertex;

  /* "pypivoter.pyx":933
 *     vertexLookup[vertexSets[pBeginR[0]]] = vertexLocation
 *     vertexSets[pBeginR[0]] = vertex
 *     vertexLookup[vertex] = pBeginR[0]             # <<<<<<<<<<<<<<
 * 
 *     pNewBeginP[0] = pBeginP[0] # cython analog of *pNewBeginP = *pBeginP
 */
  (__pyx_v_vertexLookup[__pyx_v_vertex]) = (__pyx_v_pBeginR[0]);

  /* "pypivoter.pyx":935
 *     vertexLookup[vertex] = pBeginR[0]
 * 
 *     pNewBeginP[0] = pBeginP[0] # cython analog of *pNewBeginP = *pBeginP             # <<<<<<<<<<<<<<
 *     pNewBeginR[0] = pBeginP[0] # cython analog of *pNewBeginR = *pBeginP
 * 
 */
  (__pyx_v_pNewBeginP[0]) = (__pyx_v_pBeginP[0]);

  /* "pypivoter.pyx":936
 * 
 *     pNewBeginP[0] = pBeginP[0] # cython analog of *pNewBeginP = *pBeginP
 *     pNewBeginR[0] = pBeginP[0] # cython analog of *pNewBeginR = *pBeginP             # <<<<<<<<<<<<<<
 * 
 *     cdef int sizeOfP = pBeginR[0] - pBeginP[0]
 */
  (__pyx_v_pNewBeginR[0]) = (__pyx_v_pBeginP[0]);

  /* "pypivoter.pyx":938
 *     pNewBeginR[0] = pBeginP[0] # cython analog of *pNewBeginR = *pBeginP
 * 
 *     cdef int sizeOfP = pBeginR[0] - pBeginP[0]             # <<<<<<<<<<<<<<
 * 
 *     cdef int j = pBeginP[0]
 */
  __pyx_v_sizeOfP = ((__pyx_v_pBeginR[0]) - (__pyx_v_pBeginP[0]));

  /* "pypivoter.pyx":940
 *     cdef int sizeOfP = pBeginR[0] - pBeginP[0]
 * 
 *     cdef int j = pBeginP[0]             # <<<<<<<<<<<<<<
 *     cdef int neighbor, neighborLocation, numPotentialNeighbors, k
 *     while j < pBeginR[0]:
 */
  __pyx_v_j = (__pyx_v_pBeginP[0]);

  /* "pypivoter.pyx":942
 *     cdef int j = pBeginP[0]
 *     cdef int neighbor, neighborLocation, numPotentialNeighbors, k
 *     while j < pBeginR[0]:             # <<<<<<<<<<<<<<
 *         neighbor = vertexSets[j]
 *         neighborLocation = j
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_j < (__pyx_v_pBeginR[0])) != 0);
    if (!__pyx_t_2) break;

    /* "pypivoter.pyx":943
 *     cdef int neighbor, neighborLocation, numPotentialNeighbors, k
 *     while j < pBeginR[0]:
 *         neighbor = vertexSets[j]             # <<<<<<<<<<<<<<
 *         neighborLocation = j
 * 
 */
    __pyx_v_neighbor = (__pyx_v_vertexSets[__pyx_v_j]);

    /* "pypivoter.pyx":944
 *     while j < pBeginR[0]:
 *         neighbor = vertexSets[j]
 *         neighborLocation = j             # <<<<<<<<<<<<<<
 * 
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[neighbor])
 */
    __pyx_v_neighborLocation = __pyx_v_j;

    /* "pypivoter.pyx":946
 *         neighborLocation = j
 * 
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[neighbor])             # <<<<<<<<<<<<<<
 *         k = 0
 *         while k < numPotentialNeighbors:
 */
    __pyx_t_3 = (__pyx_v_numNeighbors[__pyx_v_neighbor]);
    __pyx_t_4 = __pyx_v_sizeOfP;
    if (((__pyx_t_3 < __pyx_t_4) != 0)) {
      __pyx_t_5 = __pyx_t_3;
    } else {
      __pyx_t_5 = __pyx_t_4;
    }
    __pyx_v_numPotentialNeighbors = __pyx_t_5;

    /* "pypivoter.pyx":947
 * 
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[neighbor])
 *         k = 0             # <<<<<<<<<<<<<<
 *         while k < numPotentialNeighbors:
 *             if neighborsInP[neighbor][k] == vertex:
 */
    __pyx_v_k = 0;

    /* "pypivoter.pyx":948
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[neighbor])
 *         k = 0
 *         while k < numPotentialNeighbors:             # <<<<<<<<<<<<<<
 *             if neighborsInP[neighbor][k] == vertex:
 *                 vertexSets[neighborLocation] = vertexSets[pNewBeginR[0]]
 */
    while (1) {
      __pyx_t_2 = ((__pyx_v_k < __pyx_v_numPotentialNeighbors) != 0);
      if (!__pyx_t_2) break;

      /* "pypivoter.pyx":949
 *         k = 0
 *         while k < numPotentialNeighbors:
 *             if neighborsInP[neighbor][k] == vertex:             # <<<<<<<<<<<<<<
 *                 vertexSets[neighborLocation] = vertexSets[pNewBeginR[0]]
 *                 vertexLookup[vertexSets[pNewBeginR[0]]] = neighborLocation
 */
      __pyx_t_2 = ((((__pyx_v_neighborsInP[__pyx_v_neighbor])[__pyx_v_k]) == __pyx_v_vertex) != 0);
      if (__pyx_t_2) {

        /* "pypivoter.pyx":950
 *         while k < numPotentialNeighbors:
 *             if neighborsInP[neighbor][k] == vertex:
 *                 vertexSets[neighborLocation] = vertexSets[pNewBeginR[0]]             # <<<<<<<<<<<<<<
 *                 vertexLookup[vertexSets[pNewBeginR[0]]] = neighborLocation
 *                 vertexSets[pNewBeginR[0]] = neighbor
 */
        (__pyx_v_vertexSets[__pyx_v_neighborLocation]) = (__pyx_v_vertexSets[(__pyx_v_pNewBeginR[0])]);

        /* "pypivoter.pyx":951
 *             if neighborsInP[neighbor][k] == vertex:
 *                 vertexSets[neighborLocation] = vertexSets[pNewBeginR[0]]
 *                 vertexLookup[vertexSets[pNewBeginR[0]]] = neighborLocation             # <<<<<<<<<<<<<<
 *                 vertexSets[pNewBeginR[0]] = neighbor
 *                 vertexLookup[neighbor] = pNewBeginR[0]
 */
        (__pyx_v_vertexLookup[(__pyx_v_vertexSets[(__pyx_v_pNewBeginR[0])])]) = __pyx_v_neighborLocation;

        /* "pypivoter.pyx":952
 *                 vertexSets[neighborLocation] = vertexSets[pNewBeginR[0]]
 *                 vertexLookup[vertexSets[pNewBeginR[0]]] = neighborLocation
 *                 vertexSets[pNewBeginR[0]] = neighbor             # <<<<<<<<<<<<<<
 *                 vertexLookup[neighbor] = pNewBeginR[0]
 *                 pNewBeginR[0] += 1 # cython analog of *pnewBeginR += 1
 */
        (__pyx_v_vertexSets[(__pyx_v_pNewBeginR[0])]) = __pyx_v_neighbor;

        /* "pypivoter.pyx":953
 *                 vertexLookup[vertexSets[pNewBeginR[0]]] = neighborLocation
 *                 vertexSets[pNewBeginR[0]] = neighbor
 *                 vertexLookup[neighbor] = pNewBeginR[0]             # <<<<<<<<<<<<<<
 *                 pNewBeginR[0] += 1 # cython analog of *pnewBeginR += 1
 *             k += 1
 */
        (__pyx_v_vertexLookup[__pyx_v_neighbor]) = (__pyx_v_pNewBeginR[0]);

        /* "pypivoter.pyx":954
 *                 vertexSets[pNewBeginR[0]] = neighbor
 *                 vertexLookup[neighbor] = pNewBeginR[0]
 *                 pNewBeginR[0] += 1 # cython analog of *pnewBeginR += 1             # <<<<<<<<<<<<<<
 *             k += 1
 * 
 */
        __pyx_t_1 = 0;
        (__pyx_v_pNewBeginR[__pyx_t_1]) = ((__pyx_v_pNewBeginR[__pyx_t_1]) + 1);

        /* "pypivoter.pyx":949
 *         k = 0
 *         while k < numPotentialNeighbors:
 *             if neighborsInP[neighbor][k] == vertex:             # <<<<<<<<<<<<<<
 *                 vertexSets[neighborLocation] = vertexSets[pNewBeginR[0]]
 *                 vertexLookup[vertexSets[pNewBeginR[0]]] = neighborLocation
 */
      }

      /* "pypivoter.pyx":955
 *                 vertexLookup[neighbor] = pNewBeginR[0]
 *                 pNewBeginR[0] += 1 # cython analog of *pnewBeginR += 1
 *             k += 1             # <<<<<<<<<<<<<<
 * 
 *         j += 1
 */
      __pyx_v_k = (__pyx_v_k + 1);
    }

    /* "pypivoter.pyx":957
 *             k += 1
 * 
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 *     j = pNewBeginP[0]
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pypivoter.pyx":959
 *         j += 1
 * 
 *     j = pNewBeginP[0]             # <<<<<<<<<<<<<<
 *     cdef int thisVertex, numNeighborsInP
 *     while j < pNewBeginR[0]:
 */
  __pyx_v_j = (__pyx_v_pNewBeginP[0]);

  /* "pypivoter.pyx":961
 *     j = pNewBeginP[0]
 *     cdef int thisVertex, numNeighborsInP
 *     while j < pNewBeginR[0]:             # <<<<<<<<<<<<<<
 *         thisVertex = vertexSets[j]
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[thisVertex])
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_j < (__pyx_v_pNewBeginR[0])) != 0);
    if (!__pyx_t_2) break;

    /* "pypivoter.pyx":962
 *     cdef int thisVertex, numNeighborsInP
 *     while j < pNewBeginR[0]:
 *         thisVertex = vertexSets[j]             # <<<<<<<<<<<<<<
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[thisVertex])
 *         numNeighborsInP = 0
 */
    __pyx_v_thisVertex = (__pyx_v_vertexSets[__pyx_v_j]);

    /* "pypivoter.pyx":963
 *     while j < pNewBeginR[0]:
 *         thisVertex = vertexSets[j]
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[thisVertex])             # <<<<<<<<<<<<<<
 *         numNeighborsInP = 0
 *         k = 0
 */
    __pyx_t_5 = (__pyx_v_numNeighbors[__pyx_v_thisVertex]);
    __pyx_t_3 = __pyx_v_sizeOfP;
    if (((__pyx_t_5 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_5;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_numPotentialNeighbors = __pyx_t_4;

    /* "pypivoter.pyx":964
 *         thisVertex = vertexSets[j]
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[thisVertex])
 *         numNeighborsInP = 0             # <<<<<<<<<<<<<<
 *         k = 0
 *         while k < numPotentialNeighbors:
 */
    __pyx_v_numNeighborsInP = 0;

    /* "pypivoter.pyx":965
 *         numPotentialNeighbors = min(sizeOfP, numNeighbors[thisVertex])
 *         numNeighborsInP = 0
 *         k = 0             # <<<<<<<<<<<<<<
 *         while k < numPotentialNeighbors:
 *             neighbor = neighborsInP[thisVertex][k]
 */
    __pyx_v_k = 0;

    /* "pypivoter.pyx":966
 *         numNeighborsInP = 0
 *         k = 0
 *         while k < numPotentialNeighbors:             # <<<<<<<<<<<<<<
 *             neighbor = neighborsInP[thisVertex][k]
 *             neighborLocation = vertexLookup[neighbor]
 */
    while (1) {
      __pyx_t_2 = ((__pyx_v_k < __pyx_v_numPotentialNeighbors) != 0);
      if (!__pyx_t_2) break;

      /* "pypivoter.pyx":967
 *         k = 0
 *         while k < numPotentialNeighbors:
 *             neighbor = neighborsInP[thisVertex][k]             # <<<<<<<<<<<<<<
 *             neighborLocation = vertexLookup[neighbor]
 *             if neighborLocation >= pNewBeginP[0] and \
 */
      __pyx_v_neighbor = ((__pyx_v_neighborsInP[__pyx_v_thisVertex])[__pyx_v_k]);

      /* "pypivoter.pyx":968
 *         while k < numPotentialNeighbors:
 *             neighbor = neighborsInP[thisVertex][k]
 *             neighborLocation = vertexLookup[neighbor]             # <<<<<<<<<<<<<<
 *             if neighborLocation >= pNewBeginP[0] and \
 *                     neighborLocation < pNewBeginR[0]:
 */
      __pyx_v_neighborLocation = (__pyx_v_vertexLookup[__pyx_v_neighbor]);

      /* "pypivoter.pyx":969
 *             neighbor = neighborsInP[thisVertex][k]
 *             neighborLocation = vertexLookup[neighbor]
 *             if neighborLocation >= pNewBeginP[0] and \             # <<<<<<<<<<<<<<
 *                     neighborLocation < pNewBeginR[0]:
 *                 neighborsInP[thisVertex][k] = \
 */
      __pyx_t_6 = ((__pyx_v_neighborLocation >= (__pyx_v_pNewBeginP[0])) != 0);
      if (__pyx_t_6) {
      } else {
        __pyx_t_2 = __pyx_t_6;
        goto __pyx_L13_bool_binop_done;
      }

      /* "pypivoter.pyx":970
 *             neighborLocation = vertexLookup[neighbor]
 *             if neighborLocation >= pNewBeginP[0] and \
 *                     neighborLocation < pNewBeginR[0]:             # <<<<<<<<<<<<<<
 *                 neighborsInP[thisVertex][k] = \
 *                     neighborsInP[thisVertex][numNeighborsInP]
 */
      __pyx_t_6 = ((__pyx_v_neighborLocation < (__pyx_v_pNewBeginR[0])) != 0);
      __pyx_t_2 = __pyx_t_6;
      __pyx_L13_bool_binop_done:;

      /* "pypivoter.pyx":969
 *             neighbor = neighborsInP[thisVertex][k]
 *             neighborLocation = vertexLookup[neighbor]
 *             if neighborLocation >= pNewBeginP[0] and \             # <<<<<<<<<<<<<<
 *                     neighborLocation < pNewBeginR[0]:
 *                 neighborsInP[thisVertex][k] = \
 */
      if (__pyx_t_2) {

        /* "pypivoter.pyx":971
 *             if neighborLocation >= pNewBeginP[0] and \
 *                     neighborLocation < pNewBeginR[0]:
 *                 neighborsInP[thisVertex][k] = \             # <<<<<<<<<<<<<<
 *                     neighborsInP[thisVertex][numNeighborsInP]
 *                 neighborsInP[thisVertex][numNeighborsInP] = neighbor
 */
        ((__pyx_v_neighborsInP[__pyx_v_thisVertex])[__pyx_v_k]) = ((__pyx_v_neighborsInP[__pyx_v_thisVertex])[__pyx_v_numNeighborsInP]);

        /* "pypivoter.pyx":973
 *                 neighborsInP[thisVertex][k] = \
 *                     neighborsInP[thisVertex][numNeighborsInP]
 *                 neighborsInP[thisVertex][numNeighborsInP] = neighbor             # <<<<<<<<<<<<<<
 *                 numNeighborsInP += 1
 *             k += 1
 */
        ((__pyx_v_neighborsInP[__pyx_v_thisVertex])[__pyx_v_numNeighborsInP]) = __pyx_v_neighbor;

        /* "pypivoter.pyx":974
 *                     neighborsInP[thisVertex][numNeighborsInP]
 *                 neighborsInP[thisVertex][numNeighborsInP] = neighbor
 *                 numNeighborsInP += 1             # <<<<<<<<<<<<<<
 *             k += 1
 *         j += 1
 */
        __pyx_v_numNeighborsInP = (__pyx_v_numNeighborsInP + 1);

        /* "pypivoter.pyx":969
 *             neighbor = neighborsInP[thisVertex][k]
 *             neighborLocation = vertexLookup[neighbor]
 *             if neighborLocation >= pNewBeginP[0] and \             # <<<<<<<<<<<<<<
 *                     neighborLocation < pNewBeginR[0]:
 *                 neighborsInP[thisVertex][k] = \
 */
      }

      /* "pypivoter.pyx":975
 *                 neighborsInP[thisVertex][numNeighborsInP] = neighbor
 *                 numNeighborsInP += 1
 *             k += 1             # <<<<<<<<<<<<<<
 *         j += 1
 * 
 */
      __pyx_v_k = (__pyx_v_k + 1);
    }

    /* "pypivoter.pyx":976
 *                 numNeighborsInP += 1
 *             k += 1
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pypivoter.pyx":894
 * 
 * 
 * cdef void moveToR(int vertex, int* vertexSets, int* vertexLookup,             # <<<<<<<<<<<<<<
 *                   int** neighborsInP, int* numNeighbors, int* pBeginP,
 *                   int* pBeginR, int* pNewBeginP, int* pNewBeginR):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":979
 * 
 * 
 * cdef void moveFromRToX(int vertex, int* vertexSets, int* vertexLookup,             # <<<<<<<<<<<<<<
 *                        int* pBeginP, int* pBeginR):
 *     """Move a vertex from the set R to the set X and update all necessary
 */

static void __pyx_f_9pypivoter_moveFromRToX(int __pyx_v_vertex, int *__pyx_v_vertexSets, int *__pyx_v_vertexLookup, int *__pyx_v_pBeginP, int *__pyx_v_pBeginR) {
  int __pyx_v_vertexLocation;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("moveFromRToX", 0);

  /* "pypivoter.pyx":998
 *        The index where set R begins in vertexSets.
 *     """
 *     cdef int vertexLocation = vertexLookup[vertex]             # <<<<<<<<<<<<<<
 * 
 *     # swap vertex into X and increment beginP and beginR
 */
  __pyx_v_vertexLocation = (__pyx_v_vertexLookup[__pyx_v_vertex]);

  /* "pypivoter.pyx":1001
 * 
 *     # swap vertex into X and increment beginP and beginR
 *     vertexSets[vertexLocation] = vertexSets[pBeginP[0]]             # <<<<<<<<<<<<<<
 *     vertexLookup[vertexSets[pBeginP[0]]] = vertexLocation
 *     vertexSets[pBeginP[0]] = vertex
 */
  (__pyx_v_vertexSets[__pyx_v_vertexLocation]) = (__pyx_v_vertexSets[(__pyx_v_pBeginP[0])]);

  /* "pypivoter.pyx":1002
 *     # swap vertex into X and increment beginP and beginR
 *     vertexSets[vertexLocation] = vertexSets[pBeginP[0]]
 *     vertexLookup[vertexSets[pBeginP[0]]] = vertexLocation             # <<<<<<<<<<<<<<
 *     vertexSets[pBeginP[0]] = vertex
 *     vertexLookup[vertex] = pBeginP[0]
 */
  (__pyx_v_vertexLookup[(__pyx_v_vertexSets[(__pyx_v_pBeginP[0])])]) = __pyx_v_vertexLocation;

  /* "pypivoter.pyx":1003
 *     vertexSets[vertexLocation] = vertexSets[pBeginP[0]]
 *     vertexLookup[vertexSets[pBeginP[0]]] = vertexLocation
 *     vertexSets[pBeginP[0]] = vertex             # <<<<<<<<<<<<<<
 *     vertexLookup[vertex] = pBeginP[0]
 * 
 */
  (__pyx_v_vertexSets[(__pyx_v_pBeginP[0])]) = __pyx_v_vertex;

  /* "pypivoter.pyx":1004
 *     vertexLookup[vertexSets[pBeginP[0]]] = vertexLocation
 *     vertexSets[pBeginP[0]] = vertex
 *     vertexLookup[vertex] = pBeginP[0]             # <<<<<<<<<<<<<<
 * 
 *     pBeginP[0] = pBeginP[0] + 1 # cython analog of *pBeginP = *pBeginP + 1
 */
  (__pyx_v_vertexLookup[__pyx_v_vertex]) = (__pyx_v_pBeginP[0]);

  /* "pypivoter.pyx":1006
 *     vertexLookup[vertex] = pBeginP[0]
 * 
 *     pBeginP[0] = pBeginP[0] + 1 # cython analog of *pBeginP = *pBeginP + 1             # <<<<<<<<<<<<<<
 *     pBeginR[0] = pBeginR[0] + 1 # cython analog of *pBeginR = *pBeginR + 1
 * 
 */
  (__pyx_v_pBeginP[0]) = ((__pyx_v_pBeginP[0]) + 1);

  /* "pypivoter.pyx":1007
 * 
 *     pBeginP[0] = pBeginP[0] + 1 # cython analog of *pBeginP = *pBeginP + 1
 *     pBeginR[0] = pBeginR[0] + 1 # cython analog of *pBeginR = *pBeginR + 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_pBeginR[0]) = ((__pyx_v_pBeginR[0]) + 1);

  /* "pypivoter.pyx":979
 * 
 * 
 * cdef void moveFromRToX(int vertex, int* vertexSets, int* vertexLookup,             # <<<<<<<<<<<<<<
 *                        int* pBeginP, int* pBeginR):
 *     """Move a vertex from the set R to the set X and update all necessary
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":1010
 * 
 * 
 * cdef int findBestPivotNonNeighbors(int** pivotNonNeighbors,             # <<<<<<<<<<<<<<
 *                                    int* numNonNeighbors,
 *                                    int* vertexSets, int* vertexLookup,
 */

static int __pyx_f_9pypivoter_findBestPivotNonNeighbors(int **__pyx_v_pivotNonNeighbors, int *__pyx_v_numNonNeighbors, int *__pyx_v_vertexSets, int *__pyx_v_vertexLookup, int **__pyx_v_neighborsInP, int *__pyx_v_numNeighbors, int __pyx_v_beginP, int __pyx_v_beginR) {
  int __pyx_v_pivot;
  int __pyx_v_maxIntersectionSize;
  int __pyx_v_j;
  int __pyx_v_vertex;
  int __pyx_v_numPotentialNeighbors;
  int __pyx_v_numNeighborsInP;
  int __pyx_v_k;
  int __pyx_v_neighbor;
  int __pyx_v_neighborLocation;
  int __pyx_v_numPivotNeighbors;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  long __pyx_t_6;
  __Pyx_RefNannySetupContext("findBestPivotNonNeighbors", 0);

  /* "pypivoter.pyx":1050
 *         The integer index of the pivot vertex (i.e. the one with the most nbrs.)
 *     """
 *     cdef int pivot = -1             # <<<<<<<<<<<<<<
 *     cdef int maxIntersectionSize = -1
 * 
 */
  __pyx_v_pivot = -1;

  /* "pypivoter.pyx":1051
 *     """
 *     cdef int pivot = -1
 *     cdef int maxIntersectionSize = -1             # <<<<<<<<<<<<<<
 * 
 *     # iterate over each vertex in P union X to find the vertex with the most
 */
  __pyx_v_maxIntersectionSize = -1;

  /* "pypivoter.pyx":1055
 *     # iterate over each vertex in P union X to find the vertex with the most
 *     # neighbors in P.
 *     cdef int j = beginP             # <<<<<<<<<<<<<<
 *     cdef int vertex, numPotentialNeighbors, numNeighborsInP, \
 *              k, neighbor, neighborLocation
 */
  __pyx_v_j = __pyx_v_beginP;

  /* "pypivoter.pyx":1058
 *     cdef int vertex, numPotentialNeighbors, numNeighborsInP, \
 *              k, neighbor, neighborLocation
 *     while j < beginR:             # <<<<<<<<<<<<<<
 *         vertex = vertexSets[j]
 *         numPotentialNeighbors = min(beginR - beginP, numNeighbors[vertex])
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_j < __pyx_v_beginR) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":1059
 *              k, neighbor, neighborLocation
 *     while j < beginR:
 *         vertex = vertexSets[j]             # <<<<<<<<<<<<<<
 *         numPotentialNeighbors = min(beginR - beginP, numNeighbors[vertex])
 * 
 */
    __pyx_v_vertex = (__pyx_v_vertexSets[__pyx_v_j]);

    /* "pypivoter.pyx":1060
 *     while j < beginR:
 *         vertex = vertexSets[j]
 *         numPotentialNeighbors = min(beginR - beginP, numNeighbors[vertex])             # <<<<<<<<<<<<<<
 * 
 *         numNeighborsInP = 0
 */
    __pyx_t_2 = (__pyx_v_numNeighbors[__pyx_v_vertex]);
    __pyx_t_3 = (__pyx_v_beginR - __pyx_v_beginP);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_numPotentialNeighbors = __pyx_t_4;

    /* "pypivoter.pyx":1062
 *         numPotentialNeighbors = min(beginR - beginP, numNeighbors[vertex])
 * 
 *         numNeighborsInP = 0             # <<<<<<<<<<<<<<
 * 
 *         k = 0
 */
    __pyx_v_numNeighborsInP = 0;

    /* "pypivoter.pyx":1064
 *         numNeighborsInP = 0
 * 
 *         k = 0             # <<<<<<<<<<<<<<
 *         while k < numPotentialNeighbors:
 *             neighbor = neighborsInP[vertex][k]
 */
    __pyx_v_k = 0;

    /* "pypivoter.pyx":1065
 * 
 *         k = 0
 *         while k < numPotentialNeighbors:             # <<<<<<<<<<<<<<
 *             neighbor = neighborsInP[vertex][k]
 *             neighborLocation = vertexLookup[neighbor]
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_k < __pyx_v_numPotentialNeighbors) != 0);
      if (!__pyx_t_1) break;

      /* "pypivoter.pyx":1066
 *         k = 0
 *         while k < numPotentialNeighbors:
 *             neighbor = neighborsInP[vertex][k]             # <<<<<<<<<<<<<<
 *             neighborLocation = vertexLookup[neighbor]
 * 
 */
      __pyx_v_neighbor = ((__pyx_v_neighborsInP[__pyx_v_vertex])[__pyx_v_k]);

      /* "pypivoter.pyx":1067
 *         while k < numPotentialNeighbors:
 *             neighbor = neighborsInP[vertex][k]
 *             neighborLocation = vertexLookup[neighbor]             # <<<<<<<<<<<<<<
 * 
 *             if neighborLocation >= beginP and neighborLocation < beginR:
 */
      __pyx_v_neighborLocation = (__pyx_v_vertexLookup[__pyx_v_neighbor]);

      /* "pypivoter.pyx":1069
 *             neighborLocation = vertexLookup[neighbor]
 * 
 *             if neighborLocation >= beginP and neighborLocation < beginR:             # <<<<<<<<<<<<<<
 *                 numNeighborsInP += 1
 *             else:
 */
      __pyx_t_5 = ((__pyx_v_neighborLocation >= __pyx_v_beginP) != 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_1 = __pyx_t_5;
        goto __pyx_L8_bool_binop_done;
      }
      __pyx_t_5 = ((__pyx_v_neighborLocation < __pyx_v_beginR) != 0);
      __pyx_t_1 = __pyx_t_5;
      __pyx_L8_bool_binop_done:;
      if (__pyx_t_1) {

        /* "pypivoter.pyx":1070
 * 
 *             if neighborLocation >= beginP and neighborLocation < beginR:
 *                 numNeighborsInP += 1             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
        __pyx_v_numNeighborsInP = (__pyx_v_numNeighborsInP + 1);

        /* "pypivoter.pyx":1069
 *             neighborLocation = vertexLookup[neighbor]
 * 
 *             if neighborLocation >= beginP and neighborLocation < beginR:             # <<<<<<<<<<<<<<
 *                 numNeighborsInP += 1
 *             else:
 */
        goto __pyx_L7;
      }

      /* "pypivoter.pyx":1072
 *                 numNeighborsInP += 1
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             k += 1
 */
      /*else*/ {
        goto __pyx_L6_break;
      }
      __pyx_L7:;

      /* "pypivoter.pyx":1074
 *                 break
 * 
 *             k += 1             # <<<<<<<<<<<<<<
 * 
 *         if numNeighborsInP > maxIntersectionSize:
 */
      __pyx_v_k = (__pyx_v_k + 1);
    }
    __pyx_L6_break:;

    /* "pypivoter.pyx":1076
 *             k += 1
 * 
 *         if numNeighborsInP > maxIntersectionSize:             # <<<<<<<<<<<<<<
 *             pivot = vertex
 *             maxIntersectionSize = numNeighborsInP
 */
    __pyx_t_1 = ((__pyx_v_numNeighborsInP > __pyx_v_maxIntersectionSize) != 0);
    if (__pyx_t_1) {

      /* "pypivoter.pyx":1077
 * 
 *         if numNeighborsInP > maxIntersectionSize:
 *             pivot = vertex             # <<<<<<<<<<<<<<
 *             maxIntersectionSize = numNeighborsInP
 * 
 */
      __pyx_v_pivot = __pyx_v_vertex;

      /* "pypivoter.pyx":1078
 *         if numNeighborsInP > maxIntersectionSize:
 *             pivot = vertex
 *             maxIntersectionSize = numNeighborsInP             # <<<<<<<<<<<<<<
 * 
 *         j += 1
 */
      __pyx_v_maxIntersectionSize = __pyx_v_numNeighborsInP;

      /* "pypivoter.pyx":1076
 *             k += 1
 * 
 *         if numNeighborsInP > maxIntersectionSize:             # <<<<<<<<<<<<<<
 *             pivot = vertex
 *             maxIntersectionSize = numNeighborsInP
 */
    }

    /* "pypivoter.pyx":1080
 *             maxIntersectionSize = numNeighborsInP
 * 
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 *     # Compute non neighbors of pivot by marking its neighbors and moving non-
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pypivoter.pyx":1088
 *     # Enough space is initialized for all of P; this is slightly space
 *     # inefficient, but it results in faster computation of non-neighbors.
 *     pivotNonNeighbors[0] = <int*> calloc(beginR - beginP, sizeof(int))             # <<<<<<<<<<<<<<
 *     # cython analog of *pivotNonNeighbors = ...
 *     memcpy(pivotNonNeighbors[0], &vertexSets[beginP],
 */
  (__pyx_v_pivotNonNeighbors[0]) = ((int *)calloc((__pyx_v_beginR - __pyx_v_beginP), (sizeof(int))));

  /* "pypivoter.pyx":1090
 *     pivotNonNeighbors[0] = <int*> calloc(beginR - beginP, sizeof(int))
 *     # cython analog of *pivotNonNeighbors = ...
 *     memcpy(pivotNonNeighbors[0], &vertexSets[beginP],             # <<<<<<<<<<<<<<
 *            (beginR - beginP) * sizeof(int))
 * 
 */
  (void)(memcpy((__pyx_v_pivotNonNeighbors[0]), (&(__pyx_v_vertexSets[__pyx_v_beginP])), ((__pyx_v_beginR - __pyx_v_beginP) * (sizeof(int)))));

  /* "pypivoter.pyx":1094
 * 
 *     # numNonNeighbors will be decremented as neighbors are found
 *     numNonNeighbors[0] = beginR - beginP             # <<<<<<<<<<<<<<
 *     # cython analog of *numNonNeighbors = beginR - beginP
 * 
 */
  (__pyx_v_numNonNeighbors[0]) = (__pyx_v_beginR - __pyx_v_beginP);

  /* "pypivoter.pyx":1097
 *     # cython analog of *numNonNeighbors = beginR - beginP
 * 
 *     cdef int numPivotNeighbors = min(beginR - beginP, numNeighbors[pivot])             # <<<<<<<<<<<<<<
 * 
 *     # mark the neighbors of pivot that are in P
 */
  __pyx_t_4 = (__pyx_v_numNeighbors[__pyx_v_pivot]);
  __pyx_t_2 = (__pyx_v_beginR - __pyx_v_beginP);
  if (((__pyx_t_4 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_4;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_numPivotNeighbors = __pyx_t_3;

  /* "pypivoter.pyx":1100
 * 
 *     # mark the neighbors of pivot that are in P
 *     j = 0             # <<<<<<<<<<<<<<
 *     while j < numPivotNeighbors:
 *         neighbor = neighborsInP[pivot][j]
 */
  __pyx_v_j = 0;

  /* "pypivoter.pyx":1101
 *     # mark the neighbors of pivot that are in P
 *     j = 0
 *     while j < numPivotNeighbors:             # <<<<<<<<<<<<<<
 *         neighbor = neighborsInP[pivot][j]
 *         neighborLocation = vertexLookup[neighbor]
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_j < __pyx_v_numPivotNeighbors) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":1102
 *     j = 0
 *     while j < numPivotNeighbors:
 *         neighbor = neighborsInP[pivot][j]             # <<<<<<<<<<<<<<
 *         neighborLocation = vertexLookup[neighbor]
 * 
 */
    __pyx_v_neighbor = ((__pyx_v_neighborsInP[__pyx_v_pivot])[__pyx_v_j]);

    /* "pypivoter.pyx":1103
 *     while j < numPivotNeighbors:
 *         neighbor = neighborsInP[pivot][j]
 *         neighborLocation = vertexLookup[neighbor]             # <<<<<<<<<<<<<<
 * 
 *         if neighborLocation >= beginP and neighborLocation < beginR:
 */
    __pyx_v_neighborLocation = (__pyx_v_vertexLookup[__pyx_v_neighbor]);

    /* "pypivoter.pyx":1105
 *         neighborLocation = vertexLookup[neighbor]
 * 
 *         if neighborLocation >= beginP and neighborLocation < beginR:             # <<<<<<<<<<<<<<
 *             pivotNonNeighbors[0][neighborLocation - beginP] = -1
 *             # cython analog of *pivotNonNeighbors[neighborLocation - beginP] =-1
 */
    __pyx_t_5 = ((__pyx_v_neighborLocation >= __pyx_v_beginP) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_5 = ((__pyx_v_neighborLocation < __pyx_v_beginR) != 0);
    __pyx_t_1 = __pyx_t_5;
    __pyx_L14_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pypivoter.pyx":1106
 * 
 *         if neighborLocation >= beginP and neighborLocation < beginR:
 *             pivotNonNeighbors[0][neighborLocation - beginP] = -1             # <<<<<<<<<<<<<<
 *             # cython analog of *pivotNonNeighbors[neighborLocation - beginP] =-1
 *         else:
 */
      ((__pyx_v_pivotNonNeighbors[0])[(__pyx_v_neighborLocation - __pyx_v_beginP)]) = -1;

      /* "pypivoter.pyx":1105
 *         neighborLocation = vertexLookup[neighbor]
 * 
 *         if neighborLocation >= beginP and neighborLocation < beginR:             # <<<<<<<<<<<<<<
 *             pivotNonNeighbors[0][neighborLocation - beginP] = -1
 *             # cython analog of *pivotNonNeighbors[neighborLocation - beginP] =-1
 */
      goto __pyx_L13;
    }

    /* "pypivoter.pyx":1109
 *             # cython analog of *pivotNonNeighbors[neighborLocation - beginP] =-1
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         j += 1
 * 
 */
    /*else*/ {
      goto __pyx_L12_break;
    }
    __pyx_L13:;

    /* "pypivoter.pyx":1110
 *         else:
 *             break
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 *     # Move non-neighbors of pivot in P to the beginning of pivotNonNeighbors
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }
  __pyx_L12_break:;

  /* "pypivoter.pyx":1116
 *     # If a vertex is marked as a neighbor, it is moved to the end of
 *     # pivotNonNeighbors and numNonNeighbors is decremented.
 *     j = 0             # <<<<<<<<<<<<<<
 *     while j < numNonNeighbors[0]:
 *         vertex = pivotNonNeighbors[0][j]
 */
  __pyx_v_j = 0;

  /* "pypivoter.pyx":1117
 *     # pivotNonNeighbors and numNonNeighbors is decremented.
 *     j = 0
 *     while j < numNonNeighbors[0]:             # <<<<<<<<<<<<<<
 *         vertex = pivotNonNeighbors[0][j]
 * 
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_j < (__pyx_v_numNonNeighbors[0])) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":1118
 *     j = 0
 *     while j < numNonNeighbors[0]:
 *         vertex = pivotNonNeighbors[0][j]             # <<<<<<<<<<<<<<
 * 
 *         if vertex == -1:
 */
    __pyx_v_vertex = ((__pyx_v_pivotNonNeighbors[0])[__pyx_v_j]);

    /* "pypivoter.pyx":1120
 *         vertex = pivotNonNeighbors[0][j]
 * 
 *         if vertex == -1:             # <<<<<<<<<<<<<<
 *             numNonNeighbors[0] -= 1 # cython analog of *numNonNeighbors* -= 1
 *             pivotNonNeighbors[0][j] = pivotNonNeighbors[0][numNonNeighbors[0]]
 */
    __pyx_t_1 = ((__pyx_v_vertex == -1L) != 0);
    if (__pyx_t_1) {

      /* "pypivoter.pyx":1121
 * 
 *         if vertex == -1:
 *             numNonNeighbors[0] -= 1 # cython analog of *numNonNeighbors* -= 1             # <<<<<<<<<<<<<<
 *             pivotNonNeighbors[0][j] = pivotNonNeighbors[0][numNonNeighbors[0]]
 *             continue
 */
      __pyx_t_6 = 0;
      (__pyx_v_numNonNeighbors[__pyx_t_6]) = ((__pyx_v_numNonNeighbors[__pyx_t_6]) - 1);

      /* "pypivoter.pyx":1122
 *         if vertex == -1:
 *             numNonNeighbors[0] -= 1 # cython analog of *numNonNeighbors* -= 1
 *             pivotNonNeighbors[0][j] = pivotNonNeighbors[0][numNonNeighbors[0]]             # <<<<<<<<<<<<<<
 *             continue
 *         j += 1
 */
      ((__pyx_v_pivotNonNeighbors[0])[__pyx_v_j]) = ((__pyx_v_pivotNonNeighbors[0])[(__pyx_v_numNonNeighbors[0])]);

      /* "pypivoter.pyx":1123
 *             numNonNeighbors[0] -= 1 # cython analog of *numNonNeighbors* -= 1
 *             pivotNonNeighbors[0][j] = pivotNonNeighbors[0][numNonNeighbors[0]]
 *             continue             # <<<<<<<<<<<<<<
 *         j += 1
 * 
 */
      goto __pyx_L16_continue;

      /* "pypivoter.pyx":1120
 *         vertex = pivotNonNeighbors[0][j]
 * 
 *         if vertex == -1:             # <<<<<<<<<<<<<<
 *             numNonNeighbors[0] -= 1 # cython analog of *numNonNeighbors* -= 1
 *             pivotNonNeighbors[0][j] = pivotNonNeighbors[0][numNonNeighbors[0]]
 */
    }

    /* "pypivoter.pyx":1124
 *             pivotNonNeighbors[0][j] = pivotNonNeighbors[0][numNonNeighbors[0]]
 *             continue
 *         j += 1             # <<<<<<<<<<<<<<
 * 
 *     return pivot
 */
    __pyx_v_j = (__pyx_v_j + 1);
    __pyx_L16_continue:;
  }

  /* "pypivoter.pyx":1126
 *         j += 1
 * 
 *     return pivot             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_pivot;
  goto __pyx_L0;

  /* "pypivoter.pyx":1010
 * 
 * 
 * cdef int findBestPivotNonNeighbors(int** pivotNonNeighbors,             # <<<<<<<<<<<<<<
 *                                    int* numNonNeighbors,
 *                                    int* vertexSets, int* vertexLookup,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":1129
 * 
 * 
 * cdef void listAllCliquesDegeneracyRecursive(int** cliques, int* cliqueCounts,             # <<<<<<<<<<<<<<
 *                                             int* vertexSets, int* vertexLookup,
 *                                             int** neighborsInP,
 */

static void __pyx_f_9pypivoter_listAllCliquesDegeneracyRecursive(int **__pyx_v_cliques, int *__pyx_v_cliqueCounts, int *__pyx_v_vertexSets, int *__pyx_v_vertexLookup, int **__pyx_v_neighborsInP, int *__pyx_v_numNeighbors, int *__pyx_v_isHold, int *__pyx_v_isPivot, int __pyx_v_beginP, int __pyx_v_beginR, int __pyx_v_n, int __pyx_v_max_k, int __pyx_v_rsize, int __pyx_v_drop, int *__pyx_v_cliqueNum, int __pyx_v_enumerate) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_k;
  int __pyx_v_l;
  int __pyx_v_m;
  int __pyx_v_nCliquesToAdd;
  int __pyx_v_nAdded;
  int __pyx_v_nCr_num;
  int __pyx_v_nCr_denom;
  int *__pyx_v_candidatesToIterateThrough;
  int __pyx_v_numCandidatesToIterateThrough;
  int __pyx_v_pivot;
  int __pyx_v_iterator;
  int __pyx_v_vertex;
  int __pyx_v_vertexLocation;
  int __pyx_v_newBeginP;
  int __pyx_v_newBeginR;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  long __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("listAllCliquesDegeneracyRecursive", 0);

  /* "pypivoter.pyx":1188
 *     cdef int i, j, k, l, m
 *     cdef int vertexNum, nCliquesToAdd, nAdded, nCr_num, nCr_denom
 *     if beginP >= beginR or rsize - drop > max_k:             # <<<<<<<<<<<<<<
 *         if enumerate == 1: # enumerate cliques
 *             nCliquesToAdd = ipow(2, drop)
 */
  __pyx_t_2 = ((__pyx_v_beginP >= __pyx_v_beginR) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (((__pyx_v_rsize - __pyx_v_drop) > __pyx_v_max_k) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pypivoter.pyx":1189
 *     cdef int vertexNum, nCliquesToAdd, nAdded, nCr_num, nCr_denom
 *     if beginP >= beginR or rsize - drop > max_k:
 *         if enumerate == 1: # enumerate cliques             # <<<<<<<<<<<<<<
 *             nCliquesToAdd = ipow(2, drop)
 *             nAdded = 0
 */
    __pyx_t_1 = ((__pyx_v_enumerate == 1) != 0);
    if (__pyx_t_1) {

      /* "pypivoter.pyx":1190
 *     if beginP >= beginR or rsize - drop > max_k:
 *         if enumerate == 1: # enumerate cliques
 *             nCliquesToAdd = ipow(2, drop)             # <<<<<<<<<<<<<<
 *             nAdded = 0
 *             for i in range(nCliquesToAdd):
 */
      __pyx_v_nCliquesToAdd = __pyx_f_9pypivoter_ipow(2, __pyx_v_drop);

      /* "pypivoter.pyx":1191
 *         if enumerate == 1: # enumerate cliques
 *             nCliquesToAdd = ipow(2, drop)
 *             nAdded = 0             # <<<<<<<<<<<<<<
 *             for i in range(nCliquesToAdd):
 *                 # for each clique, add all holds and some subset of the pivots
 */
      __pyx_v_nAdded = 0;

      /* "pypivoter.pyx":1192
 *             nCliquesToAdd = ipow(2, drop)
 *             nAdded = 0
 *             for i in range(nCliquesToAdd):             # <<<<<<<<<<<<<<
 *                 # for each clique, add all holds and some subset of the pivots
 *                 k = 0 # this is not the same k as the clique size
 */
      __pyx_t_3 = __pyx_v_nCliquesToAdd;
      __pyx_t_4 = __pyx_t_3;
      for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "pypivoter.pyx":1194
 *             for i in range(nCliquesToAdd):
 *                 # for each clique, add all holds and some subset of the pivots
 *                 k = 0 # this is not the same k as the clique size             # <<<<<<<<<<<<<<
 *                       # this counter keeps track of the # of pivots seen
 *                 l = 0 # this counter keeps track of the # of pivots accepted
 */
        __pyx_v_k = 0;

        /* "pypivoter.pyx":1196
 *                 k = 0 # this is not the same k as the clique size
 *                       # this counter keeps track of the # of pivots seen
 *                 l = 0 # this counter keeps track of the # of pivots accepted             # <<<<<<<<<<<<<<
 *                 m = 0 # this counter keeps track of the # of holds seen/accepted
 *                 for j in range(beginR, n):
 */
        __pyx_v_l = 0;

        /* "pypivoter.pyx":1197
 *                       # this counter keeps track of the # of pivots seen
 *                 l = 0 # this counter keeps track of the # of pivots accepted
 *                 m = 0 # this counter keeps track of the # of holds seen/accepted             # <<<<<<<<<<<<<<
 *                 for j in range(beginR, n):
 *                     if isPivot[vertexSets[j]]:
 */
        __pyx_v_m = 0;

        /* "pypivoter.pyx":1198
 *                 l = 0 # this counter keeps track of the # of pivots accepted
 *                 m = 0 # this counter keeps track of the # of holds seen/accepted
 *                 for j in range(beginR, n):             # <<<<<<<<<<<<<<
 *                     if isPivot[vertexSets[j]]:
 *                         if (i >> k) & 1:
 */
        __pyx_t_6 = __pyx_v_n;
        __pyx_t_7 = __pyx_t_6;
        for (__pyx_t_8 = __pyx_v_beginR; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
          __pyx_v_j = __pyx_t_8;

          /* "pypivoter.pyx":1199
 *                 m = 0 # this counter keeps track of the # of holds seen/accepted
 *                 for j in range(beginR, n):
 *                     if isPivot[vertexSets[j]]:             # <<<<<<<<<<<<<<
 *                         if (i >> k) & 1:
 *                             cliques[cliqueNum[0] + nAdded][rsize - drop + l] = \
 */
          __pyx_t_1 = ((__pyx_v_isPivot[(__pyx_v_vertexSets[__pyx_v_j])]) != 0);
          if (__pyx_t_1) {

            /* "pypivoter.pyx":1200
 *                 for j in range(beginR, n):
 *                     if isPivot[vertexSets[j]]:
 *                         if (i >> k) & 1:             # <<<<<<<<<<<<<<
 *                             cliques[cliqueNum[0] + nAdded][rsize - drop + l] = \
 *                                 vertexSets[j]
 */
            __pyx_t_1 = (((__pyx_v_i >> __pyx_v_k) & 1) != 0);
            if (__pyx_t_1) {

              /* "pypivoter.pyx":1201
 *                     if isPivot[vertexSets[j]]:
 *                         if (i >> k) & 1:
 *                             cliques[cliqueNum[0] + nAdded][rsize - drop + l] = \             # <<<<<<<<<<<<<<
 *                                 vertexSets[j]
 *                             l += 1
 */
              ((__pyx_v_cliques[((__pyx_v_cliqueNum[0]) + __pyx_v_nAdded)])[((__pyx_v_rsize - __pyx_v_drop) + __pyx_v_l)]) = (__pyx_v_vertexSets[__pyx_v_j]);

              /* "pypivoter.pyx":1203
 *                             cliques[cliqueNum[0] + nAdded][rsize - drop + l] = \
 *                                 vertexSets[j]
 *                             l += 1             # <<<<<<<<<<<<<<
 *                         k += 1
 *                     elif isHold[vertexSets[j]]:
 */
              __pyx_v_l = (__pyx_v_l + 1);

              /* "pypivoter.pyx":1200
 *                 for j in range(beginR, n):
 *                     if isPivot[vertexSets[j]]:
 *                         if (i >> k) & 1:             # <<<<<<<<<<<<<<
 *                             cliques[cliqueNum[0] + nAdded][rsize - drop + l] = \
 *                                 vertexSets[j]
 */
            }

            /* "pypivoter.pyx":1204
 *                                 vertexSets[j]
 *                             l += 1
 *                         k += 1             # <<<<<<<<<<<<<<
 *                     elif isHold[vertexSets[j]]:
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]
 */
            __pyx_v_k = (__pyx_v_k + 1);

            /* "pypivoter.pyx":1199
 *                 m = 0 # this counter keeps track of the # of holds seen/accepted
 *                 for j in range(beginR, n):
 *                     if isPivot[vertexSets[j]]:             # <<<<<<<<<<<<<<
 *                         if (i >> k) & 1:
 *                             cliques[cliqueNum[0] + nAdded][rsize - drop + l] = \
 */
            goto __pyx_L11;
          }

          /* "pypivoter.pyx":1205
 *                             l += 1
 *                         k += 1
 *                     elif isHold[vertexSets[j]]:             # <<<<<<<<<<<<<<
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]
 *                         m += 1
 */
          __pyx_t_1 = ((__pyx_v_isHold[(__pyx_v_vertexSets[__pyx_v_j])]) != 0);
          if (__pyx_t_1) {

            /* "pypivoter.pyx":1206
 *                         k += 1
 *                     elif isHold[vertexSets[j]]:
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]             # <<<<<<<<<<<<<<
 *                         m += 1
 *                 if l + m <= max_k:
 */
            ((__pyx_v_cliques[((__pyx_v_cliqueNum[0]) + __pyx_v_nAdded)])[__pyx_v_m]) = (__pyx_v_vertexSets[__pyx_v_j]);

            /* "pypivoter.pyx":1207
 *                     elif isHold[vertexSets[j]]:
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]
 *                         m += 1             # <<<<<<<<<<<<<<
 *                 if l + m <= max_k:
 *                     # if clique does not exceed max_k, increment nAdded so
 */
            __pyx_v_m = (__pyx_v_m + 1);

            /* "pypivoter.pyx":1205
 *                             l += 1
 *                         k += 1
 *                     elif isHold[vertexSets[j]]:             # <<<<<<<<<<<<<<
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]
 *                         m += 1
 */
          }
          __pyx_L11:;
        }

        /* "pypivoter.pyx":1208
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]
 *                         m += 1
 *                 if l + m <= max_k:             # <<<<<<<<<<<<<<
 *                     # if clique does not exceed max_k, increment nAdded so
 *                     # that it is not overwritten
 */
        __pyx_t_1 = (((__pyx_v_l + __pyx_v_m) <= __pyx_v_max_k) != 0);
        if (__pyx_t_1) {

          /* "pypivoter.pyx":1211
 *                     # if clique does not exceed max_k, increment nAdded so
 *                     # that it is not overwritten
 *                     nAdded += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     for j in range(l + m):
 */
          __pyx_v_nAdded = (__pyx_v_nAdded + 1);

          /* "pypivoter.pyx":1208
 *                         cliques[cliqueNum[0] + nAdded][m] = vertexSets[j]
 *                         m += 1
 *                 if l + m <= max_k:             # <<<<<<<<<<<<<<
 *                     # if clique does not exceed max_k, increment nAdded so
 *                     # that it is not overwritten
 */
          goto __pyx_L13;
        }

        /* "pypivoter.pyx":1213
 *                     nAdded += 1
 *                 else:
 *                     for j in range(l + m):             # <<<<<<<<<<<<<<
 *                         cliques[cliqueNum[0] + nAdded][j] = -1
 *             cliqueNum[0] += nAdded
 */
        /*else*/ {
          __pyx_t_6 = (__pyx_v_l + __pyx_v_m);
          __pyx_t_7 = __pyx_t_6;
          for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
            __pyx_v_j = __pyx_t_8;

            /* "pypivoter.pyx":1214
 *                 else:
 *                     for j in range(l + m):
 *                         cliques[cliqueNum[0] + nAdded][j] = -1             # <<<<<<<<<<<<<<
 *             cliqueNum[0] += nAdded
 *         else: # count cliques
 */
            ((__pyx_v_cliques[((__pyx_v_cliqueNum[0]) + __pyx_v_nAdded)])[__pyx_v_j]) = -1;
          }
        }
        __pyx_L13:;
      }

      /* "pypivoter.pyx":1215
 *                     for j in range(l + m):
 *                         cliques[cliqueNum[0] + nAdded][j] = -1
 *             cliqueNum[0] += nAdded             # <<<<<<<<<<<<<<
 *         else: # count cliques
 *             i = drop
 */
      __pyx_t_9 = 0;
      (__pyx_v_cliqueNum[__pyx_t_9]) = ((__pyx_v_cliqueNum[__pyx_t_9]) + __pyx_v_nAdded);

      /* "pypivoter.pyx":1189
 *     cdef int vertexNum, nCliquesToAdd, nAdded, nCr_num, nCr_denom
 *     if beginP >= beginR or rsize - drop > max_k:
 *         if enumerate == 1: # enumerate cliques             # <<<<<<<<<<<<<<
 *             nCliquesToAdd = ipow(2, drop)
 *             nAdded = 0
 */
      goto __pyx_L6;
    }

    /* "pypivoter.pyx":1217
 *             cliqueNum[0] += nAdded
 *         else: # count cliques
 *             i = drop             # <<<<<<<<<<<<<<
 *             k = rsize - drop
 *             while i >= 0 and k <= max_k:
 */
    /*else*/ {
      __pyx_v_i = __pyx_v_drop;

      /* "pypivoter.pyx":1218
 *         else: # count cliques
 *             i = drop
 *             k = rsize - drop             # <<<<<<<<<<<<<<
 *             while i >= 0 and k <= max_k:
 *                 k = rsize - i
 */
      __pyx_v_k = (__pyx_v_rsize - __pyx_v_drop);

      /* "pypivoter.pyx":1219
 *             i = drop
 *             k = rsize - drop
 *             while i >= 0 and k <= max_k:             # <<<<<<<<<<<<<<
 *                 k = rsize - i
 *                 nCr_num = 1
 */
      while (1) {
        __pyx_t_2 = ((__pyx_v_i >= 0) != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_2 = ((__pyx_v_k <= __pyx_v_max_k) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L18_bool_binop_done:;
        if (!__pyx_t_1) break;

        /* "pypivoter.pyx":1220
 *             k = rsize - drop
 *             while i >= 0 and k <= max_k:
 *                 k = rsize - i             # <<<<<<<<<<<<<<
 *                 nCr_num = 1
 *                 nCr_denom = 1
 */
        __pyx_v_k = (__pyx_v_rsize - __pyx_v_i);

        /* "pypivoter.pyx":1221
 *             while i >= 0 and k <= max_k:
 *                 k = rsize - i
 *                 nCr_num = 1             # <<<<<<<<<<<<<<
 *                 nCr_denom = 1
 *                 for j in range(i):
 */
        __pyx_v_nCr_num = 1;

        /* "pypivoter.pyx":1222
 *                 k = rsize - i
 *                 nCr_num = 1
 *                 nCr_denom = 1             # <<<<<<<<<<<<<<
 *                 for j in range(i):
 *                     nCr_num *= drop - j
 */
        __pyx_v_nCr_denom = 1;

        /* "pypivoter.pyx":1223
 *                 nCr_num = 1
 *                 nCr_denom = 1
 *                 for j in range(i):             # <<<<<<<<<<<<<<
 *                     nCr_num *= drop - j
 *                     nCr_denom *= j + 1
 */
        __pyx_t_3 = __pyx_v_i;
        __pyx_t_4 = __pyx_t_3;
        for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
          __pyx_v_j = __pyx_t_5;

          /* "pypivoter.pyx":1224
 *                 nCr_denom = 1
 *                 for j in range(i):
 *                     nCr_num *= drop - j             # <<<<<<<<<<<<<<
 *                     nCr_denom *= j + 1
 *                 cliqueCounts[k] += nCr_num // nCr_denom
 */
          __pyx_v_nCr_num = (__pyx_v_nCr_num * (__pyx_v_drop - __pyx_v_j));

          /* "pypivoter.pyx":1225
 *                 for j in range(i):
 *                     nCr_num *= drop - j
 *                     nCr_denom *= j + 1             # <<<<<<<<<<<<<<
 *                 cliqueCounts[k] += nCr_num // nCr_denom
 *                 # cliqueCounts[k] += <int> nCr[drop][i]
 */
          __pyx_v_nCr_denom = (__pyx_v_nCr_denom * (__pyx_v_j + 1));
        }

        /* "pypivoter.pyx":1226
 *                     nCr_num *= drop - j
 *                     nCr_denom *= j + 1
 *                 cliqueCounts[k] += nCr_num // nCr_denom             # <<<<<<<<<<<<<<
 *                 # cliqueCounts[k] += <int> nCr[drop][i]
 *                 i -= 1
 */
        __pyx_t_3 = __pyx_v_k;
        if (unlikely(__pyx_v_nCr_denom == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
          __PYX_ERR(0, 1226, __pyx_L1_error)
        }
        else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_nCr_denom == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_nCr_num))) {
          PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
          __PYX_ERR(0, 1226, __pyx_L1_error)
        }
        (__pyx_v_cliqueCounts[__pyx_t_3]) = ((__pyx_v_cliqueCounts[__pyx_t_3]) + __Pyx_div_int(__pyx_v_nCr_num, __pyx_v_nCr_denom));

        /* "pypivoter.pyx":1228
 *                 cliqueCounts[k] += nCr_num // nCr_denom
 *                 # cliqueCounts[k] += <int> nCr[drop][i]
 *                 i -= 1             # <<<<<<<<<<<<<<
 *         return
 * 
 */
        __pyx_v_i = (__pyx_v_i - 1);
      }
    }
    __pyx_L6:;

    /* "pypivoter.pyx":1229
 *                 # cliqueCounts[k] += <int> nCr[drop][i]
 *                 i -= 1
 *         return             # <<<<<<<<<<<<<<
 * 
 *     cdef int* candidatesToIterateThrough
 */
    goto __pyx_L0;

    /* "pypivoter.pyx":1188
 *     cdef int i, j, k, l, m
 *     cdef int vertexNum, nCliquesToAdd, nAdded, nCr_num, nCr_denom
 *     if beginP >= beginR or rsize - drop > max_k:             # <<<<<<<<<<<<<<
 *         if enumerate == 1: # enumerate cliques
 *             nCliquesToAdd = ipow(2, drop)
 */
  }

  /* "pypivoter.pyx":1232
 * 
 *     cdef int* candidatesToIterateThrough
 *     cdef int numCandidatesToIterateThrough = 0             # <<<<<<<<<<<<<<
 * 
 *     # get the candidates to add to R to make a maxmial clique
 */
  __pyx_v_numCandidatesToIterateThrough = 0;

  /* "pypivoter.pyx":1235
 * 
 *     # get the candidates to add to R to make a maxmial clique
 *     cdef int pivot = findBestPivotNonNeighbors(&candidatesToIterateThrough,             # <<<<<<<<<<<<<<
 *                                                &numCandidatesToIterateThrough,
 *                                                vertexSets, vertexLookup,
 */
  __pyx_v_pivot = __pyx_f_9pypivoter_findBestPivotNonNeighbors((&__pyx_v_candidatesToIterateThrough), (&__pyx_v_numCandidatesToIterateThrough), __pyx_v_vertexSets, __pyx_v_vertexLookup, __pyx_v_neighborsInP, __pyx_v_numNeighbors, __pyx_v_beginP, __pyx_v_beginR);

  /* "pypivoter.pyx":1244
 *     # for maximal cliques
 *     cdef int iterator, vertex, vertexLocation, newBeginP, newBeginR
 *     if numCandidatesToIterateThrough != 0:             # <<<<<<<<<<<<<<
 *         iterator = 0
 *         while iterator < numCandidatesToIterateThrough:
 */
  __pyx_t_1 = ((__pyx_v_numCandidatesToIterateThrough != 0) != 0);
  if (__pyx_t_1) {

    /* "pypivoter.pyx":1245
 *     cdef int iterator, vertex, vertexLocation, newBeginP, newBeginR
 *     if numCandidatesToIterateThrough != 0:
 *         iterator = 0             # <<<<<<<<<<<<<<
 *         while iterator < numCandidatesToIterateThrough:
 *             # vertex to be added to the partial clique
 */
    __pyx_v_iterator = 0;

    /* "pypivoter.pyx":1246
 *     if numCandidatesToIterateThrough != 0:
 *         iterator = 0
 *         while iterator < numCandidatesToIterateThrough:             # <<<<<<<<<<<<<<
 *             # vertex to be added to the partial clique
 *             vertex = candidatesToIterateThrough[iterator]
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_iterator < __pyx_v_numCandidatesToIterateThrough) != 0);
      if (!__pyx_t_1) break;

      /* "pypivoter.pyx":1248
 *         while iterator < numCandidatesToIterateThrough:
 *             # vertex to be added to the partial clique
 *             vertex = candidatesToIterateThrough[iterator]             # <<<<<<<<<<<<<<
 *             # to add vertex into the partial clique, represented by R,
 *             # swap vertex into R and update all data structures
 */
      __pyx_v_vertex = (__pyx_v_candidatesToIterateThrough[__pyx_v_iterator]);

      /* "pypivoter.pyx":1251
 *             # to add vertex into the partial clique, represented by R,
 *             # swap vertex into R and update all data structures
 *             moveToR(vertex, vertexSets, vertexLookup, neighborsInP,             # <<<<<<<<<<<<<<
 *                     numNeighbors, &beginP, &beginR, &newBeginP, &newBeginR)
 * 
 */
      __pyx_f_9pypivoter_moveToR(__pyx_v_vertex, __pyx_v_vertexSets, __pyx_v_vertexLookup, __pyx_v_neighborsInP, __pyx_v_numNeighbors, (&__pyx_v_beginP), (&__pyx_v_beginR), (&__pyx_v_newBeginP), (&__pyx_v_newBeginR));

      /* "pypivoter.pyx":1255
 * 
 *             # recursively compute maximal cliques with new sets R, P, and X
 *             if vertex == pivot:             # <<<<<<<<<<<<<<
 *                 isPivot[vertex] = 1 # keep track of pivot vertices in R
 * 
 */
      __pyx_t_1 = ((__pyx_v_vertex == __pyx_v_pivot) != 0);
      if (__pyx_t_1) {

        /* "pypivoter.pyx":1256
 *             # recursively compute maximal cliques with new sets R, P, and X
 *             if vertex == pivot:
 *                 isPivot[vertex] = 1 # keep track of pivot vertices in R             # <<<<<<<<<<<<<<
 * 
 *                 listAllCliquesDegeneracyRecursive(cliques, cliqueCounts,
 */
        (__pyx_v_isPivot[__pyx_v_vertex]) = 1;

        /* "pypivoter.pyx":1258
 *                 isPivot[vertex] = 1 # keep track of pivot vertices in R
 * 
 *                 listAllCliquesDegeneracyRecursive(cliques, cliqueCounts,             # <<<<<<<<<<<<<<
 *                                                   vertexSets, vertexLookup,
 *                                                   neighborsInP, numNeighbors,
 */
        __pyx_f_9pypivoter_listAllCliquesDegeneracyRecursive(__pyx_v_cliques, __pyx_v_cliqueCounts, __pyx_v_vertexSets, __pyx_v_vertexLookup, __pyx_v_neighborsInP, __pyx_v_numNeighbors, __pyx_v_isHold, __pyx_v_isPivot, __pyx_v_newBeginP, __pyx_v_newBeginR, __pyx_v_n, __pyx_v_max_k, (__pyx_v_rsize + 1), (__pyx_v_drop + 1), __pyx_v_cliqueNum, __pyx_v_enumerate);

        /* "pypivoter.pyx":1265
 *                                                   drop+1, cliqueNum, enumerate)
 * 
 *                 isPivot[vertex] = 0 # reset state of isPivot for higher calls             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
        (__pyx_v_isPivot[__pyx_v_vertex]) = 0;

        /* "pypivoter.pyx":1255
 * 
 *             # recursively compute maximal cliques with new sets R, P, and X
 *             if vertex == pivot:             # <<<<<<<<<<<<<<
 *                 isPivot[vertex] = 1 # keep track of pivot vertices in R
 * 
 */
        goto __pyx_L25;
      }

      /* "pypivoter.pyx":1268
 * 
 *             else:
 *                 isHold[vertex] = 1 # keep track of hold vertices in R             # <<<<<<<<<<<<<<
 * 
 *                 listAllCliquesDegeneracyRecursive(cliques, cliqueCounts,
 */
      /*else*/ {
        (__pyx_v_isHold[__pyx_v_vertex]) = 1;

        /* "pypivoter.pyx":1270
 *                 isHold[vertex] = 1 # keep track of hold vertices in R
 * 
 *                 listAllCliquesDegeneracyRecursive(cliques, cliqueCounts,             # <<<<<<<<<<<<<<
 *                                                   vertexSets, vertexLookup,
 *                                                   neighborsInP, numNeighbors,
 */
        __pyx_f_9pypivoter_listAllCliquesDegeneracyRecursive(__pyx_v_cliques, __pyx_v_cliqueCounts, __pyx_v_vertexSets, __pyx_v_vertexLookup, __pyx_v_neighborsInP, __pyx_v_numNeighbors, __pyx_v_isHold, __pyx_v_isPivot, __pyx_v_newBeginP, __pyx_v_newBeginR, __pyx_v_n, __pyx_v_max_k, (__pyx_v_rsize + 1), __pyx_v_drop, __pyx_v_cliqueNum, __pyx_v_enumerate);

        /* "pypivoter.pyx":1277
 *                                                   drop, cliqueNum, enumerate)
 * 
 *                 isHold[vertex] = 0 # reset state of isHold for higher calls             # <<<<<<<<<<<<<<
 * 
 *             moveFromRToX(vertex, vertexSets, vertexLookup, &beginP, &beginR)
 */
        (__pyx_v_isHold[__pyx_v_vertex]) = 0;
      }
      __pyx_L25:;

      /* "pypivoter.pyx":1279
 *                 isHold[vertex] = 0 # reset state of isHold for higher calls
 * 
 *             moveFromRToX(vertex, vertexSets, vertexLookup, &beginP, &beginR)             # <<<<<<<<<<<<<<
 * 
 *             iterator += 1
 */
      __pyx_f_9pypivoter_moveFromRToX(__pyx_v_vertex, __pyx_v_vertexSets, __pyx_v_vertexLookup, (&__pyx_v_beginP), (&__pyx_v_beginR));

      /* "pypivoter.pyx":1281
 *             moveFromRToX(vertex, vertexSets, vertexLookup, &beginP, &beginR)
 * 
 *             iterator += 1             # <<<<<<<<<<<<<<
 * 
 *         # swap vertices back from X to P, for higher recursive calls
 */
      __pyx_v_iterator = (__pyx_v_iterator + 1);
    }

    /* "pypivoter.pyx":1284
 * 
 *         # swap vertices back from X to P, for higher recursive calls
 *         iterator = 0             # <<<<<<<<<<<<<<
 *         while iterator < numCandidatesToIterateThrough:
 *             vertex = candidatesToIterateThrough[iterator]
 */
    __pyx_v_iterator = 0;

    /* "pypivoter.pyx":1285
 *         # swap vertices back from X to P, for higher recursive calls
 *         iterator = 0
 *         while iterator < numCandidatesToIterateThrough:             # <<<<<<<<<<<<<<
 *             vertex = candidatesToIterateThrough[iterator]
 *             vertexLocation = vertexLookup[vertex]
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_iterator < __pyx_v_numCandidatesToIterateThrough) != 0);
      if (!__pyx_t_1) break;

      /* "pypivoter.pyx":1286
 *         iterator = 0
 *         while iterator < numCandidatesToIterateThrough:
 *             vertex = candidatesToIterateThrough[iterator]             # <<<<<<<<<<<<<<
 *             vertexLocation = vertexLookup[vertex]
 * 
 */
      __pyx_v_vertex = (__pyx_v_candidatesToIterateThrough[__pyx_v_iterator]);

      /* "pypivoter.pyx":1287
 *         while iterator < numCandidatesToIterateThrough:
 *             vertex = candidatesToIterateThrough[iterator]
 *             vertexLocation = vertexLookup[vertex]             # <<<<<<<<<<<<<<
 * 
 *             beginP -= 1
 */
      __pyx_v_vertexLocation = (__pyx_v_vertexLookup[__pyx_v_vertex]);

      /* "pypivoter.pyx":1289
 *             vertexLocation = vertexLookup[vertex]
 * 
 *             beginP -= 1             # <<<<<<<<<<<<<<
 *             vertexSets[vertexLocation] = vertexSets[beginP]
 *             vertexSets[beginP] = vertex
 */
      __pyx_v_beginP = (__pyx_v_beginP - 1);

      /* "pypivoter.pyx":1290
 * 
 *             beginP -= 1
 *             vertexSets[vertexLocation] = vertexSets[beginP]             # <<<<<<<<<<<<<<
 *             vertexSets[beginP] = vertex
 *             vertexLookup[vertex] = beginP
 */
      (__pyx_v_vertexSets[__pyx_v_vertexLocation]) = (__pyx_v_vertexSets[__pyx_v_beginP]);

      /* "pypivoter.pyx":1291
 *             beginP -= 1
 *             vertexSets[vertexLocation] = vertexSets[beginP]
 *             vertexSets[beginP] = vertex             # <<<<<<<<<<<<<<
 *             vertexLookup[vertex] = beginP
 *             vertexLookup[vertexSets[vertexLocation]] = vertexLocation
 */
      (__pyx_v_vertexSets[__pyx_v_beginP]) = __pyx_v_vertex;

      /* "pypivoter.pyx":1292
 *             vertexSets[vertexLocation] = vertexSets[beginP]
 *             vertexSets[beginP] = vertex
 *             vertexLookup[vertex] = beginP             # <<<<<<<<<<<<<<
 *             vertexLookup[vertexSets[vertexLocation]] = vertexLocation
 * 
 */
      (__pyx_v_vertexLookup[__pyx_v_vertex]) = __pyx_v_beginP;

      /* "pypivoter.pyx":1293
 *             vertexSets[beginP] = vertex
 *             vertexLookup[vertex] = beginP
 *             vertexLookup[vertexSets[vertexLocation]] = vertexLocation             # <<<<<<<<<<<<<<
 * 
 *             iterator += 1
 */
      (__pyx_v_vertexLookup[(__pyx_v_vertexSets[__pyx_v_vertexLocation])]) = __pyx_v_vertexLocation;

      /* "pypivoter.pyx":1295
 *             vertexLookup[vertexSets[vertexLocation]] = vertexLocation
 * 
 *             iterator += 1             # <<<<<<<<<<<<<<
 * 
 *     # don't need to check for emptiness before freeing, since something will
 */
      __pyx_v_iterator = (__pyx_v_iterator + 1);
    }

    /* "pypivoter.pyx":1244
 *     # for maximal cliques
 *     cdef int iterator, vertex, vertexLocation, newBeginP, newBeginR
 *     if numCandidatesToIterateThrough != 0:             # <<<<<<<<<<<<<<
 *         iterator = 0
 *         while iterator < numCandidatesToIterateThrough:
 */
  }

  /* "pypivoter.pyx":1299
 *     # don't need to check for emptiness before freeing, since something will
 *     # always be there (we allocated enough memory for all of P, a nonempty set)
 *     free(candidatesToIterateThrough)             # <<<<<<<<<<<<<<
 * 
 *     return
 */
  free(__pyx_v_candidatesToIterateThrough);

  /* "pypivoter.pyx":1301
 *     free(candidatesToIterateThrough)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  goto __pyx_L0;

  /* "pypivoter.pyx":1129
 * 
 * 
 * cdef void listAllCliquesDegeneracyRecursive(int** cliques, int* cliqueCounts,             # <<<<<<<<<<<<<<
 *                                             int* vertexSets, int* vertexLookup,
 *                                             int** neighborsInP,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pypivoter.listAllCliquesDegeneracyRecursive", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":1304
 * 
 * 
 * cdef void listAllCliquesDegeneracy(int** cliques, int* cliqueCounts,             # <<<<<<<<<<<<<<
 *                                    NeighborListArray** orderingArray,
 *                                    int n, int max_k, int enumerate):
 */

static void __pyx_f_9pypivoter_listAllCliquesDegeneracy(int **__pyx_v_cliques, int *__pyx_v_cliqueCounts, __pyx_t_9pypivoter_NeighborListArray **__pyx_v_orderingArray, int __pyx_v_n, int __pyx_v_max_k, int __pyx_v_enumerate) {
  int *__pyx_v_vertexSets;
  int *__pyx_v_vertexLookup;
  int **__pyx_v_neighborsInP;
  int *__pyx_v_numNeighbors;
  int *__pyx_v_isHold;
  int *__pyx_v_isPivot;
  int __pyx_v_i;
  int __pyx_v_vertex;
  int __pyx_v_beginP;
  int __pyx_v_beginR;
  int __pyx_v_drop;
  int __pyx_v_rsize;
  int *__pyx_v_cliqueNum;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("listAllCliquesDegeneracy", 0);

  /* "pypivoter.pyx":1331
 *     # Vertex sets are stored in an array as follows: |--X--|--P--|--R--|
 *     # Vertices are moved to R during the recursion thru the succinct clique tree
 *     cdef int* vertexSets = <int*> calloc(n, sizeof(int))             # <<<<<<<<<<<<<<
 *     # vertex i is stored in vertexSets[vertexLookup[i]]
 *     cdef int* vertexLookup = <int*> calloc(n, sizeof(int))
 */
  __pyx_v_vertexSets = ((int *)calloc(__pyx_v_n, (sizeof(int))));

  /* "pypivoter.pyx":1333
 *     cdef int* vertexSets = <int*> calloc(n, sizeof(int))
 *     # vertex i is stored in vertexSets[vertexLookup[i]]
 *     cdef int* vertexLookup = <int*> calloc(n, sizeof(int))             # <<<<<<<<<<<<<<
 * 
 *     cdef int** neighborsInP = <int**> calloc(n * max_k, sizeof(int))
 */
  __pyx_v_vertexLookup = ((int *)calloc(__pyx_v_n, (sizeof(int))));

  /* "pypivoter.pyx":1335
 *     cdef int* vertexLookup = <int*> calloc(n, sizeof(int))
 * 
 *     cdef int** neighborsInP = <int**> calloc(n * max_k, sizeof(int))             # <<<<<<<<<<<<<<
 *     cdef int* numNeighbors = <int*> calloc(n, sizeof(int))
 *     cdef int* isHold = <int*> calloc(n, sizeof(int))
 */
  __pyx_v_neighborsInP = ((int **)calloc((__pyx_v_n * __pyx_v_max_k), (sizeof(int))));

  /* "pypivoter.pyx":1336
 * 
 *     cdef int** neighborsInP = <int**> calloc(n * max_k, sizeof(int))
 *     cdef int* numNeighbors = <int*> calloc(n, sizeof(int))             # <<<<<<<<<<<<<<
 *     cdef int* isHold = <int*> calloc(n, sizeof(int))
 *     cdef int* isPivot = <int*> calloc(n, sizeof(int))
 */
  __pyx_v_numNeighbors = ((int *)calloc(__pyx_v_n, (sizeof(int))));

  /* "pypivoter.pyx":1337
 *     cdef int** neighborsInP = <int**> calloc(n * max_k, sizeof(int))
 *     cdef int* numNeighbors = <int*> calloc(n, sizeof(int))
 *     cdef int* isHold = <int*> calloc(n, sizeof(int))             # <<<<<<<<<<<<<<
 *     cdef int* isPivot = <int*> calloc(n, sizeof(int))
 * 
 */
  __pyx_v_isHold = ((int *)calloc(__pyx_v_n, (sizeof(int))));

  /* "pypivoter.pyx":1338
 *     cdef int* numNeighbors = <int*> calloc(n, sizeof(int))
 *     cdef int* isHold = <int*> calloc(n, sizeof(int))
 *     cdef int* isPivot = <int*> calloc(n, sizeof(int))             # <<<<<<<<<<<<<<
 * 
 *     cdef int i = 0
 */
  __pyx_v_isPivot = ((int *)calloc(__pyx_v_n, (sizeof(int))));

  /* "pypivoter.pyx":1340
 *     cdef int* isPivot = <int*> calloc(n, sizeof(int))
 * 
 *     cdef int i = 0             # <<<<<<<<<<<<<<
 * 
 *     while i < n:
 */
  __pyx_v_i = 0;

  /* "pypivoter.pyx":1342
 *     cdef int i = 0
 * 
 *     while i < n:             # <<<<<<<<<<<<<<
 *         vertexLookup[i] = i
 *         vertexSets[i] = i
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_n) != 0);
    if (!__pyx_t_1) break;

    /* "pypivoter.pyx":1343
 * 
 *     while i < n:
 *         vertexLookup[i] = i             # <<<<<<<<<<<<<<
 *         vertexSets[i] = i
 *         neighborsInP[i] = <int*> calloc(max_k, sizeof(int))
 */
    (__pyx_v_vertexLookup[__pyx_v_i]) = __pyx_v_i;

    /* "pypivoter.pyx":1344
 *     while i < n:
 *         vertexLookup[i] = i
 *         vertexSets[i] = i             # <<<<<<<<<<<<<<
 *         neighborsInP[i] = <int*> calloc(max_k, sizeof(int))
 *         numNeighbors[i] = 1
 */
    (__pyx_v_vertexSets[__pyx_v_i]) = __pyx_v_i;

    /* "pypivoter.pyx":1345
 *         vertexLookup[i] = i
 *         vertexSets[i] = i
 *         neighborsInP[i] = <int*> calloc(max_k, sizeof(int))             # <<<<<<<<<<<<<<
 *         numNeighbors[i] = 1
 *         i += 1
 */
    (__pyx_v_neighborsInP[__pyx_v_i]) = ((int *)calloc(__pyx_v_max_k, (sizeof(int))));

    /* "pypivoter.pyx":1346
 *         vertexSets[i] = i
 *         neighborsInP[i] = <int*> calloc(max_k, sizeof(int))
 *         numNeighbors[i] = 1             # <<<<<<<<<<<<<<
 *         i += 1
 * 
 */
    (__pyx_v_numNeighbors[__pyx_v_i]) = 1;

    /* "pypivoter.pyx":1347
 *         neighborsInP[i] = <int*> calloc(max_k, sizeof(int))
 *         numNeighbors[i] = 1
 *         i += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef int vertex, beginP, beginR, drop, rsize
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }

  /* "pypivoter.pyx":1350
 * 
 *     cdef int vertex, beginP, beginR, drop, rsize
 *     cdef int* cliqueNum = <int*> malloc(sizeof(int))             # <<<<<<<<<<<<<<
 *     cliqueNum[0] = 0
 *     cdef int j
 */
  __pyx_v_cliqueNum = ((int *)malloc((sizeof(int))));

  /* "pypivoter.pyx":1351
 *     cdef int vertex, beginP, beginR, drop, rsize
 *     cdef int* cliqueNum = <int*> malloc(sizeof(int))
 *     cliqueNum[0] = 0             # <<<<<<<<<<<<<<
 *     cdef int j
 *     # for each vertex
 */
  (__pyx_v_cliqueNum[0]) = 0;

  /* "pypivoter.pyx":1354
 *     cdef int j
 *     # for each vertex
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         vertex = orderingArray[i].vertex
 *         # set P to be later neighbors and X to be earlier neighbors of vertex
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pypivoter.pyx":1355
 *     # for each vertex
 *     for i in range(n):
 *         vertex = orderingArray[i].vertex             # <<<<<<<<<<<<<<
 *         # set P to be later neighbors and X to be earlier neighbors of vertex
 *         fillInPandXForRecursiveCall(i, vertex, vertexSets, vertexLookup,
 */
    __pyx_t_5 = (__pyx_v_orderingArray[__pyx_v_i])->vertex;
    __pyx_v_vertex = __pyx_t_5;

    /* "pypivoter.pyx":1357
 *         vertex = orderingArray[i].vertex
 *         # set P to be later neighbors and X to be earlier neighbors of vertex
 *         fillInPandXForRecursiveCall(i, vertex, vertexSets, vertexLookup,             # <<<<<<<<<<<<<<
 *                                     orderingArray, neighborsInP, numNeighbors,
 *                                     n - 1, &beginP, &beginR)
 */
    __pyx_f_9pypivoter_fillInPandXForRecursiveCall(__pyx_v_i, __pyx_v_vertex, __pyx_v_vertexSets, __pyx_v_vertexLookup, __pyx_v_orderingArray, __pyx_v_neighborsInP, __pyx_v_numNeighbors, (__pyx_v_n - 1), (&__pyx_v_beginP), (&__pyx_v_beginR));

    /* "pypivoter.pyx":1363
 *         # recursively compute maximal cliques containing a vertex, some of its
 *         # later neighbors, and none of its earlier neighbors
 *         drop = 0             # <<<<<<<<<<<<<<
 *         rsize = 1
 *         isHold[vertex] = 1 # label the first vertex in the branch as a hold
 */
    __pyx_v_drop = 0;

    /* "pypivoter.pyx":1364
 *         # later neighbors, and none of its earlier neighbors
 *         drop = 0
 *         rsize = 1             # <<<<<<<<<<<<<<
 *         isHold[vertex] = 1 # label the first vertex in the branch as a hold
 * 
 */
    __pyx_v_rsize = 1;

    /* "pypivoter.pyx":1365
 *         drop = 0
 *         rsize = 1
 *         isHold[vertex] = 1 # label the first vertex in the branch as a hold             # <<<<<<<<<<<<<<
 * 
 *         listAllCliquesDegeneracyRecursive(cliques, cliqueCounts,
 */
    (__pyx_v_isHold[__pyx_v_vertex]) = 1;

    /* "pypivoter.pyx":1367
 *         isHold[vertex] = 1 # label the first vertex in the branch as a hold
 * 
 *         listAllCliquesDegeneracyRecursive(cliques, cliqueCounts,             # <<<<<<<<<<<<<<
 *                                           vertexSets, vertexLookup,
 *                                           neighborsInP, numNeighbors, isHold,
 */
    __pyx_f_9pypivoter_listAllCliquesDegeneracyRecursive(__pyx_v_cliques, __pyx_v_cliqueCounts, __pyx_v_vertexSets, __pyx_v_vertexLookup, __pyx_v_neighborsInP, __pyx_v_numNeighbors, __pyx_v_isHold, __pyx_v_isPivot, __pyx_v_beginP, __pyx_v_beginR, __pyx_v_n, __pyx_v_max_k, __pyx_v_rsize, __pyx_v_drop, __pyx_v_cliqueNum, __pyx_v_enumerate);

    /* "pypivoter.pyx":1373
 *                                           rsize, drop, cliqueNum, enumerate)
 * 
 *         isHold[vertex] = 0 # reset the state of isHold for future iterations             # <<<<<<<<<<<<<<
 * 
 *     cliqueCounts[0] = 1 # account for empty set as a clique
 */
    (__pyx_v_isHold[__pyx_v_vertex]) = 0;
  }

  /* "pypivoter.pyx":1375
 *         isHold[vertex] = 0 # reset the state of isHold for future iterations
 * 
 *     cliqueCounts[0] = 1 # account for empty set as a clique             # <<<<<<<<<<<<<<
 * 
 *     return
 */
  (__pyx_v_cliqueCounts[0]) = 1;

  /* "pypivoter.pyx":1377
 *     cliqueCounts[0] = 1 # account for empty set as a clique
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  goto __pyx_L0;

  /* "pypivoter.pyx":1304
 * 
 * 
 * cdef void listAllCliquesDegeneracy(int** cliques, int* cliqueCounts,             # <<<<<<<<<<<<<<
 *                                    NeighborListArray** orderingArray,
 *                                    int n, int max_k, int enumerate):
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pypivoter.pyx":1380
 * 
 * 
 * cdef int* countCliques_c(LinkedList** adjListLinked, int n, int* max_k):             # <<<<<<<<<<<<<<
 *     """Given an adjacency list of a graph as an array of linked lists of
 *        integers, output the number of k-cliques in the graph as an int array.
 */

static int *__pyx_f_9pypivoter_countCliques_c(__pyx_t_9pypivoter_LinkedList **__pyx_v_adjListLinked, int __pyx_v_n, int *__pyx_v_max_k) {
  int __pyx_v_deg;
  int __pyx_v_m;
  __pyx_t_9pypivoter_NeighborListArray **__pyx_v_orderingArray;
  int __pyx_v_i;
  int *__pyx_v_cliqueCounts;
  int *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("countCliques_c", 0);

  /* "pypivoter.pyx":1401
 *         Integer array of k-clique counts, indexed by k <= degeneracy.
 *     """
 *     cdef int deg = 0, m = 0 # degeneracy and 2x edge number             # <<<<<<<<<<<<<<
 * 
 *     cdef NeighborListArray** orderingArray = \
 */
  __pyx_v_deg = 0;
  __pyx_v_m = 0;

  /* "pypivoter.pyx":1404
 * 
 *     cdef NeighborListArray** orderingArray = \
 *         computeDegeneracyOrderArray(adjListLinked, n)             # <<<<<<<<<<<<<<
 * 
 *     cdef int i
 */
  __pyx_v_orderingArray = __pyx_f_9pypivoter_computeDegeneracyOrderArray(__pyx_v_adjListLinked, __pyx_v_n);

  /* "pypivoter.pyx":1407
 * 
 *     cdef int i
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if deg < orderingArray[i].laterDegree:
 *             deg = orderingArray[i].laterDegree
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":1408
 *     cdef int i
 *     for i in range(n):
 *         if deg < orderingArray[i].laterDegree:             # <<<<<<<<<<<<<<
 *             deg = orderingArray[i].laterDegree
 *         m += orderingArray[i].laterDegree
 */
    __pyx_t_4 = ((__pyx_v_deg < (__pyx_v_orderingArray[__pyx_v_i])->laterDegree) != 0);
    if (__pyx_t_4) {

      /* "pypivoter.pyx":1409
 *     for i in range(n):
 *         if deg < orderingArray[i].laterDegree:
 *             deg = orderingArray[i].laterDegree             # <<<<<<<<<<<<<<
 *         m += orderingArray[i].laterDegree
 * 
 */
      __pyx_t_5 = (__pyx_v_orderingArray[__pyx_v_i])->laterDegree;
      __pyx_v_deg = __pyx_t_5;

      /* "pypivoter.pyx":1408
 *     cdef int i
 *     for i in range(n):
 *         if deg < orderingArray[i].laterDegree:             # <<<<<<<<<<<<<<
 *             deg = orderingArray[i].laterDegree
 *         m += orderingArray[i].laterDegree
 */
    }

    /* "pypivoter.pyx":1410
 *         if deg < orderingArray[i].laterDegree:
 *             deg = orderingArray[i].laterDegree
 *         m += orderingArray[i].laterDegree             # <<<<<<<<<<<<<<
 * 
 *     if max_k[0] == 0:
 */
    __pyx_v_m = (__pyx_v_m + (__pyx_v_orderingArray[__pyx_v_i])->laterDegree);
  }

  /* "pypivoter.pyx":1412
 *         m += orderingArray[i].laterDegree
 * 
 *     if max_k[0] == 0:             # <<<<<<<<<<<<<<
 *         max_k[0] = deg + 1
 * 
 */
  __pyx_t_4 = (((__pyx_v_max_k[0]) == 0) != 0);
  if (__pyx_t_4) {

    /* "pypivoter.pyx":1413
 * 
 *     if max_k[0] == 0:
 *         max_k[0] = deg + 1             # <<<<<<<<<<<<<<
 * 
 *     cdef int* cliqueCounts = <int*> calloc(max_k[0] + 1, sizeof(int))
 */
    (__pyx_v_max_k[0]) = (__pyx_v_deg + 1);

    /* "pypivoter.pyx":1412
 *         m += orderingArray[i].laterDegree
 * 
 *     if max_k[0] == 0:             # <<<<<<<<<<<<<<
 *         max_k[0] = deg + 1
 * 
 */
  }

  /* "pypivoter.pyx":1415
 *         max_k[0] = deg + 1
 * 
 *     cdef int* cliqueCounts = <int*> calloc(max_k[0] + 1, sizeof(int))             # <<<<<<<<<<<<<<
 *     listAllCliquesDegeneracy(NULL, cliqueCounts, orderingArray, n, max_k[0], 0)
 *     # pass NULL pointer since the cliques argument will never be used
 */
  __pyx_v_cliqueCounts = ((int *)calloc(((__pyx_v_max_k[0]) + 1), (sizeof(int))));

  /* "pypivoter.pyx":1416
 * 
 *     cdef int* cliqueCounts = <int*> calloc(max_k[0] + 1, sizeof(int))
 *     listAllCliquesDegeneracy(NULL, cliqueCounts, orderingArray, n, max_k[0], 0)             # <<<<<<<<<<<<<<
 *     # pass NULL pointer since the cliques argument will never be used
 * 
 */
  __pyx_f_9pypivoter_listAllCliquesDegeneracy(NULL, __pyx_v_cliqueCounts, __pyx_v_orderingArray, __pyx_v_n, (__pyx_v_max_k[0]), 0);

  /* "pypivoter.pyx":1419
 *     # pass NULL pointer since the cliques argument will never be used
 * 
 *     free(orderingArray)             # <<<<<<<<<<<<<<
 * 
 *     return cliqueCounts
 */
  free(__pyx_v_orderingArray);

  /* "pypivoter.pyx":1421
 *     free(orderingArray)
 * 
 *     return cliqueCounts             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_cliqueCounts;
  goto __pyx_L0;

  /* "pypivoter.pyx":1380
 * 
 * 
 * cdef int* countCliques_c(LinkedList** adjListLinked, int n, int* max_k):             # <<<<<<<<<<<<<<
 *     """Given an adjacency list of a graph as an array of linked lists of
 *        integers, output the number of k-cliques in the graph as an int array.
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":1424
 * 
 * 
 * cdef int** enumerateCliques_c(LinkedList** adjListLinked, int n, int* max_k,             # <<<<<<<<<<<<<<
 *                              int* nCliques):
 *     """Given an adjacency list of a graph as an array of linked lists of
 */

static int **__pyx_f_9pypivoter_enumerateCliques_c(__pyx_t_9pypivoter_LinkedList **__pyx_v_adjListLinked, int __pyx_v_n, int *__pyx_v_max_k, int *__pyx_v_nCliques) {
  int __pyx_v_deg;
  int __pyx_v_m;
  __pyx_t_9pypivoter_NeighborListArray **__pyx_v_orderingArray;
  int __pyx_v_i;
  int *__pyx_v_cliqueCounts;
  int __pyx_v_new_max_k;
  int **__pyx_v_cliques;
  int __pyx_v_j;
  int **__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  long __pyx_t_6;
  long __pyx_t_7;
  long __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("enumerateCliques_c", 0);

  /* "pypivoter.pyx":1450
 *         for cliques of length less than max_k.
 *     """
 *     cdef int deg = 0, m = 0 # degeneracy and 2x edge number             # <<<<<<<<<<<<<<
 * 
 *     cdef NeighborListArray** orderingArray = \
 */
  __pyx_v_deg = 0;
  __pyx_v_m = 0;

  /* "pypivoter.pyx":1453
 * 
 *     cdef NeighborListArray** orderingArray = \
 *         computeDegeneracyOrderArray(adjListLinked, n)             # <<<<<<<<<<<<<<
 * 
 *     cdef int i
 */
  __pyx_v_orderingArray = __pyx_f_9pypivoter_computeDegeneracyOrderArray(__pyx_v_adjListLinked, __pyx_v_n);

  /* "pypivoter.pyx":1456
 * 
 *     cdef int i
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if deg < orderingArray[i].laterDegree:
 *             deg = orderingArray[i].laterDegree
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":1457
 *     cdef int i
 *     for i in range(n):
 *         if deg < orderingArray[i].laterDegree:             # <<<<<<<<<<<<<<
 *             deg = orderingArray[i].laterDegree
 *         m += orderingArray[i].laterDegree
 */
    __pyx_t_4 = ((__pyx_v_deg < (__pyx_v_orderingArray[__pyx_v_i])->laterDegree) != 0);
    if (__pyx_t_4) {

      /* "pypivoter.pyx":1458
 *     for i in range(n):
 *         if deg < orderingArray[i].laterDegree:
 *             deg = orderingArray[i].laterDegree             # <<<<<<<<<<<<<<
 *         m += orderingArray[i].laterDegree
 * 
 */
      __pyx_t_5 = (__pyx_v_orderingArray[__pyx_v_i])->laterDegree;
      __pyx_v_deg = __pyx_t_5;

      /* "pypivoter.pyx":1457
 *     cdef int i
 *     for i in range(n):
 *         if deg < orderingArray[i].laterDegree:             # <<<<<<<<<<<<<<
 *             deg = orderingArray[i].laterDegree
 *         m += orderingArray[i].laterDegree
 */
    }

    /* "pypivoter.pyx":1459
 *         if deg < orderingArray[i].laterDegree:
 *             deg = orderingArray[i].laterDegree
 *         m += orderingArray[i].laterDegree             # <<<<<<<<<<<<<<
 * 
 *     if max_k[0] == 0:
 */
    __pyx_v_m = (__pyx_v_m + (__pyx_v_orderingArray[__pyx_v_i])->laterDegree);
  }

  /* "pypivoter.pyx":1461
 *         m += orderingArray[i].laterDegree
 * 
 *     if max_k[0] == 0:             # <<<<<<<<<<<<<<
 *         max_k[0] = deg + 1
 * 
 */
  __pyx_t_4 = (((__pyx_v_max_k[0]) == 0) != 0);
  if (__pyx_t_4) {

    /* "pypivoter.pyx":1462
 * 
 *     if max_k[0] == 0:
 *         max_k[0] = deg + 1             # <<<<<<<<<<<<<<
 * 
 *     cdef int* cliqueCounts = <int*> calloc(max_k[0] + 1, sizeof(int))
 */
    (__pyx_v_max_k[0]) = (__pyx_v_deg + 1);

    /* "pypivoter.pyx":1461
 *         m += orderingArray[i].laterDegree
 * 
 *     if max_k[0] == 0:             # <<<<<<<<<<<<<<
 *         max_k[0] = deg + 1
 * 
 */
  }

  /* "pypivoter.pyx":1464
 *         max_k[0] = deg + 1
 * 
 *     cdef int* cliqueCounts = <int*> calloc(max_k[0] + 1, sizeof(int))             # <<<<<<<<<<<<<<
 *     listAllCliquesDegeneracy(NULL, cliqueCounts, orderingArray, n, max_k[0], 0)
 *     # pass NULL pointer since the cliques argument will never be used
 */
  __pyx_v_cliqueCounts = ((int *)calloc(((__pyx_v_max_k[0]) + 1), (sizeof(int))));

  /* "pypivoter.pyx":1465
 * 
 *     cdef int* cliqueCounts = <int*> calloc(max_k[0] + 1, sizeof(int))
 *     listAllCliquesDegeneracy(NULL, cliqueCounts, orderingArray, n, max_k[0], 0)             # <<<<<<<<<<<<<<
 *     # pass NULL pointer since the cliques argument will never be used
 * 
 */
  __pyx_f_9pypivoter_listAllCliquesDegeneracy(NULL, __pyx_v_cliqueCounts, __pyx_v_orderingArray, __pyx_v_n, (__pyx_v_max_k[0]), 0);

  /* "pypivoter.pyx":1469
 * 
 *     cdef int new_max_k
 *     for i in range(max_k[0] + 1):             # <<<<<<<<<<<<<<
 *         nCliques[0] += cliqueCounts[i]
 *         if cliqueCounts[i] > 0:
 */
  __pyx_t_6 = ((__pyx_v_max_k[0]) + 1);
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_7; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "pypivoter.pyx":1470
 *     cdef int new_max_k
 *     for i in range(max_k[0] + 1):
 *         nCliques[0] += cliqueCounts[i]             # <<<<<<<<<<<<<<
 *         if cliqueCounts[i] > 0:
 *             new_max_k = i
 */
    __pyx_t_8 = 0;
    (__pyx_v_nCliques[__pyx_t_8]) = ((__pyx_v_nCliques[__pyx_t_8]) + (__pyx_v_cliqueCounts[__pyx_v_i]));

    /* "pypivoter.pyx":1471
 *     for i in range(max_k[0] + 1):
 *         nCliques[0] += cliqueCounts[i]
 *         if cliqueCounts[i] > 0:             # <<<<<<<<<<<<<<
 *             new_max_k = i
 *     max_k[0] = new_max_k
 */
    __pyx_t_4 = (((__pyx_v_cliqueCounts[__pyx_v_i]) > 0) != 0);
    if (__pyx_t_4) {

      /* "pypivoter.pyx":1472
 *         nCliques[0] += cliqueCounts[i]
 *         if cliqueCounts[i] > 0:
 *             new_max_k = i             # <<<<<<<<<<<<<<
 *     max_k[0] = new_max_k
 * 
 */
      __pyx_v_new_max_k = __pyx_v_i;

      /* "pypivoter.pyx":1471
 *     for i in range(max_k[0] + 1):
 *         nCliques[0] += cliqueCounts[i]
 *         if cliqueCounts[i] > 0:             # <<<<<<<<<<<<<<
 *             new_max_k = i
 *     max_k[0] = new_max_k
 */
    }
  }

  /* "pypivoter.pyx":1473
 *         if cliqueCounts[i] > 0:
 *             new_max_k = i
 *     max_k[0] = new_max_k             # <<<<<<<<<<<<<<
 * 
 *     cdef int** cliques = <int**> calloc(nCliques[0] * max_k[0], sizeof(int))
 */
  (__pyx_v_max_k[0]) = __pyx_v_new_max_k;

  /* "pypivoter.pyx":1475
 *     max_k[0] = new_max_k
 * 
 *     cdef int** cliques = <int**> calloc(nCliques[0] * max_k[0], sizeof(int))             # <<<<<<<<<<<<<<
 * 
 *     cdef int j
 */
  __pyx_v_cliques = ((int **)calloc(((__pyx_v_nCliques[0]) * (__pyx_v_max_k[0])), (sizeof(int))));

  /* "pypivoter.pyx":1478
 * 
 *     cdef int j
 *     for i in range(nCliques[0]):             # <<<<<<<<<<<<<<
 *         cliques[i] = <int*> calloc(max_k[0], sizeof(int))
 *         for j in range(max_k[0]):
 */
  __pyx_t_1 = (__pyx_v_nCliques[0]);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pypivoter.pyx":1479
 *     cdef int j
 *     for i in range(nCliques[0]):
 *         cliques[i] = <int*> calloc(max_k[0], sizeof(int))             # <<<<<<<<<<<<<<
 *         for j in range(max_k[0]):
 *             cliques[i][j] = -1
 */
    (__pyx_v_cliques[__pyx_v_i]) = ((int *)calloc((__pyx_v_max_k[0]), (sizeof(int))));

    /* "pypivoter.pyx":1480
 *     for i in range(nCliques[0]):
 *         cliques[i] = <int*> calloc(max_k[0], sizeof(int))
 *         for j in range(max_k[0]):             # <<<<<<<<<<<<<<
 *             cliques[i][j] = -1
 * 
 */
    __pyx_t_5 = (__pyx_v_max_k[0]);
    __pyx_t_9 = __pyx_t_5;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_j = __pyx_t_10;

      /* "pypivoter.pyx":1481
 *         cliques[i] = <int*> calloc(max_k[0], sizeof(int))
 *         for j in range(max_k[0]):
 *             cliques[i][j] = -1             # <<<<<<<<<<<<<<
 * 
 *     listAllCliquesDegeneracy(cliques, cliqueCounts, orderingArray,
 */
      ((__pyx_v_cliques[__pyx_v_i])[__pyx_v_j]) = -1;
    }
  }

  /* "pypivoter.pyx":1483
 *             cliques[i][j] = -1
 * 
 *     listAllCliquesDegeneracy(cliques, cliqueCounts, orderingArray,             # <<<<<<<<<<<<<<
 *                              n, max_k[0], 1)
 * 
 */
  __pyx_f_9pypivoter_listAllCliquesDegeneracy(__pyx_v_cliques, __pyx_v_cliqueCounts, __pyx_v_orderingArray, __pyx_v_n, (__pyx_v_max_k[0]), 1);

  /* "pypivoter.pyx":1486
 *                              n, max_k[0], 1)
 * 
 *     free(orderingArray)             # <<<<<<<<<<<<<<
 *     free(cliqueCounts)
 * 
 */
  free(__pyx_v_orderingArray);

  /* "pypivoter.pyx":1487
 * 
 *     free(orderingArray)
 *     free(cliqueCounts)             # <<<<<<<<<<<<<<
 * 
 *     return cliques
 */
  free(__pyx_v_cliqueCounts);

  /* "pypivoter.pyx":1489
 *     free(cliqueCounts)
 * 
 *     return cliques             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_cliques;
  goto __pyx_L0;

  /* "pypivoter.pyx":1424
 * 
 * 
 * cdef int** enumerateCliques_c(LinkedList** adjListLinked, int n, int* max_k,             # <<<<<<<<<<<<<<
 *                              int* nCliques):
 *     """Given an adjacency list of a graph as an array of linked lists of
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":1492
 * 
 * 
 * cpdef countCliques(adj, int max_k):             # <<<<<<<<<<<<<<
 *     """Given an adjacency list of a graph as an n x 2 NumPy array of adjacent
 *        vertex pairs, output the number of k-cliques as a NumPy array.
 */

static PyObject *__pyx_pw_9pypivoter_1countCliques(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9pypivoter_countCliques(PyObject *__pyx_v_adj, int __pyx_v_max_k, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_n;
  int __pyx_v_m;
  int __pyx_v_adj_len;
  int *__pyx_v_adj_c[2];
  int __pyx_v_i;
  __pyx_t_9pypivoter_LinkedList **__pyx_v_adjacencyList;
  int *__pyx_v_cliqueCounts;
  PyObject *__pyx_v_cliqueCounts_np = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  long __pyx_t_11;
  long __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("countCliques", 0);

  /* "pypivoter.pyx":1510
 *         Numpy array of clique counts for k = 0 up to max_k.
 *     """
 *     cdef int n = np.max(adj) + 1 # number of vertices in the graph             # <<<<<<<<<<<<<<
 *     cdef int m = len(adj) # number of edges in the graph
 *     # create C-style int array from numpy array
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_max); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_adj) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_adj);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1510, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_n = __pyx_t_4;

  /* "pypivoter.pyx":1511
 *     """
 *     cdef int n = np.max(adj) + 1 # number of vertices in the graph
 *     cdef int m = len(adj) # number of edges in the graph             # <<<<<<<<<<<<<<
 *     # create C-style int array from numpy array
 *     cdef int adj_len = len(adj)
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_adj); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1511, __pyx_L1_error)
  __pyx_v_m = __pyx_t_5;

  /* "pypivoter.pyx":1513
 *     cdef int m = len(adj) # number of edges in the graph
 *     # create C-style int array from numpy array
 *     cdef int adj_len = len(adj)             # <<<<<<<<<<<<<<
 *     cdef int* adj_c[2]
 *     adj_c[0] = <int*> malloc(adj_len * sizeof(int))
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_adj); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1513, __pyx_L1_error)
  __pyx_v_adj_len = __pyx_t_5;

  /* "pypivoter.pyx":1515
 *     cdef int adj_len = len(adj)
 *     cdef int* adj_c[2]
 *     adj_c[0] = <int*> malloc(adj_len * sizeof(int))             # <<<<<<<<<<<<<<
 *     adj_c[1] = <int*> malloc(adj_len * sizeof(int))
 *     cdef int i
 */
  (__pyx_v_adj_c[0]) = ((int *)malloc((__pyx_v_adj_len * (sizeof(int)))));

  /* "pypivoter.pyx":1516
 *     cdef int* adj_c[2]
 *     adj_c[0] = <int*> malloc(adj_len * sizeof(int))
 *     adj_c[1] = <int*> malloc(adj_len * sizeof(int))             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(adj_len):
 */
  (__pyx_v_adj_c[1]) = ((int *)malloc((__pyx_v_adj_len * (sizeof(int)))));

  /* "pypivoter.pyx":1518
 *     adj_c[1] = <int*> malloc(adj_len * sizeof(int))
 *     cdef int i
 *     for i in range(adj_len):             # <<<<<<<<<<<<<<
 *         adj_c[0][i] = <int> adj[i][0]
 *         adj_c[1][i] = <int> adj[i][1]
 */
  __pyx_t_4 = __pyx_v_adj_len;
  __pyx_t_6 = __pyx_t_4;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "pypivoter.pyx":1519
 *     cdef int i
 *     for i in range(adj_len):
 *         adj_c[0][i] = <int> adj[i][0]             # <<<<<<<<<<<<<<
 *         adj_c[1][i] = <int> adj[i][1]
 * 
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_adj, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_adj_c[0])[__pyx_v_i]) = ((int)__pyx_t_8);

    /* "pypivoter.pyx":1520
 *     for i in range(adj_len):
 *         adj_c[0][i] = <int> adj[i][0]
 *         adj_c[1][i] = <int> adj[i][1]             # <<<<<<<<<<<<<<
 * 
 *     cdef LinkedList** adjacencyList = graphAdjArrayToDoubleEdges(n, m, adj_c)
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_adj, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1520, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    ((__pyx_v_adj_c[1])[__pyx_v_i]) = ((int)__pyx_t_8);
  }

  /* "pypivoter.pyx":1522
 *         adj_c[1][i] = <int> adj[i][1]
 * 
 *     cdef LinkedList** adjacencyList = graphAdjArrayToDoubleEdges(n, m, adj_c)             # <<<<<<<<<<<<<<
 *     cliqueCounts = countCliques_c(adjacencyList, n, &max_k)
 * 
 */
  __pyx_v_adjacencyList = __pyx_f_9pypivoter_graphAdjArrayToDoubleEdges(__pyx_v_n, __pyx_v_m, __pyx_v_adj_c);

  /* "pypivoter.pyx":1523
 * 
 *     cdef LinkedList** adjacencyList = graphAdjArrayToDoubleEdges(n, m, adj_c)
 *     cliqueCounts = countCliques_c(adjacencyList, n, &max_k)             # <<<<<<<<<<<<<<
 * 
 *     # create numpy array from C-style int array for returning the counts
 */
  __pyx_v_cliqueCounts = __pyx_f_9pypivoter_countCliques_c(__pyx_v_adjacencyList, __pyx_v_n, (&__pyx_v_max_k));

  /* "pypivoter.pyx":1526
 * 
 *     # create numpy array from C-style int array for returning the counts
 *     cliqueCounts_np = np.zeros(max_k + 1).astype(np.dtype("i"))             # <<<<<<<<<<<<<<
 *     for i in range(max_k + 1):
 *         cliqueCounts_np[i] = cliqueCounts[i]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_long((__pyx_v_max_k + 1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_astype); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_2, __pyx_n_u_i) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_n_u_i);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_cliqueCounts_np = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pypivoter.pyx":1527
 *     # create numpy array from C-style int array for returning the counts
 *     cliqueCounts_np = np.zeros(max_k + 1).astype(np.dtype("i"))
 *     for i in range(max_k + 1):             # <<<<<<<<<<<<<<
 *         cliqueCounts_np[i] = cliqueCounts[i]
 *     return np.trim_zeros(cliqueCounts_np)
 */
  __pyx_t_11 = (__pyx_v_max_k + 1);
  __pyx_t_12 = __pyx_t_11;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_12; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pypivoter.pyx":1528
 *     cliqueCounts_np = np.zeros(max_k + 1).astype(np.dtype("i"))
 *     for i in range(max_k + 1):
 *         cliqueCounts_np[i] = cliqueCounts[i]             # <<<<<<<<<<<<<<
 *     return np.trim_zeros(cliqueCounts_np)
 * 
 */
    __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_cliqueCounts[__pyx_v_i])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_cliqueCounts_np, __pyx_v_i, __pyx_t_3, int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "pypivoter.pyx":1529
 *     for i in range(max_k + 1):
 *         cliqueCounts_np[i] = cliqueCounts[i]
 *     return np.trim_zeros(cliqueCounts_np)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_trim_zeros); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_9, __pyx_v_cliqueCounts_np) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_cliqueCounts_np);
  __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pypivoter.pyx":1492
 * 
 * 
 * cpdef countCliques(adj, int max_k):             # <<<<<<<<<<<<<<
 *     """Given an adjacency list of a graph as an n x 2 NumPy array of adjacent
 *        vertex pairs, output the number of k-cliques as a NumPy array.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pypivoter.countCliques", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cliqueCounts_np);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pypivoter_1countCliques(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9pypivoter_countCliques[] = "countCliques(adj, int max_k)\nGiven an adjacency list of a graph as an n x 2 NumPy array of adjacent\n       vertex pairs, output the number of k-cliques as a NumPy array.\n\n    Parameters\n    ----------\n    adj : np.array [m x 2]\n        NumPy array of adjacent vertex pairs in a graph, with no repeats,\n        reversals, or self-adjacency.\n    max_k : int\n        The maximum size of k-cliques to be counted, or 0 if all k-cliques\n        should be counted.\n\n    Returns\n    -------\n    cliqueCounts : np.array [max_k + 1]\n        Numpy array of clique counts for k = 0 up to max_k.\n    ";
static PyObject *__pyx_pw_9pypivoter_1countCliques(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_adj = 0;
  int __pyx_v_max_k;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("countCliques (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_adj,&__pyx_n_s_max_k,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_adj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_k)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("countCliques", 1, 2, 2, 1); __PYX_ERR(0, 1492, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "countCliques") < 0)) __PYX_ERR(0, 1492, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_adj = values[0];
    __pyx_v_max_k = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_max_k == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1492, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("countCliques", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1492, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pypivoter.countCliques", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pypivoter_countCliques(__pyx_self, __pyx_v_adj, __pyx_v_max_k);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pypivoter_countCliques(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_adj, int __pyx_v_max_k) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("countCliques", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pypivoter_countCliques(__pyx_v_adj, __pyx_v_max_k, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pypivoter.countCliques", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pypivoter.pyx":1532
 * 
 * 
 * cpdef enumerateCliques(adj, int max_k):             # <<<<<<<<<<<<<<
 *     """Given an adjacency list of a graph as an n x 2 NumPy array of adjacent
 *        vertex pairs, output the number of k-cliques as a NumPy array.
 */

static PyObject *__pyx_pw_9pypivoter_3enumerateCliques(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9pypivoter_enumerateCliques(PyObject *__pyx_v_adj, int __pyx_v_max_k, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_n;
  int __pyx_v_m;
  int __pyx_v_adj_len;
  int *__pyx_v_adj_c[2];
  int __pyx_v_i;
  __pyx_t_9pypivoter_LinkedList **__pyx_v_adjacencyList;
  int __pyx_v_nCliques;
  int **__pyx_v_cliques;
  PyObject *__pyx_v_cliques_np = NULL;
  int __pyx_v_j;
  PyObject *__pyx_v_k_cliques = NULL;
  int __pyx_7genexpr__pyx_v_k;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enumerateCliques", 0);

  /* "pypivoter.pyx":1551
 *         value of k.
 *     """
 *     cdef int n = np.max(adj) + 1 # number of vertices in the graph             # <<<<<<<<<<<<<<
 *     cdef int m = len(adj) # number of edges in the graph
 *     # create C-style int array from numpy array
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_max); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_adj) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_adj);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1551, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_n = __pyx_t_4;

  /* "pypivoter.pyx":1552
 *     """
 *     cdef int n = np.max(adj) + 1 # number of vertices in the graph
 *     cdef int m = len(adj) # number of edges in the graph             # <<<<<<<<<<<<<<
 *     # create C-style int array from numpy array
 *     cdef int adj_len = len(adj)
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_adj); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1552, __pyx_L1_error)
  __pyx_v_m = __pyx_t_5;

  /* "pypivoter.pyx":1554
 *     cdef int m = len(adj) # number of edges in the graph
 *     # create C-style int array from numpy array
 *     cdef int adj_len = len(adj)             # <<<<<<<<<<<<<<
 *     cdef int* adj_c[2]
 *     adj_c[0] = <int*> malloc((adj_len + 1) * sizeof(int))
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_adj); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1554, __pyx_L1_error)
  __pyx_v_adj_len = __pyx_t_5;

  /* "pypivoter.pyx":1556
 *     cdef int adj_len = len(adj)
 *     cdef int* adj_c[2]
 *     adj_c[0] = <int*> malloc((adj_len + 1) * sizeof(int))             # <<<<<<<<<<<<<<
 *     adj_c[1] = <int*> malloc((adj_len + 1) * sizeof(int))
 *     cdef int i
 */
  (__pyx_v_adj_c[0]) = ((int *)malloc(((__pyx_v_adj_len + 1) * (sizeof(int)))));

  /* "pypivoter.pyx":1557
 *     cdef int* adj_c[2]
 *     adj_c[0] = <int*> malloc((adj_len + 1) * sizeof(int))
 *     adj_c[1] = <int*> malloc((adj_len + 1) * sizeof(int))             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(adj_len):
 */
  (__pyx_v_adj_c[1]) = ((int *)malloc(((__pyx_v_adj_len + 1) * (sizeof(int)))));

  /* "pypivoter.pyx":1559
 *     adj_c[1] = <int*> malloc((adj_len + 1) * sizeof(int))
 *     cdef int i
 *     for i in range(adj_len):             # <<<<<<<<<<<<<<
 *         adj_c[0][i] = <int> adj[i][0]
 *         adj_c[1][i] = <int> adj[i][1]
 */
  __pyx_t_4 = __pyx_v_adj_len;
  __pyx_t_6 = __pyx_t_4;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "pypivoter.pyx":1560
 *     cdef int i
 *     for i in range(adj_len):
 *         adj_c[0][i] = <int> adj[i][0]             # <<<<<<<<<<<<<<
 *         adj_c[1][i] = <int> adj[i][1]
 * 
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_adj, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1560, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1560, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1560, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_adj_c[0])[__pyx_v_i]) = ((int)__pyx_t_8);

    /* "pypivoter.pyx":1561
 *     for i in range(adj_len):
 *         adj_c[0][i] = <int> adj[i][0]
 *         adj_c[1][i] = <int> adj[i][1]             # <<<<<<<<<<<<<<
 * 
 *     cdef LinkedList** adjacencyList = graphAdjArrayToDoubleEdges(n, m, adj_c)
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_adj, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    ((__pyx_v_adj_c[1])[__pyx_v_i]) = ((int)__pyx_t_8);
  }

  /* "pypivoter.pyx":1563
 *         adj_c[1][i] = <int> adj[i][1]
 * 
 *     cdef LinkedList** adjacencyList = graphAdjArrayToDoubleEdges(n, m, adj_c)             # <<<<<<<<<<<<<<
 *     cdef int nCliques
 *     cliques = enumerateCliques_c(adjacencyList, n, &max_k, &nCliques)
 */
  __pyx_v_adjacencyList = __pyx_f_9pypivoter_graphAdjArrayToDoubleEdges(__pyx_v_n, __pyx_v_m, __pyx_v_adj_c);

  /* "pypivoter.pyx":1565
 *     cdef LinkedList** adjacencyList = graphAdjArrayToDoubleEdges(n, m, adj_c)
 *     cdef int nCliques
 *     cliques = enumerateCliques_c(adjacencyList, n, &max_k, &nCliques)             # <<<<<<<<<<<<<<
 * 
 *     # create numpy array from C-style int array for returning the cliques
 */
  __pyx_v_cliques = __pyx_f_9pypivoter_enumerateCliques_c(__pyx_v_adjacencyList, __pyx_v_n, (&__pyx_v_max_k), (&__pyx_v_nCliques));

  /* "pypivoter.pyx":1568
 * 
 *     # create numpy array from C-style int array for returning the cliques
 *     cliques_np = np.zeros((nCliques, max_k)).astype(np.dtype("i"))             # <<<<<<<<<<<<<<
 *     cdef int j
 *     for i in range(nCliques):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_nCliques); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_max_k); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_10);
  __pyx_t_2 = 0;
  __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_11);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_astype); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_dtype); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_n_u_i) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_n_u_i);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_cliques_np = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pypivoter.pyx":1570
 *     cliques_np = np.zeros((nCliques, max_k)).astype(np.dtype("i"))
 *     cdef int j
 *     for i in range(nCliques):             # <<<<<<<<<<<<<<
 *         for j in range(max_k):
 *             cliques_np[i][j] = cliques[i][j]
 */
  __pyx_t_4 = __pyx_v_nCliques;
  __pyx_t_6 = __pyx_t_4;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "pypivoter.pyx":1571
 *     cdef int j
 *     for i in range(nCliques):
 *         for j in range(max_k):             # <<<<<<<<<<<<<<
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 */
    __pyx_t_8 = __pyx_v_max_k;
    __pyx_t_12 = __pyx_t_8;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_j = __pyx_t_13;

      /* "pypivoter.pyx":1572
 *     for i in range(nCliques):
 *         for j in range(max_k):
 *             cliques_np[i][j] = cliques[i][j]             # <<<<<<<<<<<<<<
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(((__pyx_v_cliques[__pyx_v_i])[__pyx_v_j])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1572, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_cliques_np, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1572, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (unlikely(__Pyx_SetItemInt(__pyx_t_9, __pyx_v_j, __pyx_t_3, int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 1572, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }

  /* "pypivoter.pyx":1575
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \             # <<<<<<<<<<<<<<
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]
 *     return k_cliques
 */
  { /* enter inner scope */

    /* "pypivoter.pyx":1573
 *         for j in range(max_k):
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,             # <<<<<<<<<<<<<<
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "pypivoter.pyx":1575
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \             # <<<<<<<<<<<<<<
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]
 *     return k_cliques
 */
    __pyx_t_4 = __pyx_v_max_k;
    __pyx_t_6 = __pyx_t_4;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_7genexpr__pyx_v_k = __pyx_t_7;

      /* "pypivoter.pyx":1573
 *         for j in range(max_k):
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,             # <<<<<<<<<<<<<<
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_logical_and); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_7genexpr__pyx_v_k); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_slice_);
      __Pyx_GIVEREF(__pyx_slice_);
      PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_slice_);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_cliques_np, __pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __Pyx_PyInt_EqObjC(__pyx_t_1, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pypivoter.pyx":1574
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 *                                            cliques_np[:, k-1] != -1)][:, :k]             # <<<<<<<<<<<<<<
 *                  for k in range(max_k)] + \
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]
 */
      __pyx_t_1 = __Pyx_PyInt_From_long((__pyx_7genexpr__pyx_v_k - 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_slice_);
      __Pyx_GIVEREF(__pyx_slice_);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_slice_);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_cliques_np, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyInt_NeObjC(__pyx_t_1, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_t_11, __pyx_t_2};
        __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1573, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_t_11, __pyx_t_2};
        __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1573, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_14 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1573, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        if (__pyx_t_1) {
          __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1); __pyx_t_1 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_8, __pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_8, __pyx_t_2);
        __pyx_t_11 = 0;
        __pyx_t_2 = 0;
        __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_14, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1573, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "pypivoter.pyx":1573
 *         for j in range(max_k):
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,             # <<<<<<<<<<<<<<
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \
 */
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_v_cliques_np, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "pypivoter.pyx":1574
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 *                                            cliques_np[:, k-1] != -1)][:, :k]             # <<<<<<<<<<<<<<
 *                  for k in range(max_k)] + \
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]
 */
      __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_7genexpr__pyx_v_k); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_14 = PySlice_New(Py_None, __pyx_t_9, Py_None); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_slice_);
      __Pyx_GIVEREF(__pyx_slice_);
      PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_slice_);
      __Pyx_GIVEREF(__pyx_t_14);
      PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_14);
      __pyx_t_14 = 0;
      __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_t_10, __pyx_t_9); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_14))) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    }
  } /* exit inner scope */

  /* "pypivoter.pyx":1576
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]             # <<<<<<<<<<<<<<
 *     return k_cliques
 */
  __pyx_t_14 = __Pyx_PyInt_From_long((__pyx_v_max_k - 1)); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_slice_);
  __Pyx_GIVEREF(__pyx_slice_);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_slice_);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_14);
  __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_cliques_np, __pyx_t_9); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyInt_NeObjC(__pyx_t_14, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_cliques_np, __pyx_t_9); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyList_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_14);
  PyList_SET_ITEM(__pyx_t_9, 0, __pyx_t_14);
  __pyx_t_14 = 0;

  /* "pypivoter.pyx":1575
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \             # <<<<<<<<<<<<<<
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]
 *     return k_cliques
 */
  __pyx_t_14 = PyNumber_Add(__pyx_t_3, __pyx_t_9); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_k_cliques = ((PyObject*)__pyx_t_14);
  __pyx_t_14 = 0;

  /* "pypivoter.pyx":1577
 *                  for k in range(max_k)] + \
 *                 [cliques_np[cliques_np[:, max_k-1] != -1]]
 *     return k_cliques             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_k_cliques);
  __pyx_r = __pyx_v_k_cliques;
  goto __pyx_L0;

  /* "pypivoter.pyx":1532
 * 
 * 
 * cpdef enumerateCliques(adj, int max_k):             # <<<<<<<<<<<<<<
 *     """Given an adjacency list of a graph as an n x 2 NumPy array of adjacent
 *        vertex pairs, output the number of k-cliques as a NumPy array.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("pypivoter.enumerateCliques", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cliques_np);
  __Pyx_XDECREF(__pyx_v_k_cliques);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pypivoter_3enumerateCliques(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9pypivoter_2enumerateCliques[] = "enumerateCliques(adj, int max_k)\nGiven an adjacency list of a graph as an n x 2 NumPy array of adjacent\n       vertex pairs, output the number of k-cliques as a NumPy array.\n\n    Parameters\n    ----------\n    adj : np.array [m x 2]\n        NumPy array of adjacent vertex pairs in a graph, with no repeats,\n        reversals, or self-adjacency.\n    max_k : int\n        The maximum size of k-cliques to be counted, or 0 if all k-cliques\n        should be counted.\n\n    Returns\n    -------\n    k_cliques : list\n        A list of NumPy arrays containing the indices of the k-cliques for each\n        value of k.\n    ";
static PyObject *__pyx_pw_9pypivoter_3enumerateCliques(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_adj = 0;
  int __pyx_v_max_k;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enumerateCliques (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_adj,&__pyx_n_s_max_k,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_adj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_k)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("enumerateCliques", 1, 2, 2, 1); __PYX_ERR(0, 1532, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "enumerateCliques") < 0)) __PYX_ERR(0, 1532, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_adj = values[0];
    __pyx_v_max_k = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_max_k == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1532, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("enumerateCliques", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1532, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pypivoter.enumerateCliques", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pypivoter_2enumerateCliques(__pyx_self, __pyx_v_adj, __pyx_v_max_k);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pypivoter_2enumerateCliques(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_adj, int __pyx_v_max_k) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enumerateCliques", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pypivoter_enumerateCliques(__pyx_v_adj, __pyx_v_max_k, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pypivoter.enumerateCliques", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {"countCliques", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9pypivoter_1countCliques, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9pypivoter_countCliques},
  {"enumerateCliques", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9pypivoter_3enumerateCliques, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9pypivoter_2enumerateCliques},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_pypivoter(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_pypivoter},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "pypivoter",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_adj, __pyx_k_adj, sizeof(__pyx_k_adj), 0, 0, 1, 1},
  {&__pyx_n_s_astype, __pyx_k_astype, sizeof(__pyx_k_astype), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_u_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_logical_and, __pyx_k_logical_and, sizeof(__pyx_k_logical_and), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max, __pyx_k_max, sizeof(__pyx_k_max), 0, 0, 1, 1},
  {&__pyx_n_s_max_k, __pyx_k_max_k, sizeof(__pyx_k_max_k), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_os, __pyx_k_os, sizeof(__pyx_k_os), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_trim_zeros, __pyx_k_trim_zeros, sizeof(__pyx_k_trim_zeros), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 446, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pypivoter.pyx":1573
 *         for j in range(max_k):
 *             cliques_np[i][j] = cliques[i][j]
 *     k_cliques = [cliques_np[np.logical_and(cliques_np[:, k] == -1,             # <<<<<<<<<<<<<<
 *                                            cliques_np[:, k-1] != -1)][:, :k]
 *                  for k in range(max_k)] + \
 */
  __pyx_slice_ = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice_)) __PYX_ERR(0, 1573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice_);
  __Pyx_GIVEREF(__pyx_slice_);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initpypivoter(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initpypivoter(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_pypivoter(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_pypivoter(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_pypivoter(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'pypivoter' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_pypivoter(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("pypivoter", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pypivoter) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pypivoter")) {
      if (unlikely(PyDict_SetItemString(modules, "pypivoter", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pypivoter.pyx":1
 * import os             # <<<<<<<<<<<<<<
 * import numpy as np
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_os, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pypivoter.pyx":2
 * import os
 * import numpy as np             # <<<<<<<<<<<<<<
 * 
 * from libc.stdlib cimport calloc, malloc, free, exit
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pypivoter.pyx":1
 * import os             # <<<<<<<<<<<<<<
 * import numpy as np
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pypivoter", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pypivoter");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* None */
static CYTHON_INLINE int __Pyx_div_int(int a, int b) {
    int q = a / b;
    int r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_NeObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_FALSE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a != b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_TRUE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_TRUE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a != (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
